@@ -25,6 +25,7 @@
 #include "undo.h"
 #include "cursor.h"
 #include "highlight.h"
+#include "textedit.h"
 
 static void create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
 				     DDisplay *ddisp);
@@ -71,7 +72,7 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
       handle1->connect_type != HANDLE_NONCONNECTABLE) {
     ConnectionPoint *connectionpoint;
     connectionpoint =
-      object_find_connectpoint_display(ddisp, &origpoint, obj);
+      object_find_connectpoint_display(ddisp, &origpoint, obj, TRUE);
     if (connectionpoint != NULL) {
       (obj->ops->move)(obj, &origpoint);
       /* Make sure to not parent to the object connected to */
@@ -103,15 +104,12 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
   }
   diagram_select(ddisp->diagram, obj);
 
-  obj->ops->selectf(obj, &clickedpoint,
-		ddisp->renderer);
-
   tool->obj = obj;
 
   /* Connect first handle if possible: */
   if ((handle1!= NULL) &&
       (handle1->connect_type != HANDLE_NONCONNECTABLE)) {
-    object_connect_display(ddisp, obj, handle1);
+    object_connect_display(ddisp, obj, handle1, TRUE);
   }
 
   object_add_updates(obj, ddisp->diagram);
@@ -145,6 +143,7 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
 			     DDisplay *ddisp)
 {
   GList *list = NULL;
+  DiaObject *obj = tool->obj;
   if (tool->moving) {
     gdk_pointer_ungrab (event->time);
 
@@ -161,7 +160,7 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
 
   if (tool->moving) {
     if (tool->handle->connect_type != HANDLE_NONCONNECTABLE) {
-      object_connect_display(ddisp, tool->obj, tool->handle);
+      object_connect_display(ddisp, tool->obj, tool->handle, TRUE);
       diagram_update_connections_selection(ddisp->diagram);
       diagram_flush(ddisp->diagram);
     }
@@ -171,6 +170,7 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
   }
   
   highlight_reset_all(ddisp->diagram);
+  textedit_activate_object(ddisp, obj, NULL);
   diagram_update_extents(ddisp->diagram);
   diagram_modified(ddisp->diagram);
 
@@ -185,8 +185,11 @@ create_object_motion(CreateObjectTool *tool, GdkEventMotion *event,
 		   DDisplay *ddisp)
 {
   Point to;
-  ConnectionPoint *connectionpoint;
-  
+  ConnectionPoint *connectionpoint = NULL;
+  gchar *postext;
+  GtkStatusbar *statusbar;
+  guint context_id;
+
   if (!tool->moving)
     return;
   
@@ -196,24 +199,44 @@ create_object_motion(CreateObjectTool *tool, GdkEventMotion *event,
   parent_handle_move_out_check(tool->obj, &to);
 
   /* Move to ConnectionPoint if near: */
-  if ((tool->handle != NULL &&
-       tool->handle->connect_type != HANDLE_NONCONNECTABLE) &&
-      ((connectionpoint =
-	object_find_connectpoint_display(ddisp, &to, tool->obj)) != NULL)) {
-    to = connectionpoint->pos;
-    highlight_object(connectionpoint->object, NULL, ddisp->diagram);
-    ddisplay_set_all_cursor(get_cursor(CURSOR_CONNECT));
-  } else {
+  if (tool->handle != NULL &&
+      tool->handle->connect_type != HANDLE_NONCONNECTABLE) {
+    connectionpoint =
+      object_find_connectpoint_display(ddisp, &to, tool->obj, TRUE);
+    
+    if (connectionpoint != NULL) {
+      to = connectionpoint->pos;
+      highlight_object(connectionpoint->object, NULL, ddisp->diagram);
+      ddisplay_set_all_cursor(get_cursor(CURSOR_CONNECT));
+    }
+  }
+  
+  if (connectionpoint == NULL) {
     /* No connectionopoint near, then snap to grid (if enabled) */
     snap_to_grid(ddisp, &to.x, &to.y);
     highlight_reset_all(ddisp->diagram);
     ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
   }
-
+      
   object_add_updates(tool->obj, ddisp->diagram);
   tool->obj->ops->move_handle(tool->obj, tool->handle, &to, connectionpoint,
 			      HANDLE_MOVE_CREATE, 0);
   object_add_updates(tool->obj, ddisp->diagram);
+
+  /* Put current mouse position in status bar */
+  statusbar = GTK_STATUSBAR (ddisp->modified_status);
+  context_id = gtk_statusbar_get_context_id (statusbar, "ObjectPos");
+    
+  postext = g_strdup_printf("%.3f, %.3f - %.3f, %.3f",
+			    tool->obj->bounding_box.left,
+			    tool->obj->bounding_box.top,
+			    tool->obj->bounding_box.right,
+			    tool->obj->bounding_box.bottom);
+			       
+  gtk_statusbar_pop (statusbar, context_id); 
+  gtk_statusbar_push (statusbar, context_id, postext);
+
+  g_free(postext);
   
   diagram_flush(ddisp->diagram);
 
