@@ -30,8 +30,8 @@
 static GtkWidget *dialog = NULL;
 static GtkWidget *dialog_vbox = NULL;
 static GtkWidget *object_part = NULL;
-static ObjectType *current_objtype = NULL;
-static Object *current_object = NULL;
+static DiaObjectType *current_objtype = NULL;
+static DiaObject *current_object = NULL;
 
 static GtkWidget *no_defaults_dialog = NULL;
 
@@ -95,10 +95,10 @@ defaults_respond(GtkWidget *widget, gint response_id, gpointer data)
 }
 
 void
-defaults_show(ObjectType *objtype)
+defaults_show(DiaObjectType *objtype)
 {
   GtkWidget *defaults;
-  Object *def_object = NULL;
+  DiaObject *def_object = NULL;
   gchar *title = NULL;
 
   if (objtype != NULL) {
