@@ -80,9 +80,12 @@ set_font(DiaRenderer *self, DiaFont *font, real height)
   DiaPsFt2Renderer *renderer = DIA_PS_FT2_RENDERER(self);
 
   renderer->current_font = font;
-  /* Dammit!  We have a random factor once again! */
+  /* Dammit!  We have a random factor once again! 
+   * And not only here but also in dia_font_scaled_build_layout() below ...
+   */
   renderer->current_height = height*4.3;
-  pango_context_set_font_description(dia_font_get_context(), font->pfd);
+  pango_context_set_font_description(dia_font_get_context(), 
+                                     dia_font_get_description(font));
 }
 
 /* ********************************************************* */
@@ -99,8 +102,8 @@ static int paps_move_to( FT_Vector* to,
 {
   OutlineInfo *outline_info = (OutlineInfo*)user_data;
   fprintf(outline_info->OUT, "%d %d moveto\n",
-	  to->x ,
-	  to->y );
+	  (int)to->x ,
+	  (int)to->y );
   return 0;
 }
 
@@ -109,8 +112,8 @@ static int paps_line_to( FT_Vector*  to,
 {
   OutlineInfo *outline_info = (OutlineInfo*)user_data;
   fprintf(outline_info->OUT, "%d %d lineto\n",
-	  to->x ,
-	  to->y );
+	  (int)to->x ,
+	  (int)to->y );
   return 0;
 }
 
@@ -120,10 +123,10 @@ static int paps_conic_to( FT_Vector*  control,
 {
   OutlineInfo *outline_info = (OutlineInfo*)user_data;
   fprintf(outline_info->OUT, "%d %d %d %d conicto\n",
-	  control->x  ,
-	  control->y  ,
-	  to->x   ,
-	  to->y  );
+	  (int)control->x  ,
+	  (int)control->y  ,
+	  (int)to->x   ,
+	  (int)to->y  );
   return 0;
 }
 
@@ -135,12 +138,12 @@ static int paps_cubic_to( FT_Vector*  control1,
   OutlineInfo *outline_info = (OutlineInfo*)user_data;
   fprintf(outline_info->OUT,
 	  "%d %d %d %d %d %d curveto\n",
-	  control1->x , 
-	  control1->y ,
-	  control2->x ,
-	  control2->y ,
-	  to->x ,
-	  to->y );
+	  (int)control1->x , 
+	  (int)control1->y ,
+	  (int)control2->x ,
+	  (int)control2->y ,
+	  (int)to->x ,
+	  (int)to->y );
   return 0;
 }
 
@@ -184,16 +187,11 @@ void postscript_draw_contour(DiaPsRenderer *renderer,
 {
   GSList *runs_list;
   int num_runs = 0;
-  PangoRectangle ink_rect, logical_rect;
-  int byte_width;
-  FT_Bitmap bitmap;
-  guchar *buf;
 
   /* First calculate number of runs in text */
   runs_list = pango_line->runs;
   while(runs_list)
   {
-    PangoLayoutRun *run = runs_list->data;
     num_runs++;
     runs_list = runs_list->next;
   }
@@ -206,10 +204,6 @@ void postscript_draw_contour(DiaPsRenderer *renderer,
     PangoGlyphString *glyphs = run->glyphs;
     PangoAnalysis *analysis = &item->analysis;
     PangoFont *font = analysis->font;
-    /*
-      PangoFont *font = pango_context_load_font(renderer->context,
-      renderer->current_font->pfd);
-    */
     FT_Face ft_face;
     int bidi_level;
     int num_glyphs;
@@ -278,6 +272,10 @@ void draw_bezier_outline(DiaPsRenderer *renderer,
   FT_Int load_flags = FT_LOAD_DEFAULT|FT_LOAD_NO_BITMAP;
   FT_Glyph glyph;
   FT_Error error;
+  gchar px_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar py_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar d1_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar d2_buf[G_ASCII_DTOSTR_BUF_SIZE];
 
   /* Need to transform */
 
@@ -297,17 +295,25 @@ void draw_bezier_outline(DiaPsRenderer *renderer,
   outline_info.OUT = renderer->file;
 
   fprintf(renderer->file, 
-	  "gsave %f %f translate %f %f scale\n",
-	  pos_x, pos_y, 2.54/72.0, -2.54/72.0);
+	  "gsave %s %s translate %s %s scale\n",
+	  g_ascii_formatd(px_buf, sizeof(px_buf), "%f", pos_x),
+	  g_ascii_formatd(py_buf, sizeof(py_buf), "%f", pos_y),
+	  g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", 2.54/72.0),
+	  g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", -2.54/72.0) );
   fprintf(renderer->file, "start_ol\n");
 
   if ((error=FT_Load_Glyph(face, glyph_index, load_flags))) {
     fprintf(stderr, "Can't load glyph: %d\n", error);
     return;
   }
-  FT_Get_Glyph (face->glyph, &glyph);
-  FT_Outline_Decompose (&(((FT_OutlineGlyph)glyph)->outline),
-                        &outlinefunc, &outline_info);
+  if ((error=FT_Get_Glyph (face->glyph, &glyph))) {
+    fprintf(stderr, "Can't get glyph: %d\n", error);
+    FT_Done_Glyph (glyph);
+    return;
+  }
+  if (face->glyph->format == FT_GLYPH_FORMAT_OUTLINE)
+    FT_Outline_Decompose (&(((FT_OutlineGlyph)glyph)->outline),
+                          &outlinefunc, &outline_info);
   fprintf(renderer->file, "end_ol grestore \n");
   
   FT_Done_Glyph (glyph);
@@ -319,25 +325,34 @@ draw_string(DiaRenderer *self,
 	    Point *pos, Alignment alignment,
 	    Color *color)
 {
-  //DiaPsRenderer *renderer = DIA_PS_RENDERER(self);
   DiaPsFt2Renderer *renderer = DIA_PS_FT2_RENDERER(self);
   PangoLayout *layout;
   int width;
   int line, linecount;
   double xpos = pos->x, ypos = pos->y;
+/* Using the global PangoContext does not allow to have renderer specific 
+ * different ones. Or it implies the push/pop _context mess. Anyway just 
+ * get rid of warnings for now. But the local code may be resurreted 
+ * sooner or later...                                               --hb
+ */
+#define USE_GLOBAL_CONTEXT
+#ifndef USE_GLOBAL_CONTEXT
   PangoAttrList* list;
   PangoAttribute* attr;
   guint length;
+#endif
 
   if ((!text)||(text == (const char *)(1))) return;
 
   lazy_setcolor(DIA_PS_RENDERER(renderer),color);
 
   /* Make sure the letters aren't too wide. */
+#ifdef USE_GLOBAL_CONTEXT
   layout = dia_font_scaled_build_layout(text, renderer->current_font,
 					renderer->current_height/0.7, 
 					20.0);
-  /*
+#else
+  /* approximately what would be required but w/o dia_font_get_context() */
   dia_font_set_height(renderer->current_font, renderer->current_height);
   layout = pango_layout_new(dia_font_get_context());
 
@@ -346,7 +361,7 @@ draw_string(DiaRenderer *self,
         
   list = pango_attr_list_new();
 
-  attr = pango_attr_font_desc_new(renderer->current_font->pfd);
+  attr = pango_attr_font_desc_new(dia_font_get_description(renderer->current_font));
   attr->start_index = 0;
   attr->end_index = length;
   pango_attr_list_insert(list,attr);
@@ -356,7 +371,8 @@ draw_string(DiaRenderer *self,
 
   pango_layout_set_indent(layout,0);
   pango_layout_set_justify(layout,FALSE);
-  */
+#endif
+
   switch (alignment) {
   case ALIGN_LEFT:
     pango_layout_set_alignment(layout,PANGO_ALIGN_LEFT);
@@ -438,8 +454,6 @@ dia_ps_ft2_renderer_get_type (void)
 static void
 dia_ps_ft2_renderer_finalize (GObject *object)
 {
-  DiaPsFt2Renderer *dia_ps_ft2_renderer = DIA_PS_FT2_RENDERER (object);
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
