@@ -36,6 +36,7 @@
 #include "interface.h"
 #include "focus.h"
 #include "color.h"
+#include "object.h"
 #include "handle_ops.h"
 #include "connectionpoint_ops.h"
 #include "menus.h"
@@ -49,9 +50,6 @@
 #include "render_gdk.h"
 #include "render_libart.h"
 
-#include "pixmaps/on-grid.xpm"
-#include "pixmaps/off-grid.xpm"
-
 static GHashTable *display_ht = NULL;
 static GdkCursor *current_cursor = NULL;
 
@@ -71,13 +69,14 @@ static void
 update_zoom_status(DDisplay *ddisp)
 {
   GtkWidget *zoomcombo;
-  gchar zoom_text[7];
+  gchar* zoom_text;
 
   zoomcombo = ddisp->zoom_status;
-  sprintf (zoom_text, "%.1f%%",
+  zoom_text = g_strdup_printf("%.1f%%",
 	   ddisp->zoom_factor * 100.0 / DDISPLAY_NORMAL_ZOOM);
   gtk_entry_set_text(GTK_ENTRY(gtk_object_get_user_data(GTK_OBJECT(zoomcombo))),
 		     zoom_text);
+  g_free(zoom_text); /* Copied by gtk_entry_set_text */
 }
 
 static void
@@ -103,17 +102,181 @@ update_modified_status(DDisplay *ddisp)
   }
 }
 
-DDisplay *
-new_display(Diagram *dia)
+static void
+selection_changed (Diagram* dia, int n, DDisplay* ddisp)
+{
+  GtkStatusbar *statusbar;
+  guint context_id;
+
+  statusbar = GTK_STATUSBAR (ddisp->modified_status);
+  context_id = gtk_statusbar_get_context_id (statusbar, "Selection");
+
+  if (n > 1)
+  {
+    gchar *msg;
+
+    msg = g_strdup_printf (_("Selection of %d objects"), n);
+    gtk_statusbar_pop (statusbar, context_id);
+    gtk_statusbar_push (statusbar, context_id, msg);
+    g_free (msg);
+  }
+  else if (n == 1)
+  {
+    /* find the selected objects name - and display it */
+    DiaObject *object = (DiaObject *)ddisp->diagram->data->selected->data;
+    gchar *name = object_get_displayname (object);
+    gchar *msg = g_strdup_printf (_("Selected '%s'"), name);
+
+    gtk_statusbar_pop (statusbar, context_id);
+    gtk_statusbar_push (statusbar, context_id, msg);
+
+    g_free (name);
+    g_free (msg);
+  }
+  else
+  {
+    gtk_statusbar_pop (statusbar, context_id); 
+  }
+}
+
+static void
+append_im_menu (DDisplay* ddisp, GtkMenuItem* im_menu_item)
+{
+  GtkWidget* im_menu;
+  GtkWidget* im_menu_tearoff;
+  
+  im_menu = gtk_menu_new ();
+  im_menu_tearoff = gtk_tearoff_menu_item_new ();
+  gtk_menu_shell_append (GTK_MENU_SHELL(im_menu), im_menu_tearoff);
+  gtk_im_multicontext_append_menuitems (
+      GTK_IM_MULTICONTEXT(ddisp->im_context),
+      GTK_MENU_SHELL(im_menu));
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM(im_menu_item), im_menu);
+}
+
+/** Initialize the various GTK-level thinks in a display after the internal
+ *  data has been set.
+ * @param ddisp A display with all non-GTK/GDK items set.
+ */
+static void
+initialize_display_widgets(DDisplay *ddisp)
 {
-  DDisplay *ddisp;
-  char *filename;
-  int embedded = app_is_embedded();
-  Rectangle visible;
   GtkMenuItem* im_menu_item;
   GtkWidget* im_menu;
   GtkWidget* im_menu_tearoff;
   static gboolean input_methods_done = FALSE;
+  Diagram *dia = ddisp->diagram;
+  gchar *filename;
+
+  /*  ddisp->renderer = new_gdk_renderer(ddisp);*/
+
+  ddisp->im_context = gtk_im_multicontext_new();
+  g_signal_connect (G_OBJECT (ddisp->im_context), "commit",
+                    G_CALLBACK (ddisplay_im_context_commit), ddisp);
+  ddisp->preedit_string = NULL;
+  g_signal_connect (G_OBJECT (ddisp->im_context), "preedit_changed",
+                    G_CALLBACK (ddisplay_im_context_preedit_changed),
+                    ddisp);
+  ddisp->preedit_attrs = NULL;
+  
+  filename = strrchr(dia->filename, G_DIR_SEPARATOR);
+  if (filename==NULL) {
+    filename = dia->filename;
+  } else {
+    filename++;
+  }
+  create_display_shell(ddisp, prefs.new_view.width, prefs.new_view.height,
+		       filename, prefs.new_view.use_menu_bar, !app_is_embedded());
+  
+  ddisplay_update_statusbar (ddisp);
+
+  ddisplay_set_origo(ddisp, ddisp->visible.left, ddisp->visible.top);
+  ddisplay_update_scrollbars(ddisp);
+  ddisplay_add_update_all(ddisp);
+
+  if (!display_ht)
+    display_ht = g_hash_table_new ((GHashFunc) display_hash, NULL);
+
+  if (!app_is_embedded())
+    ddisplay_set_cursor(ddisp, current_cursor);
+  
+  g_hash_table_insert (display_ht, ddisp->shell, ddisp);
+  g_hash_table_insert (display_ht, ddisp->canvas, ddisp);
+
+  if (!input_methods_done) {
+      im_menu_item = menus_get_item_from_path("<Display>/Input Methods", NULL);
+      if (im_menu_item) {
+        append_im_menu (ddisp, im_menu_item);
+	input_methods_done = TRUE;
+      }
+  }
+  /* the diagram menubar gets recreated for every diagram */ 	 
+  if (ddisp->menu_bar) {
+    im_menu_item = menus_get_item_from_path("<DisplayMBar>/Input Methods", ddisp->mbar_item_factory);
+    if (im_menu_item)
+      append_im_menu (ddisp, im_menu_item);
+  }	 
+}
+
+/** Make a copy of an existing display.  The original does not need to have
+ *  the various GTK-related fields initialized, and so can just have been read
+ *  from a savefile.
+ *  Note that size and position are not handled here yet, but taken from prefs.
+ * @param A display object with non-GTK/GDK fields initialized (same fields as
+ *  new_display initializes before calling initialize_display_widgets()).
+ * @returns A newly allocated display, inserted into the diagram list, with
+ *  same basic layout as the original.
+ */
+DDisplay *
+copy_display(DDisplay *orig_ddisp)
+{
+  DDisplay *ddisp;
+  Diagram *dia = orig_ddisp->diagram;
+  
+  ddisp = g_new0(DDisplay,1);
+
+  /* initialize the whole struct to 0 so that we are sure to catch errors.*/
+  memset (&ddisp->updatable_menu_items, 0, sizeof (UpdatableMenuItems));
+  
+  ddisp->diagram = orig_ddisp->diagram;
+  /* Every display has its own reference */
+  g_object_ref(dia);
+
+  ddisp->grid = orig_ddisp->grid;
+
+  ddisp->show_cx_pts = orig_ddisp->show_cx_pts;
+
+  ddisp->autoscroll = orig_ddisp->autoscroll;
+  ddisp->mainpoint_magnetism = orig_ddisp->mainpoint_magnetism;
+
+  ddisp->aa_renderer = orig_ddisp->aa_renderer;
+  
+  ddisp->update_areas = orig_ddisp->update_areas;
+  ddisp->display_areas = orig_ddisp->display_areas;
+  ddisp->update_id = 0;
+
+  diagram_add_ddisplay(dia, ddisp);
+  g_signal_connect (dia, "selection_changed", G_CALLBACK(selection_changed), ddisp);
+  ddisp->origo = orig_ddisp->origo;
+  ddisp->zoom_factor = orig_ddisp->zoom_factor;
+  ddisp->visible = orig_ddisp->visible;
+
+  initialize_display_widgets(ddisp);
+  return ddisp;  /*  set the user data  */
+}
+
+
+/** Create a new display for a diagram, using prefs settings.
+ * @param dia Otherwise initialize diagram to create a display for.
+ * @returns A newly created display.
+ */
+DDisplay *
+new_display(Diagram *dia)
+{
+  GtkMenuItem* im_menu_item;
+  DDisplay *ddisp;
+  Rectangle visible;
+  static gboolean input_methods_done = FALSE;
   
   ddisp = g_new0(DDisplay,1);
 
@@ -121,16 +284,20 @@ new_display(Diagram *dia)
   ddisp->mbar_item_factory = NULL;
 
   ddisp->rulers = NULL;
+  /*
   ddisp->visible_grid = NULL;
   ddisp->snap_to_grid = NULL;
   ddisp->show_cx_pts_mitem = NULL;
 #ifdef HAVE_LIBART
   ddisp->antialiased = NULL;
 #endif
+  */
   /* initialize the whole struct to 0 so that we are sure to catch errors.*/
   memset (&ddisp->updatable_menu_items, 0, sizeof (UpdatableMenuItems));
   
   ddisp->diagram = dia;
+  /* Every display has it's own reference */
+  g_object_ref(dia);
 
   ddisp->grid.visible = prefs.grid.visible;
   ddisp->grid.snap = prefs.grid.snap;
@@ -138,23 +305,16 @@ new_display(Diagram *dia)
   ddisp->show_cx_pts = prefs.show_cx_pts;
 
   ddisp->autoscroll = TRUE;
+  ddisp->mainpoint_magnetism = TRUE;
 
   ddisp->aa_renderer = 0;
-  ddisp->renderer = new_gdk_renderer(ddisp);
   
   ddisp->update_areas = NULL;
   ddisp->display_areas = NULL;
   ddisp->update_id = 0;
 
-  filename = strrchr(dia->filename, G_DIR_SEPARATOR);
-  if (filename==NULL) {
-    filename = dia->filename;
-  } else {
-    filename++;
-  }
-
   diagram_add_ddisplay(dia, ddisp);
-
+  g_signal_connect (dia, "selection_changed", G_CALLBACK(selection_changed), ddisp);
   ddisp->origo.x = 0.0;
   ddisp->origo.y = 0.0;
   ddisp->zoom_factor = prefs.new_view.zoom/100.0*DDISPLAY_NORMAL_ZOOM;
@@ -172,50 +332,7 @@ new_display(Diagram *dia)
 
   ddisp->visible = visible;
 
-  ddisp->im_context = gtk_im_multicontext_new();
-  g_signal_connect (G_OBJECT (ddisp->im_context), "commit",
-                    G_CALLBACK (ddisplay_im_context_commit), ddisp);
-  ddisp->preedit_string = NULL;
-  g_signal_connect (G_OBJECT (ddisp->im_context), "preedit_changed",
-                    G_CALLBACK (ddisplay_im_context_preedit_changed),
-                    ddisp);
-  ddisp->preedit_attrs = NULL;
-  
-  create_display_shell(ddisp, prefs.new_view.width, prefs.new_view.height,
-		       filename, prefs.new_view.use_menu_bar, !embedded);
-  
-  ddisplay_update_statusbar (ddisp);
-
-  ddisplay_set_origo(ddisp, visible.left, visible.top);
-  ddisp->visible = visible; /* force the visible area extents */
-  ddisplay_update_scrollbars(ddisp);
-  ddisplay_add_update_all(ddisp);
-
-  if (!display_ht)
-    display_ht = g_hash_table_new ((GHashFunc) display_hash, NULL);
-
-  if (!embedded)
-    ddisplay_set_cursor(ddisp, current_cursor);
-  
-  g_hash_table_insert (display_ht, ddisp->shell, ddisp);
-  g_hash_table_insert (display_ht, ddisp->canvas, ddisp);
-
-
-  if (!input_methods_done) {
-      im_menu_item = menus_get_item_from_path("<Display>/Input Methods", NULL);
-      if (im_menu_item) {          
-          im_menu = gtk_menu_new();
-          im_menu_tearoff = gtk_tearoff_menu_item_new();
-          gtk_menu_shell_append(GTK_MENU_SHELL(im_menu),im_menu_tearoff);
-          gtk_im_multicontext_append_menuitems(
-              GTK_IM_MULTICONTEXT(ddisp->im_context),
-              GTK_MENU_SHELL(im_menu));
-          
-          gtk_menu_item_set_submenu( GTK_MENU_ITEM(im_menu_item), im_menu);
-          input_methods_done = TRUE;
-      }
-  }
-
+  initialize_display_widgets(ddisp);
   return ddisp;  /*  set the user data  */
 }
 
@@ -344,6 +461,23 @@ ddisplay_add_update_all(DDisplay *ddisp)
   ddisplay_add_update(ddisp, &ddisp->visible);
 }
 
+/** Marks a rectangle for update, with a pixel border around it.
+ */
+void
+ddisplay_add_update_with_border(DDisplay *ddisp, Rectangle *rect,
+				int pixel_border)
+{
+  Rectangle r;
+  real size = ddisplay_untransform_length(ddisp, pixel_border+1);
+
+  r.left = rect->left-size;
+  r.top = rect->top-size;
+  r.right = rect->right+size;
+  r.bottom = rect->bottom+size;
+
+  ddisplay_add_update(ddisp, &r);
+}
+
 void
 ddisplay_add_update(DDisplay *ddisp, Rectangle *rect)
 {
@@ -418,7 +552,7 @@ ddisplay_add_display_area(DDisplay *ddisp,
   }
 }
 
-static gint
+static gboolean
 ddisplay_update_handler(DDisplay *ddisp)
 {
   GSList *l;
@@ -436,7 +570,7 @@ ddisplay_update_handler(DDisplay *ddisp)
     totrect = *(Rectangle *) l->data;
   
     g_return_val_if_fail (   renderer->clip_region_clear != NULL
-                          && renderer->clip_region_add_rect != NULL, 0);
+                          && renderer->clip_region_add_rect != NULL, FALSE);
 
     renderer->clip_region_clear (ddisp->renderer);
 
@@ -463,7 +597,7 @@ ddisplay_update_handler(DDisplay *ddisp)
   while(l!=NULL) {
     ir = (IRectangle *) l->data;
 
-    g_return_val_if_fail (renderer->copy_to_window, 0);
+    g_return_val_if_fail (renderer->copy_to_window, FALSE);
     renderer->copy_to_window(ddisp->renderer, 
                              ddisp->canvas->window,
                              ir->left, ir->top,
@@ -489,7 +623,7 @@ ddisplay_flush(DDisplay *ddisp)
 }
 
 static void
-ddisplay_obj_render(Object *obj, DiaRenderer *renderer,
+ddisplay_obj_render(DiaObject *obj, DiaRenderer *renderer,
 		    int active_layer,
 		    gpointer data)
 {
@@ -497,7 +631,8 @@ ddisplay_obj_render(Object *obj, DiaRenderer *renderer,
   int i;
 
   DIA_RENDERER_GET_CLASS(renderer)->draw_object(renderer, obj);
-  if (active_layer && ddisp->show_cx_pts) {
+  if (ddisp->show_cx_pts && 
+      obj->parent_layer != NULL && obj->parent_layer->connectable) {
     for (i=0;i<obj->num_connections;i++) {
       connectionpoint_draw(obj->connections[i], ddisp);
     }
@@ -508,9 +643,12 @@ void
 ddisplay_render_pixmap(DDisplay *ddisp, Rectangle *update)
 {
   GList *list;
-  Object *obj;
+  DiaObject *obj;
   int i;
   DiaInteractiveRendererInterface *renderer;
+#ifdef TRACES
+  GTimer *timer;
+#endif
   
   if (ddisp->renderer==NULL) {
     printf("ERROR! Renderer was NULL!!\n");
@@ -521,6 +659,7 @@ ddisplay_render_pixmap(DDisplay *ddisp, Rectangle *update)
 
   /* Erase background */
   g_return_if_fail (renderer->fill_pixel_rect != NULL);
+  DIA_RENDERER_GET_CLASS(ddisp->renderer)->begin_render(ddisp->renderer);
   renderer->fill_pixel_rect (ddisp->renderer,
 			     0, 0,
 		             dia_renderer_get_width_pixels (ddisp->renderer),
@@ -531,19 +670,26 @@ ddisplay_render_pixmap(DDisplay *ddisp, Rectangle *update)
   grid_draw(ddisp, update);
   pagebreak_draw(ddisp, update);
 
+#ifdef TRACES
+  timer = g_timer_new();
+#endif
   data_render(ddisp->diagram->data, ddisp->renderer, update,
 	      ddisplay_obj_render, (gpointer) ddisp);
-  
+#ifdef TRACES
+  g_print ("data_render(%g%%) took %g seconds\n", ddisp->zoom_factor * 5.0, g_timer_elapsed (timer, NULL));
+  g_timer_destroy (timer);
+#endif
   /* Draw handles for all selected objects */
   list = ddisp->diagram->data->selected;
   while (list!=NULL) {
-    obj = (Object *) list->data;
+    obj = (DiaObject *) list->data;
 
     for (i=0;i<obj->num_handles;i++) {
        handle_draw(obj->handles[i], ddisp);
      }
     list = g_list_next(list);
   }
+  DIA_RENDERER_GET_CLASS(ddisp->renderer)->end_render(ddisp->renderer);
 }
 
 void
@@ -600,8 +746,8 @@ ddisplay_set_origo(DDisplay *ddisp, coord x, coord y)
   
   visible->left = ddisp->origo.x;
   visible->top = ddisp->origo.y;
-  visible->right = ddisp->origo.x + width/ddisp->zoom_factor;
-  visible->bottom = ddisp->origo.y + height/ddisp->zoom_factor;
+  visible->right = ddisp->origo.x + ddisplay_untransform_length(ddisp, width);
+  visible->bottom = ddisp->origo.y + ddisplay_untransform_length(ddisp, height);
 
   gtk_ruler_set_range  (GTK_RULER (ddisp->hrule),
 			visible->left,
@@ -653,7 +799,7 @@ ddisplay_set_snap_to_grid(DDisplay *ddisp, gboolean snap)
   if (ddisp->menu_bar == NULL) {
     snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Snap To Grid", NULL));
   } else {
-    snap_to_grid = ddisp->snap_to_grid;
+    snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Snap To Grid", ddisp->mbar_item_factory));
   }
 
   /* Currently, this can cause double emit, but that's a small problem.
@@ -667,48 +813,39 @@ ddisplay_set_snap_to_grid(DDisplay *ddisp, gboolean snap)
 static void
 update_snap_grid_status(DDisplay *ddisp)
 {
-  GtkWidget *image = NULL;
-  GdkPixmap       *pixmap;
-  gint            xsize, ysize;
-  GtkWidget *child;
-
-  if (ddisp->grid.snap) {
-    image = GTK_WIDGET(g_object_get_data(G_OBJECT(ddisp->grid_status),
-					 "on grid image"));
-    if (image == NULL) {
-      GdkColormap *cmap = gtk_widget_get_colormap(ddisp->grid_status);
-      g_assert(cmap != NULL);
-      image = gtk_image_new_from_pixmap(gdk_pixmap_colormap_create_from_xpm_d(NULL, cmap, NULL, NULL, on_grid_xpm), NULL);
-      g_object_ref(G_OBJECT(image));
-      g_object_set_data(G_OBJECT(ddisp->grid_status), "on grid image", image);
-    }
-  } else {
-    image = GTK_WIDGET(g_object_get_data(G_OBJECT(ddisp->grid_status),
-					 "off grid image"));
-    if (image == NULL) {
-      GdkColormap *cmap = gtk_widget_get_colormap(ddisp->grid_status);
-      g_assert(cmap != NULL);
-      image = gtk_image_new_from_pixmap(gdk_pixmap_colormap_create_from_xpm_d(NULL, cmap, NULL, NULL, off_grid_xpm), NULL);
-      g_object_ref(G_OBJECT(image));
-      g_object_set_data(G_OBJECT(ddisp->grid_status), "off grid image", image);
-      
-    }
-  }
-  child = gtk_bin_get_child(GTK_BIN(ddisp->grid_status));
-  if (child != NULL)
-    gtk_container_remove(GTK_CONTAINER(ddisp->grid_status), child);
-
-  gtk_image_get_pixmap(GTK_IMAGE(image), &pixmap, NULL);
-  gdk_drawable_get_size(GDK_DRAWABLE(pixmap), &xsize, &ysize);
-
-  gtk_container_add(GTK_CONTAINER(ddisp->grid_status),
-		    image);
-  gtk_widget_set_size_request(GTK_WIDGET(ddisp->grid_status), xsize, ysize);
-  gtk_widget_show(image);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ddisp->grid_status),
 			       ddisp->grid.snap);
 }
 
+/** Set the display's mainpoint magnetism setting, updating menu and button
+ * in the process */
+void
+ddisplay_set_snap_to_objects(DDisplay *ddisp, gboolean magnetic)
+{
+  GtkCheckMenuItem *mainpoint_magnetism;
+  ddisp->mainpoint_magnetism = magnetic;
+
+  if (ddisp->menu_bar == NULL) {
+    mainpoint_magnetism = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Snap To Objects", NULL));
+  } else {
+    mainpoint_magnetism = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Snap To Objects", ddisp->mbar_item_factory));
+  }
+
+  /* Currently, this can cause double emit, but that's a small problem.
+   */
+  gtk_check_menu_item_set_active(mainpoint_magnetism,
+				 ddisp->mainpoint_magnetism);
+  ddisplay_update_statusbar(ddisp);
+}
+
+/** Update the button showing whether mainpoint magnetism is on */
+static void
+update_mainpoint_status(DDisplay *ddisp)
+{
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ddisp->mainpoint_status),
+			       ddisp->mainpoint_magnetism);
+}
+
 /** Scroll display to where point x,y (window coords) is visible */
 gboolean
 ddisplay_autoscroll(DDisplay *ddisp, int x, int y)
@@ -856,7 +993,7 @@ ddisplay_scroll_center_point(DDisplay *ddisp, Point *p)
 /** Scroll display so that obj is centered.
  * Returns TRUE if anything changed.  */
 gboolean
-ddisplay_scroll_to_object(DDisplay *ddisp, Object *obj)
+ddisplay_scroll_to_object(DDisplay *ddisp, DiaObject *obj)
 {
   Rectangle r = obj->bounding_box;
 
@@ -872,9 +1009,6 @@ ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer)
 {
   int width, height;
 
-  if (ddisp->renderer)
-    DIA_RENDERER_GET_CLASS(ddisp->renderer)->end_render(ddisp->renderer);
-
   if (ddisp->renderer)
     g_object_unref (ddisp->renderer);
 
@@ -892,7 +1026,6 @@ ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer)
   }
 
   dia_renderer_set_size(ddisp->renderer, ddisp->canvas->window, width, height);
-  DIA_RENDERER_GET_CLASS(ddisp->renderer)->begin_render(ddisp->renderer);
 }
 
 void
@@ -939,11 +1072,14 @@ ddisp_destroy(DDisplay *ddisp)
     ddisp->update_id = 0;
   }
 
+  g_signal_handlers_disconnect_by_func (ddisp->diagram, selection_changed, ddisp);
+
   g_object_unref (G_OBJECT (ddisp->im_context));
   ddisp->im_context = NULL;
 
   ddisplay_im_context_preedit_reset(ddisp, active_focus());
 
+  /* This calls ddisplay_really_destroy */
   gtk_widget_destroy (ddisp->shell);
 }
 
@@ -985,6 +1121,8 @@ ddisplay_close(DDisplay *ddisp)
   gchar *fname;
   gchar *msg;
 
+  g_return_if_fail(ddisp != NULL);
+
   dia = ddisp->diagram;
   
   if ( (dia->display_count > 1) ||
@@ -1005,15 +1143,16 @@ ddisplay_close(DDisplay *ddisp)
                                   GTK_DIALOG_MODAL,
                                   GTK_MESSAGE_QUESTION,
                                   GTK_BUTTONS_NONE, /* no standard buttons */
-				  msg,
-                                  NULL);
+				  _("Closing diagram without saving"),
+				  NULL);
+  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), msg);
   g_free (msg);
   gtk_window_set_title (GTK_WINDOW(dialog), _("Close Diagram"));
 
   button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
   gtk_dialog_add_action_widget (GTK_DIALOG(dialog), button, GTK_RESPONSE_CANCEL);
 
-  button = gtk_button_new_with_label (_("Discard Changes"));
+  button = gtk_button_new_with_mnemonic (_("_Discard Changes"));
   gtk_dialog_add_action_widget (GTK_DIALOG(dialog), button, GTK_RESPONSE_NO);
 
   /* button = gtk_button_new_with_label (_("Save and Close")); */
@@ -1032,17 +1171,15 @@ ddisplay_close(DDisplay *ddisp)
 void
 display_update_menu_state(DDisplay *ddisp)
 {
-  static gboolean initialized = 0;
-
-  static GtkCheckMenuItem *rulers;
-  static GtkCheckMenuItem *visible_grid;
-  static GtkCheckMenuItem *snap_to_grid;
-  static GtkCheckMenuItem *show_cx_pts;
+  GtkCheckMenuItem *rulers;
+  GtkCheckMenuItem *visible_grid;
+  GtkCheckMenuItem *snap_to_grid;
+  GtkCheckMenuItem *show_cx_pts;
 #ifdef HAVE_LIBART
-  static GtkCheckMenuItem *antialiased;
+  GtkCheckMenuItem *antialiased;
 #endif
 
-  if ((!initialized) && (ddisp->menu_bar == NULL)) {
+  if (ddisp->menu_bar == NULL) {
     rulers       = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Show Rulers", NULL));
     visible_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Show Grid", NULL));
     snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Snap To Grid", NULL));
@@ -1051,39 +1188,32 @@ display_update_menu_state(DDisplay *ddisp)
 #ifdef HAVE_LIBART
     antialiased = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/AntiAliased", NULL));
 #endif
-
-    initialized = TRUE;
-  }
-  ddisplay_do_update_menu_sensitivity (ddisp);
-  
-  if (ddisp->menu_bar) {
-      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->rulers),
-				     GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
-      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->visible_grid),
-				     ddisp->grid.visible);
-      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->snap_to_grid),
-				     ddisp->grid.snap);
-      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->show_cx_pts_mitem),
-				     ddisp->show_cx_pts); 
+  } else {
+    rulers       = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Show Rulers", ddisp->mbar_item_factory));
+    visible_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Show Grid", ddisp->mbar_item_factory));
+    snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Snap To Grid", ddisp->mbar_item_factory));
+    show_cx_pts  = 
+      GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Show Connection Points", ddisp->mbar_item_factory));
 #ifdef HAVE_LIBART
-      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->antialiased),
-				     ddisp->aa_renderer);
+    antialiased = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/AntiAliased", ddisp->mbar_item_factory));
 #endif
   }
-  else {
-      gtk_check_menu_item_set_active(rulers,
-				     GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
-      gtk_check_menu_item_set_active(visible_grid,
-				     ddisp->grid.visible);
-      gtk_check_menu_item_set_active(snap_to_grid,
-				     ddisp->grid.snap);
-      gtk_check_menu_item_set_active(show_cx_pts,
-				     ddisp->show_cx_pts); 
+
+
+  ddisplay_do_update_menu_sensitivity (ddisp);
+  
+  gtk_check_menu_item_set_active(rulers,
+				 GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
+  gtk_check_menu_item_set_active(visible_grid,
+				 ddisp->grid.visible);
+  gtk_check_menu_item_set_active(snap_to_grid,
+				 ddisp->grid.snap);
+  gtk_check_menu_item_set_active(show_cx_pts,
+				 ddisp->show_cx_pts); 
 #ifdef HAVE_LIBART
-      gtk_check_menu_item_set_active(antialiased,
-				     ddisp->aa_renderer);
+  gtk_check_menu_item_set_active(antialiased,
+				 ddisp->aa_renderer);
 #endif 
-  }  
 }
 
 void 
@@ -1106,14 +1236,16 @@ ddisplay_do_update_menu_sensitivity (DDisplay *ddisp)
 void
 ddisplay_really_destroy(DDisplay *ddisp)
 {
-  Diagram *dia;
-
   if (active_display == ddisp)
     display_set_active(NULL);
 
-  dia = ddisp->diagram;
   
-  diagram_remove_ddisplay(dia, ddisp);
+  if (ddisp->diagram) {
+    diagram_remove_ddisplay(ddisp->diagram, ddisp);
+    /* if we are the last user of the diagram it will be unref'ed */
+    g_object_unref(ddisp->diagram);
+    ddisp->diagram = NULL;
+  }
 
   g_object_unref (ddisp->renderer);
   ddisp->renderer = NULL;
@@ -1174,6 +1306,7 @@ ddisplay_update_statusbar(DDisplay *ddisp)
 {
   update_zoom_status (ddisp);
   update_snap_grid_status (ddisp);
+  update_mainpoint_status (ddisp);
   update_modified_status (ddisp);
 }
 
@@ -1193,7 +1326,13 @@ display_set_active(DDisplay *ddisp)
       if (prefs.toolbox_on_top) {
         gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
                                      GTK_WINDOW(ddisp->shell));
+      } else {
+        gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
+                                     NULL);
       }
+    } else {
+      gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
+                                   NULL);
     }
   }
 }
