@@ -30,11 +30,13 @@
 #include "preferences.h"
 #include "cursor.h"
 #include "highlight.h"
+#include "textedit.h"
 
 #include "diacanvas.h"
 #include "prop_text.h"
 #include "gtk/gtk.h"
 
+
 static DiaObject *click_select_object(DDisplay *ddisp, Point *clickedpoint,
 				   GdkEventButton *event);
 static int do_if_clicked_handle(DDisplay *ddisp, ModifyTool *tool,
@@ -75,6 +77,20 @@ create_modify_tool(void)
   return (Tool *)tool;
 }
 
+static ModifierKeys 
+gdk_event_to_dia_ModifierKeys(guint event_state)
+{
+        ModifierKeys mod = MODIFIER_NONE;
+        if (event_state & GDK_SHIFT_MASK)
+                mod |= MODIFIER_SHIFT;
+   /*     Used intenally do not propagate
+    *     if (event_state & GDK_CONTROL_MASK) 
+                mod | MODIFIER_CONTROL;  
+                */
+        return mod;
+}
+
+        
 void
 free_modify_tool(Tool *tool)
 {
@@ -139,8 +155,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
       }
       
       diagram_select(diagram, obj);
-      obj->ops->selectf(obj, clickedpoint,
-		        ddisp->renderer);
+      textedit_activate_object(ddisp, obj, clickedpoint);
 
       /*
 	This stuff is buggy, fix it later.
@@ -156,8 +171,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
       return obj;
     } else { /* Clicked on already selected. */
       /*printf("Already selected\n");*/
-      obj->ops->selectf(obj, clickedpoint,
-		       ddisp->renderer);
+      textedit_activate_object(ddisp, obj, clickedpoint);
       object_add_updates_list(diagram->data->selected, diagram);
       diagram_flush(diagram);
       
@@ -337,8 +351,8 @@ modify_move_already(ModifyTool *tool, DDisplay *ddisp, Point *to)
   }
   dist = distance_point_point_manhattan(&tool->start_at, to);
   if (ddisp->grid.snap) {
-    real grid_x = ddisp->diagram->data->grid.width_x;
-    real grid_y = ddisp->diagram->data->grid.width_y;
+    real grid_x = ddisp->diagram->grid.width_x;
+    real grid_y = ddisp->diagram->grid.width_y;
     if (dist > grid_x || dist > grid_y) {
       return TRUE;
     }
@@ -350,6 +364,10 @@ modify_move_already(ModifyTool *tool, DDisplay *ddisp, Point *to)
   }
 }
 
+/** Used for highlighting mainpoint connections. */
+static Color mainpoint_color = { 1.0, 0.8, 0.0 };
+/** Used for highlighting normal connections. */
+static Color cp_color = { 1.0, 0.0, 0.0 };
 
 static void
 modify_motion(ModifyTool *tool, GdkEventMotion *event,
@@ -360,6 +378,9 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
   gboolean auto_scroll, vertical = FALSE;
   ConnectionPoint *connectionpoint;
   ObjectChange *objchange;
+  gchar *postext;
+  GtkStatusbar *statusbar;
+  guint context_id;
 
   if (tool->state==STATE_NONE)
     return; /* Fast path... */
@@ -421,7 +442,24 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
       undo_object_change(ddisp->diagram, tool->object, objchange);
     }
     object_add_updates_list(ddisp->diagram->data->selected, ddisp->diagram);
-  
+
+    object_add_updates(tool->object, ddisp->diagram);
+
+    /* Put current mouse position in status bar */
+    statusbar = GTK_STATUSBAR (ddisp->modified_status);
+    context_id = gtk_statusbar_get_context_id (statusbar, "ObjectPos");
+    
+    postext = g_strdup_printf("%.3f, %.3f - %.3f, %.3f",
+			      tool->object->bounding_box.left,
+			      tool->object->bounding_box.top,
+			      tool->object->bounding_box.right,
+			      tool->object->bounding_box.bottom);
+			       
+    gtk_statusbar_pop (statusbar, context_id); 
+    gtk_statusbar_push (statusbar, context_id, postext);
+
+    g_free(postext);
+
     diagram_update_connections_selection(ddisp->diagram);
     diagram_flush(ddisp->diagram);
     break;
@@ -440,20 +478,28 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
 
     /* Move to ConnectionPoint if near: */
     connectionpoint =
-      object_find_connectpoint_display(ddisp, &to, tool->object);
+      object_find_connectpoint_display(ddisp, &to, tool->object, TRUE);
 
     if (event->state & GDK_CONTROL_MASK)
       vertical = (fabs(full_delta.x) < fabs(full_delta.y));
 
-    if ( (tool->handle->connect_type != HANDLE_NONCONNECTABLE) &&
-	 (connectionpoint != NULL) ) {
-      to = connectionpoint->pos;
-      highlight_object(connectionpoint->object, NULL, ddisp->diagram);
-      ddisplay_set_all_cursor(get_cursor(CURSOR_CONNECT));
-    } else {
+    highlight_reset_all(ddisp->diagram);
+    if ((tool->handle->connect_type != HANDLE_NONCONNECTABLE)) {
+      if (connectionpoint != NULL) {
+	Color *hi_color;
+	to = connectionpoint->pos;
+	if (connectionpoint->flags & CP_FLAGS_MAIN) {
+	  hi_color = &mainpoint_color;
+	} else {
+	  hi_color = &cp_color;
+	}
+	highlight_object(connectionpoint->object, hi_color, ddisp->diagram);
+	ddisplay_set_all_cursor(get_cursor(CURSOR_CONNECT));
+      }
+    }
+    if (connectionpoint == NULL) {
       /* No connectionopoint near, then snap to grid (if enabled) */
       snap_to_grid(ddisp, &to.x, &to.y);
-      highlight_reset_all(ddisp->diagram);
       ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
     }
 
@@ -481,10 +527,23 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
       *tool->orig_pos = tool->handle->pos;
     }
 
+    /* Put current mouse position in status bar */
+    statusbar = GTK_STATUSBAR (ddisp->modified_status);
+    context_id = gtk_statusbar_get_context_id (statusbar, "CursorPos");
+    
+    postext = g_strdup_printf("%.3f, %.3f", to.x, to.y);
+			       
+    gtk_statusbar_pop (statusbar, context_id); 
+    gtk_statusbar_push (statusbar, context_id, postext);
+
+    g_free(postext);
+
     object_add_updates(tool->object, ddisp->diagram);
+
+    /* Handle undo */
     objchange = tool->object->ops->move_handle(tool->object, tool->handle, 
 					       &to, connectionpoint,
-					       HANDLE_MOVE_USER, 0);
+					       HANDLE_MOVE_USER, gdk_event_to_dia_ModifierKeys(event->state));
     if (objchange != NULL) {
       undo_object_change(ddisp->diagram, tool->object, objchange);
     }
@@ -528,6 +587,30 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
   tool->auto_scrolled = auto_scroll;
 }
 
+/** Find the list of objects selected by current rubberbanding. 
+ * The list should be freed after use. */
+static GList *
+find_selected_objects(DDisplay *ddisp, ModifyTool *tool)
+{
+  Rectangle r;
+  r.left = MIN(tool->start_box.x, tool->end_box.x);
+  r.right = MAX(tool->start_box.x, tool->end_box.x);
+  r.top = MIN(tool->start_box.y, tool->end_box.y);
+  r.bottom = MAX(tool->start_box.y, tool->end_box.y);
+  
+  if (prefs.reverse_rubberbanding_intersects) {
+    if (tool->start_box.x > tool->end_box.x) {
+      return
+	layer_find_objects_intersecting_rectangle(ddisp->diagram->data->active_layer, &r);
+    } else {
+      return
+	layer_find_objects_in_rectangle(ddisp->diagram->data->active_layer, &r);
+    }
+  } else {
+    return
+      layer_find_objects_in_rectangle(ddisp->diagram->data->active_layer, &r);
+  }
+}
 
 static void
 modify_button_release(ModifyTool *tool, GdkEventButton *event,
@@ -536,7 +619,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
   Point *dest_pos, to;
   GList *list;
   int i;
-  DiaObject *obj;
+  DiaObject *active_obj = NULL;
   ObjectChange *objchange;
   
   tool->break_connections = FALSE;
@@ -562,7 +645,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
       dest_pos = g_new(Point, g_list_length(list));
       i=0;
       while (list != NULL) {
-	obj = (DiaObject *)  list->data;
+	DiaObject *obj = (DiaObject *)  list->data;
 	dest_pos[i] = obj->position;
 	list = g_list_next(list); i++;
       }
@@ -594,7 +677,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     object_add_updates(tool->object, ddisp->diagram);
     objchange = tool->object->ops->move_handle(tool->object, tool->handle,
 					       &tool->last_to, NULL,
-					       HANDLE_MOVE_USER_FINAL,0);
+					       HANDLE_MOVE_USER_FINAL,gdk_event_to_dia_ModifierKeys(event->state));
     if (objchange != NULL) {
       undo_object_change(ddisp->diagram, tool->object, objchange);
     }
@@ -603,7 +686,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
 
     /* Connect if possible: */
     if (tool->handle->connect_type != HANDLE_NONCONNECTABLE) {
-      object_connect_display(ddisp, tool->object, tool->handle); /* pushes UNDO info */
+      object_connect_display(ddisp, tool->object, tool->handle, TRUE); /* pushes UNDO info */
       diagram_update_connections_selection(ddisp->diagram);
     }
     
@@ -623,6 +706,8 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     break;
   case STATE_BOX_SELECT:
     gdk_pointer_ungrab (event->time);
+    /* Remember the currently active object for reactivating. */
+    active_obj = active_focus()!=NULL?focus_get_object(active_focus()):NULL;
     /* Remove last box: */
     if (!tool->auto_scrolled) {
       gdk_draw_rectangle (ddisp->canvas->window, tool->gc, FALSE,
@@ -631,27 +716,9 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     }
 
     {
-      Rectangle r;
       GList *list, *list_to_free;
-      DiaObject *obj;
 
-      r.left = MIN(tool->start_box.x, tool->end_box.x);
-      r.right = MAX(tool->start_box.x, tool->end_box.x);
-      r.top = MIN(tool->start_box.y, tool->end_box.y);
-      r.bottom = MAX(tool->start_box.y, tool->end_box.y);
-
-      if (prefs.reverse_rubberbanding_intersects) {
-	if (tool->start_box.x > tool->end_box.x) {
-	  list = list_to_free =
-	    layer_find_objects_intersecting_rectangle(ddisp->diagram->data->active_layer, &r);
-	} else {
-	  list = list_to_free =
-	    layer_find_objects_in_rectangle(ddisp->diagram->data->active_layer, &r);
-	}
-      } else {
-	list = list_to_free =
-	  layer_find_objects_in_rectangle(ddisp->diagram->data->active_layer, &r);
-      }
+      list = list_to_free = find_selected_objects(ddisp, tool);
       
       if (selection_style == SELECT_REPLACE &&
           !(event->state & GDK_SHIFT_MASK)) {
@@ -663,7 +730,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
         GList *intersection = NULL;
 
         while (list != NULL) {
-          obj = (DiaObject *)list->data;
+          DiaObject *obj = (DiaObject *)list->data;
           
           if (diagram_is_selected(ddisp->diagram, obj)) {
             intersection = g_list_append(intersection, obj);
@@ -674,7 +741,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
         list = intersection;
         diagram_remove_all_selected(ddisp->diagram, TRUE);
         while (list != NULL) {
-          obj = (DiaObject *)list->data;
+          DiaObject *obj = (DiaObject *)list->data;
 
           diagram_select(ddisp->diagram, obj);
 
@@ -683,7 +750,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
         g_list_free(intersection);
       } else {
         while (list != NULL) {
-          obj = (DiaObject *)list->data;
+          DiaObject *obj = (DiaObject *)list->data;
           
           if (selection_style == SELECT_REMOVE) {
             if (diagram_is_selected(ddisp->diagram, obj))
@@ -706,6 +773,12 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
       
     }
     
+    if (active_obj != NULL &&
+	diagram_is_selected(ddisp->diagram, active_obj)) {
+      textedit_activate_object(ddisp, active_obj, NULL);
+    } else {
+      textedit_activate_first(ddisp);
+    }
     ddisplay_do_update_menu_sensitivity(ddisp);
     ddisplay_flush(ddisp);
 
@@ -721,16 +794,17 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
 
 #define EDIT_BORDER_WIDTH 5
 
-gboolean
+static gboolean
 modify_edit_end(GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
-  int foo = printf("Ending focus\n");
   GtkTextView *view = GTK_TEXT_VIEW(widget);
   DiaObject *obj = (DiaObject*)data;
   GQuark quark = g_quark_from_string(PROP_TYPE_TEXT);
-  PropDescription *props = obj->ops->describe_props(obj);
+  const PropDescription *props = obj->ops->describe_props(obj);
   int i;
 
+  printf("Ending focus\n");
+
   for (i = 0; props[i].name != NULL; i++) {
     printf("Testing to remove: %s\n", props[i].name);
     if (props[i].type_quark == quark) {
@@ -739,7 +813,6 @@ modify_edit_end(GtkWidget *widget, GdkEventFocus *event, gpointer data)
       Property *prop = props[i].ops->new_prop(&props[i], pdtpp_true);
       GtkTextBuffer *buf;
       GtkTextIter start, end;
-      ObjectChange *change;
 
       printf("Going to stop %d\n", i);
       buf = gtk_text_view_get_buffer(view);
@@ -758,7 +831,7 @@ modify_edit_end(GtkWidget *widget, GdkEventFocus *event, gpointer data)
 }
 
 /** Start editing the first text among selected objects, if any */
-void
+static void
 modify_edit_first_text(DDisplay *ddisp)
 {
   GList *edits = ddisp->diagram->data->text_edits;
@@ -803,7 +876,7 @@ modify_start_text_edit(DDisplay *ddisp, Text *text, DiaObject *obj, Point *click
     gtk_text_buffer_create_tag(buf,
 			       NULL,
 			       "font-desc",
-			       text->font->pfd,
+			       dia_font_get_description(text->font),
 			       NULL);
   gtk_text_buffer_get_bounds(buf, &start, &end);
   gtk_text_buffer_apply_tag(buf, fonttag, &start, &end);
@@ -837,7 +910,7 @@ modify_start_text_edit(DDisplay *ddisp, Text *text, DiaObject *obj, Point *click
 void
 modify_make_text_edit(DDisplay *ddisp, DiaObject *obj, Point *clickedpoint)
 {
-  PropDescription *props = obj->ops->describe_props(obj);
+  const PropDescription *props = obj->ops->describe_props(obj);
   int i;
   for (i = 0; props[i].name != NULL; i++) {
     GQuark type = g_quark_from_string(PROP_TYPE_TEXT);
@@ -879,7 +952,7 @@ modify_make_text_edit(DDisplay *ddisp, DiaObject *obj, Point *clickedpoint)
 	gtk_text_buffer_create_tag(buf,
 				   NULL,
 				   "font-desc",
-				   textprop->attr.font->pfd,
+				   dia_font_get_description(textprop->attr.font),
 				   NULL);
       gtk_text_buffer_get_bounds(buf, &start, &end);
       gtk_text_buffer_apply_tag(buf, fonttag, &start, &end);
