@@ -24,7 +24,6 @@
 #endif
 
 #include <stdio.h>
-#include <locale.h>
 #include <string.h> /* strlen */
 #include <signal.h>
 #include <errno.h>
@@ -71,8 +70,11 @@ print_page(DiagramData *data, DiaRenderer *diarend, Rectangle *bounds)
   gfloat tmargin = data->paper.tmargin, bmargin = data->paper.bmargin;
   gfloat lmargin = data->paper.lmargin;
   gfloat scale = data->paper.scaling;
+  gchar d1_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar d2_buf[G_ASCII_DTOSTR_BUF_SIZE];
 
   rend->paper = data->paper.name;
+  rend->is_portrait = data->paper.is_portrait;
 
   /* count the number of objects in this region */
   data_render(data, diarend, bounds,
@@ -90,22 +92,38 @@ print_page(DiagramData *data, DiaRenderer *diarend, Rectangle *bounds)
 
   /* transform coordinate system */
   if (data->paper.is_portrait) {
-    fprintf(rend->file, "%f %f scale\n", 28.346457*scale, -28.346457*scale);
-    fprintf(rend->file, "%f %f translate\n", lmargin/scale - bounds->left,
-	    -bmargin/scale - bounds->bottom);
+    fprintf(rend->file, "%s %s scale\n",
+	    g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", 28.346457*scale),
+	    g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", -28.346457*scale) );
+    fprintf(rend->file, "%s %s translate\n",
+	    g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", lmargin/scale - bounds->left),
+	    g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", -bmargin/scale - bounds->bottom) );
   } else {
     fprintf(rend->file, "90 rotate\n");
-    fprintf(rend->file, "%f %f scale\n", 28.346457*scale, -28.346457*scale);
-    fprintf(rend->file, "%f %f translate\n", lmargin/scale - bounds->left,
-	    tmargin/scale - bounds->top);
+    fprintf(rend->file, "%s %s scale\n",
+	    g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", 28.346457*scale),
+	    g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", -28.346457*scale) );
+    fprintf(rend->file, "%s %s translate\n",
+	    g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", lmargin/scale - bounds->left),
+	    g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", tmargin/scale - bounds->top) );
   }
 
   /* set up clip mask */
-  fprintf(rend->file, "n %f %f m ", bounds->left, bounds->top);
-  fprintf(rend->file, "%f %f l ", bounds->right, bounds->top);
-  fprintf(rend->file, "%f %f l ", bounds->right, bounds->bottom);
-  fprintf(rend->file, "%f %f l ", bounds->left, bounds->bottom);
-  fprintf(rend->file, "%f %f l ", bounds->left, bounds->top);
+  fprintf(rend->file, "n %s %s m ",
+	  g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", bounds->left),
+	  g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", bounds->top) );
+  fprintf(rend->file, "%s %s l ",
+	  g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", bounds->right),
+	  g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", bounds->top) );
+  fprintf(rend->file, "%s %s l ",
+	  g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", bounds->right),
+	  g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", bounds->bottom) );
+  fprintf(rend->file, "%s %s l ",
+	  g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", bounds->left),
+	  g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", bounds->bottom) );
+  fprintf(rend->file, "%s %s l ",
+	  g_ascii_formatd(d1_buf, sizeof(d1_buf), "%f", bounds->left),
+	  g_ascii_formatd(d2_buf, sizeof(d2_buf), "%f", bounds->top) );
   /* Tip from Dov Grobgeld: Clip does not destroy the path, so we should
      do a newpath afterwards */
   fprintf(rend->file, "clip n\n"); 
@@ -130,9 +148,7 @@ paginate_psprint(Diagram *dia, FILE *file)
   gfloat width, height;
   gfloat x, y, initx, inity;
   guint nobjs = 0;
-  char *old_locale;
 
-  old_locale = setlocale(LC_NUMERIC, "C");
   rend = new_psprint_renderer(dia, file);
 
 #ifdef DIA_PS_RENDERER_DUAL_PASS
@@ -156,10 +172,16 @@ paginate_psprint(Diagram *dia, FILE *file)
   }
 
   /* iterate through all the pages in the diagram */
-  for (y = inity; y < extents->bottom; y += height)
+  for (y = inity; y < extents->bottom; y += height) {
+    /* ensure we are not producing pages for epsilon */
+    if ((extents->bottom - y) < 1e-6)
+      break;
     for (x = initx; x < extents->right; x += width) {
       Rectangle page_bounds;
 
+      if ((extents->right - x) < 1e-6)
+	break;
+
       page_bounds.left = x;
       page_bounds.right = x + width;
       page_bounds.top = y;
@@ -167,10 +189,10 @@ paginate_psprint(Diagram *dia, FILE *file)
 
       nobjs += print_page(dia->data,rend, &page_bounds);
     }
+  }
 
   g_object_unref(rend);
 
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -196,11 +218,11 @@ pipe_handler(int signum)
 static gboolean
 diagram_print_destroy(GtkWidget *widget)
 {
-  Diagram **dia;
+  Diagram *dia;
 
   if ((dia = gtk_object_get_user_data(GTK_OBJECT(widget))) != NULL) {
-    diagram_remove_related_dialog(*dia, widget);
-    *dia = NULL;
+    g_object_unref(dia);
+    gtk_object_set_user_data(GTK_OBJECT(widget), NULL);
   }
 
   return FALSE;
@@ -229,9 +251,10 @@ diagram_print_ps(Diagram *dia)
 
   /* create the dialog */
   dialog = gtk_dialog_new();
-  diagram_add_related_dialog(dia, dialog);
-  gtk_object_set_user_data(GTK_OBJECT(dialog), &dia);
-  g_signal_connect(GTK_OBJECT(dialog), "delete_event",
+  /* the dialog has it's own reference to the diagram */
+  g_object_ref(dia);
+  gtk_object_set_user_data(GTK_OBJECT(dialog), dia);
+  g_signal_connect(GTK_OBJECT(dialog), "destroy",
 		   G_CALLBACK(diagram_print_destroy), NULL);
   g_signal_connect(GTK_OBJECT(dialog), "delete_event",
 		   G_CALLBACK(gtk_main_quit), NULL);
@@ -300,7 +323,7 @@ diagram_print_ps(Diagram *dia)
   gtk_entry_set_text(GTK_ENTRY(cmd), win32_printer_default ());
 #else
   {
-    gchar *printer = g_getenv("PRINTER");
+    const gchar *printer = g_getenv("PRINTER");
     
     if (printer) {
       printcmd = g_strdup_printf("lpr -P%s", printer);
@@ -313,6 +336,7 @@ diagram_print_ps(Diagram *dia)
     printcmd = NULL;
   }
 #endif
+  printf("Print dialog?\n");
   persistence_register_string_entry("printer-command", cmd);
   printcmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(cmd)));
   orig_command = printcmd;
@@ -353,14 +377,14 @@ diagram_print_ps(Diagram *dia)
   } else {
     const gchar *filename = gtk_entry_get_text(GTK_ENTRY(ofile));
     if (!g_path_is_absolute(filename)) {
-      char *diagram_dir;
+      char *dirname;
       char *full_filename;
 
-      diagram_dir = g_dirname(dia->filename);
-      full_filename = g_malloc(strlen(diagram_dir)+strlen(filename)+2);
-      sprintf(full_filename, "%s" G_DIR_SEPARATOR_S "%s", diagram_dir, filename);
+      dirname = g_path_get_dirname(dia->filename);
+      full_filename = g_build_filename(dirname, filename, NULL);
       file = fopen(full_filename, "w");
       g_free(full_filename);
+      g_free(dirname);
     } else {
       file = fopen(filename, "w");
     }
