@@ -43,6 +43,8 @@
 #include "paper.h"
 #include "interface.h"
 
+#include "persistence.h"
+
 #ifdef G_OS_WIN32
 #include <io.h> /* open, close */
 #endif
@@ -72,8 +74,12 @@ typedef struct _DiaPrefData {
   GtkWidget *widget;
   gboolean hidden;
   GList *(*choice_list_function)();
+  /** A function to call after a preference item has been updated. */
+  void (*update_function)(struct _DiaPrefData *pref, char *ptr);
 } DiaPrefData;
 
+static void update_floating_toolbox(DiaPrefData *pref, char *ptr);
+
 static int default_true = 1;
 static int default_false = 0;
 static int default_major_lines = 5;
@@ -120,9 +126,10 @@ DiaPrefData prefs_data[] =
   { "undo_depth", PREF_UINT, PREF_OFFSET(undo_depth), &default_undo_depth, 0, N_("Number of undo levels:") },
   { "reverse_rubberbanding_intersects", PREF_BOOLEAN, PREF_OFFSET(reverse_rubberbanding_intersects), &default_true, 0, N_("Reverse dragging selects\nintersecting objects") },
   { "recent_documents_list_size", PREF_UINT, PREF_OFFSET(recent_documents_list_size), &default_recent_documents, 0, N_("Recent documents list size:") },
-  { "use_menu_bar", PREF_BOOLEAN, PREF_OFFSET(new_view.use_menu_bar), &default_false, 0, N_("Use menu bar") },
+  { "use_menu_bar", PREF_BOOLEAN, PREF_OFFSET(new_view.use_menu_bar), &default_true, 0, N_("Use menu bar") },
   { "toolbox_on_top", PREF_BOOLEAN, PREF_OFFSET(toolbox_on_top),
-    &default_false, 0, N_("Keep tool box on top of diagram windows") },
+    &default_false, 0, N_("Keep tool box on top of diagram windows"),
+    NULL, FALSE, NULL, update_floating_toolbox},
   
   { NULL, PREF_NONE, 0, NULL, 1, N_("New diagram:") },
   { "is_portrait", PREF_BOOLEAN, PREF_OFFSET(new_diagram.is_portrait), &default_true, 1, N_("Portrait") },
@@ -157,6 +164,9 @@ DiaPrefData prefs_data[] =
   { "grid_y", PREF_UREAL, PREF_OFFSET(grid.y), &default_real_one, 3, N_("Y Size:") },
   { "grid_colour", PREF_COLOUR, PREF_OFFSET(new_diagram.grid_color), &default_colour, 3, N_("Color:") },
   { "grid_major", PREF_UINT, PREF_OFFSET(grid.major_lines), &default_major_lines, 3, N_("Lines per major line") },
+  { "grid_hex", PREF_BOOLEAN, PREF_OFFSET(grid.hex), &default_false, 3, N_("Hex grid") },
+  { "grid_w", PREF_UREAL, PREF_OFFSET(grid.w), &default_real_one, 3, N_("Hex Size:")
+ },
   /*  { "grid_solid", PREF_BOOLEAN, PREF_OFFSET(grid.solid), &default_true, 3, N_("Solid lines:") },  */
 
   { "render_bounding_boxes", PREF_BOOLEAN,PREF_OFFSET(render_bounding_boxes),
@@ -171,9 +181,6 @@ DiaPrefData prefs_data[] =
   { "prefer_psprint", PREF_BOOLEAN,PREF_OFFSET(prefer_psprint),
     &default_false,0,"prefer psprint", NULL, TRUE},
 
-  { "recent_sheet", PREF_UINT,PREF_OFFSET(recent_sheet),
-    &default_false,0,"recent selected sheet:",NULL, TRUE},
-
   { NULL, PREF_NONE, 0, NULL, 4, N_("Diagram tree window:") },
   { "diagram_tree_save_hidden", PREF_BOOLEAN, PREF_OFFSET(dia_tree.save_hidden),
     &default_false, 4, N_("Save hidden object types")},
@@ -181,58 +188,11 @@ DiaPrefData prefs_data[] =
     &default_dtree_dia_sort, 4, "default diagram sort order", NULL, TRUE},
   { "diagram_tree_obj_sort", PREF_UINT, PREF_OFFSET(dia_tree.obj_sort),
     &default_dtree_obj_sort, 4, "default object sort order", NULL, TRUE},
-  { "diagram_tree_hidden", PREF_STRING, PREF_OFFSET(dia_tree.hidden),
-    &DIA_TREE_DEFAULT_HIDDEN, 4, "hidden type list", NULL, TRUE},
   { NULL, PREF_END_GROUP, 0, NULL, 4, NULL },
 };
 
 #define NUM_PREFS_DATA (sizeof(prefs_data)/sizeof(DiaPrefData))
 
-static const GScannerConfig     dia_prefs_scanner_config =
-{
-  (
-   " \t\n"
-   )                    /* cset_skip_characters */,
-  (
-   G_CSET_a_2_z
-   "_"
-   G_CSET_A_2_Z
-   )                    /* cset_identifier_first */,
-  (
-   G_CSET_a_2_z
-   "_-0123456789"
-   G_CSET_A_2_Z
-   )                    /* cset_identifier_nth */,
-  ( "#\n" )             /* cpair_comment_single */,
-  
-  TRUE                  /* case_sensitive */,
-  
-  FALSE                 /* skip_comment_multi */,
-  TRUE                  /* skip_comment_single */,
-  FALSE                 /* scan_comment_multi */,
-  TRUE                  /* scan_identifier */,
-  TRUE                  /* scan_identifier_1char */,
-  FALSE                 /* scan_identifier_NULL */,
-  TRUE                  /* scan_symbols */,
-  FALSE                 /* scan_binary */,
-  FALSE                 /* scan_octal */,
-  TRUE                  /* scan_float */,
-  TRUE                  /* scan_hex */,
-  FALSE                 /* scan_hex_dollar */,
-  FALSE                 /* scan_string_sq */,
-  TRUE                  /* scan_string_dq */,
-  TRUE                  /* numbers_2_int */,
-  FALSE                 /* int_2_float */,
-  FALSE                 /* identifier_2_string */,
-  TRUE                  /* char_2_token */,
-  FALSE                 /* symbol_2_token */,
-  FALSE                 /* scope_0_fallback */,
-};
-
-typedef enum {
-  DIA_PREFS_TOKEN_BOOLEAN = G_TOKEN_LAST
-} DiaPrefsTokenType;
-
 static void prefs_create_dialog(void);
 static void prefs_set_value_in_widget(GtkWidget * widget, DiaPrefData *data,  char *ptr);
 static void prefs_get_value_from_widget(GtkWidget * widget, DiaPrefData *data, char *ptr);
@@ -269,20 +229,27 @@ prefs_set_defaults(void)
 
     switch (prefs_data[i].type) {
     case PREF_BOOLEAN:
+      *(int *)ptr = *(int *)prefs_data[i].default_value;
+      *(int *)ptr = persistence_register_boolean(prefs_data[i].name, *(int *)ptr);
+      break;
     case PREF_INT:
     case PREF_UINT:
       *(int *)ptr = *(int *)prefs_data[i].default_value;
+      *(int *)ptr = persistence_register_integer(prefs_data[i].name, *(int *)ptr);
       break;
     case PREF_REAL:
     case PREF_UREAL:
       *(real *)ptr = *(real *)prefs_data[i].default_value;
+      *(real *)ptr = persistence_register_real(prefs_data[i].name, *(real *)ptr);
       break;
     case PREF_COLOUR:
       *(Color *)ptr = *(Color *)prefs_data[i].default_value;
+      *(Color *)ptr = *persistence_register_color(prefs_data[i].name, (Color *)ptr);
       break;
     case PREF_CHOICE:
     case PREF_STRING:
       *(gchar **)ptr = *(gchar **)prefs_data[i].default_value;
+      *(gchar **)ptr = persistence_register_string(prefs_data[i].name, *(gchar **)ptr);
       break;
     case PREF_NONE:
     case PREF_END_GROUP:
@@ -296,206 +263,46 @@ prefs_save(void)
 {
   int i;
   char *ptr;
-  gchar *filename;
-  char *old_locale;
-  FILE *file;
-
-  filename = dia_config_filename("diarc");
-
-  file = fopen(filename, "w");
-
-  if (!file) {
-    message_error(_("Could not open `%s' for writing"), filename);
-    g_free(filename);
-    return;
-  }
-  
-  g_free(filename);
-
-  old_locale = setlocale(LC_NUMERIC, "C");
-  fprintf(file, "# Note: This file is automatically generated by Dia\n");
   for (i=0;i<NUM_PREFS_DATA;i++) {
     if ((prefs_data[i].type == PREF_NONE) || (prefs_data[i].type == PREF_END_GROUP))
       continue;
     
-    fprintf(file, "%s=", prefs_data[i].name);
-    
     ptr = (char *)&prefs + prefs_data[i].offset;
 
     switch (prefs_data[i].type) {
     case PREF_BOOLEAN:
-      fprintf(file, (*(int *)ptr)?"true\n":"false\n");
+      persistence_set_boolean(prefs_data[i].name, *(gint *)ptr);
       break;
     case PREF_INT:
     case PREF_UINT:
-      fprintf(file, "%d\n", *(int *)ptr);
+      persistence_set_integer(prefs_data[i].name, *(gint *)ptr);
       break;
     case PREF_REAL:
     case PREF_UREAL:
       
-      fprintf(file, "%f\n", (double) *(real *)ptr);
+      persistence_set_real(prefs_data[i].name, *(real *)ptr);
       break;
     case PREF_COLOUR:
-      fprintf(file, "%f %f %f\n", (double) ((Color *)ptr)->red,
-	      (double) ((Color *)ptr)->green, (double) ((Color *)ptr)->blue);
+      persistence_set_color(prefs_data[i].name, (Color *)ptr);
       break;
     case PREF_CHOICE:
     case PREF_STRING:
-      fprintf(file, "\"%s\"\n", *(gchar **)ptr);
+      persistence_set_string(prefs_data[i].name, *(gchar **)ptr);
       break;
     case PREF_NONE:
     case PREF_END_GROUP:
       break;
     }
   }
-  setlocale(LC_NUMERIC, old_locale);
-  fclose(file);
 }
 
 
-static guint
-prefs_parse_line(GScanner *scanner)
-{
-  guint token;
-  int symbol_nr;
-  char *ptr;
-  
-  token = g_scanner_get_next_token(scanner);
-  if (token != G_TOKEN_SYMBOL)
-    return G_TOKEN_SYMBOL;
-
-  symbol_nr = GPOINTER_TO_INT(scanner->value.v_symbol);
-  
-  token = g_scanner_get_next_token(scanner);
-  if (token != G_TOKEN_EQUAL_SIGN)
-    return G_TOKEN_EQUAL_SIGN;
-
-  token = g_scanner_get_next_token(scanner);
-
-  ptr = (unsigned char *)&prefs + prefs_data[symbol_nr].offset;
-  
-  switch (prefs_data[symbol_nr].type) {
-  case PREF_BOOLEAN:
-    if (token != G_TOKEN_IDENTIFIER)
-      return G_TOKEN_IDENTIFIER;
-    
-    if (strcmp(scanner->value.v_string, "true")==0)
-      *(int *)ptr = 1;
-    else
-    *(int *)ptr = 0;
-    break;
-    
-  case PREF_INT:
-  case PREF_UINT:
-    if (token != G_TOKEN_INT)
-      return G_TOKEN_INT;
-    
-    *(int *)ptr = scanner->value.v_int;
-    break;
-
-  case PREF_REAL:
-  case PREF_UREAL:
-    if (token != G_TOKEN_FLOAT)
-      return G_TOKEN_FLOAT;
-    
-    *(real *)ptr = scanner->value.v_float;
-    break;
-  case PREF_COLOUR:
-    if (token != G_TOKEN_FLOAT)
-      return G_TOKEN_FLOAT;
-    ((Color *)ptr)->red = scanner->value.v_float;
-
-    token = g_scanner_get_next_token(scanner);
-    if (token != G_TOKEN_FLOAT)
-      return G_TOKEN_FLOAT;
-    ((Color *)ptr)->green = scanner->value.v_float;
-
-    token = g_scanner_get_next_token(scanner);
-    if (token != G_TOKEN_FLOAT)
-      return G_TOKEN_FLOAT;
-    ((Color *)ptr)->blue = scanner->value.v_float;
-
-    break;
-  case PREF_CHOICE:
-  case PREF_STRING:
-    if (token != G_TOKEN_STRING)
-      return G_TOKEN_STRING;
-
-    *(char **)ptr = g_strdup(scanner->value.v_string);
-    break;
-  case PREF_NONE:
-  case PREF_END_GROUP:
-    break;
-  }
-
-  return G_TOKEN_NONE;
-}
-
 
 void
-prefs_load(void)
+prefs_init(void)
 {
-  int i;
-  gchar *filename;
-  int fd;
-  GScanner *scanner;
-  guint expected_token;
-
-  filename = dia_config_filename("diarc");
-
-  fd = open(filename, O_RDONLY);
-
-  if (fd < 0) {
-    const gchar *homedir = g_get_home_dir();
-
-    g_free(filename);
-    filename = g_strconcat(homedir, G_DIR_SEPARATOR_S ".diarc", NULL);
-    fd = open(filename, O_RDONLY);
-  }
-  g_free(filename);
-
   prefs_set_defaults();
 
-  if (fd < 0) {
-    return;
-  }
-
-  scanner = g_scanner_new ((GScannerConfig *) &dia_prefs_scanner_config);
- 
-  g_scanner_input_file (scanner, fd);
-
-  scanner->input_name = filename;
-  for (i = 0; i < NUM_PREFS_DATA; i++)
-    if (prefs_data[i].name != NULL) {
-      g_scanner_add_symbol(scanner, prefs_data[i].name,
-			   GINT_TO_POINTER(i));
-    }
-  while (1) {
-    if (g_scanner_peek_next_token(scanner) == G_TOKEN_EOF) {
-      break;
-    } 
-
-    expected_token = prefs_parse_line(scanner);
-      
-    if (expected_token != G_TOKEN_NONE) {
-      gchar *symbol_name;
-      gchar *msg;
-      
-      msg = NULL;
-      symbol_name = NULL;
-      g_scanner_unexp_token (scanner,
-			     expected_token,
-			     NULL,
-			     "keyword",
-			     symbol_name,
-			     msg,
-			     TRUE);
-    }
-  }
-  
-  g_scanner_destroy (scanner);
- 
-  close(fd);
   render_bounding_boxes = prefs.render_bounding_boxes;
 }
 
@@ -518,7 +325,7 @@ prefs_set_value_in_widget(GtkWidget * widget, DiaPrefData *data,
 			      (gfloat) (*((real *)ptr)));
     break;
   case PREF_COLOUR:
-    dia_color_selector_set_color(DIACOLORSELECTOR(widget), (Color *)ptr);
+    dia_color_selector_set_color(widget, (Color *)ptr);
     break;
   case PREF_CHOICE: {
     GList *names = (data->choice_list_function)();
@@ -552,8 +359,7 @@ prefs_get_value_from_widget(GtkWidget * widget, DiaPrefData *data,
     break;
   case PREF_INT:
   case PREF_UINT:
-    *((int *)ptr) =
-      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+    *((int *)ptr) = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
     break;
   case PREF_REAL:
   case PREF_UREAL:
@@ -561,7 +367,7 @@ prefs_get_value_from_widget(GtkWidget * widget, DiaPrefData *data,
       gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(widget));
     break;
   case PREF_COLOUR:
-    dia_color_selector_get_color(DIACOLORSELECTOR(widget), (Color *)ptr);
+    dia_color_selector_get_color(widget, (Color *)ptr);
     break;
   case PREF_CHOICE: {
     int index = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
@@ -570,13 +376,15 @@ prefs_get_value_from_widget(GtkWidget * widget, DiaPrefData *data,
     break;
   }
   case PREF_STRING:
-    *((gchar **)ptr) = (gchar *)
-      gtk_entry_get_text(GTK_ENTRY(widget));
+    *((gchar **)ptr) = (gchar *)gtk_entry_get_text(GTK_ENTRY(widget));
     break;
   case PREF_NONE:
   case PREF_END_GROUP:
     break;
   }
+  if (data->update_function != NULL) {
+    (data->update_function)(data, ptr);
+  }
 }
 
 static void
@@ -846,3 +654,26 @@ prefs_update_dialog_from_prefs(void)
     prefs_set_value_in_widget(widget, &prefs_data[i],  ptr);
   }
 }
+
+static void
+update_floating_toolbox(DiaPrefData *pref, char *ptr)
+{
+  if (prefs.toolbox_on_top) {
+    /* Go through all diagrams and set toolbox transient for all displays */
+    GList *diagrams;
+    for (diagrams = dia_open_diagrams(); diagrams != NULL; 
+	 diagrams = g_list_next(diagrams)) {
+      Diagram *diagram = (Diagram *)diagrams->data;
+      GSList *displays;
+      for (displays = diagram->displays; displays != NULL; 
+	   displays = g_slist_next(displays)) {
+	DDisplay *ddisp = (DDisplay *)displays->data;
+	gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
+				     GTK_WINDOW(ddisp->shell));
+      }
+    }
+  } else {
+    gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
+				 NULL);
+  }
+}
