@@ -15,13 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/** \file dia_xml.c  Helper function to convert Dia's basic to and from XML */
 #include <config.h>
 
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <locale.h>
 #include <math.h>
 #include <fcntl.h>
 
@@ -53,21 +53,28 @@
 
 #define BUFLEN 1024
 
-
+/** If all files produced by dia were good XML files, we wouldn't have to do 
+ *  this little gymnastic. Alas, during the libxml1 days, we were outputting 
+ *  files with no encoding specification (which means UTF-8 if we're in an
+ *  asciish encoding) and strings encoded in local charset (so, we wrote
+ *  broken files). 
+ *
+ *  The following logic finds if we have a broken file, and attempts to fix 
+ *  it if it's possible. If the file is correct or is unrecognisable, we pass
+ *  it untouched to libxml2.
+ * @param filename The name of the file to check.
+ * @param default_enc The default encoding to use if none is given.
+ * @return The filename given if it seems ok, or the name of a new file
+ *          with fixed contents, or NULL if we couldn't read the file.  The
+ *          caller should free this string and unlink the file if it is not
+ *          the same as `filename'.
+ * @bug The many gzclose-g_free-return sequences should be refactored into
+ *       an "exception handle" (goto+label). At least for people who think goto is
+ *       better than this. I dont. --hb
+ */
 static const gchar *
 xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
 {
-  /* If all files produced by dia were good XML files, we wouldn't have to do 
-     this little gymnastic. Alas, during the libxml1 days, we were outputting 
-     files with no encoding specification (which means UTF-8 if we're in an
-     asciish encoding) and strings encoded in local charset (so, we wrote
-     broken files). 
-
-     The following logic finds if we have a broken file, and attempts to fix 
-     it if it's possible. If the file is correct or is unrecognisable, we pass
-     it untouched to libxml2.
-  */
-
   gzFile zf = gzopen(filename,"rb");  
   gchar *buf;
   gchar *p,*pmax;
@@ -152,7 +159,8 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
 
   if (0 != strcmp(default_enc,"UTF-8")) {
     message_warning(_("The file %s has no encoding specification;\n"
-                      "assuming it is encoded in %s"),filename,default_enc);
+                      "assuming it is encoded in %s"),
+		    dia_message_filename(filename), default_enc);
   } else {
     gzclose(zf); /* we apply the standard here. */
     g_free(buf);
@@ -182,10 +190,14 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
   return res; /* caller frees the name and unlinks the file. */
 }
 
-
-
+/** Parse a given file into XML, handling old broken files correctly.
+ * @param filename The name of the file to read.
+ * @returns An XML document parsed from the file.
+ * @see xmlParseFile() in the XML2 library for details on the return value.
+ */
 xmlDocPtr
-xmlDiaParseFile(const char *filename) {
+xmlDiaParseFile(const char *filename)
+{
   G_CONST_RETURN char *local_charset = NULL;
   
   if (   !g_get_charset(&local_charset)
@@ -209,17 +221,30 @@ xmlDiaParseFile(const char *filename) {
   }
 }
 
+/** Relic of earlier, unhappier days.
+ * @param filename A file to parse.
+ * @return An XML document.
+ * @bug Could probably be inlined. So what?
+ */
 xmlDocPtr
-xmlDoParseFile(const char *filename) {
+xmlDoParseFile(const char *filename)
+{
   return xmlParseFile(filename);
 }
 
+/** Find a named attribute node in an XML object node.
+ *  Note that Dia has a concept of attribute node that is not the same
+ *  as an XML attribute.
+ * @param obj_node The node to look in.
+ * @param attrname The name of the attribute node to find.
+ * @return The node matching the given name, or NULL if none found.
+ */
 AttributeNode
 object_find_attribute(ObjectNode obj_node,
 		      const char *attrname)
 {
   AttributeNode attr;
-  char *name;
+  xmlChar *name;
 
   while (obj_node && xmlIsBlankNode(obj_node)) 
     obj_node = obj_node->next;
@@ -244,12 +269,19 @@ object_find_attribute(ObjectNode obj_node,
   return NULL;
 }
 
+/** Find an attribute in a composite XML node.
+ * @param composite_node The composite node to search.
+ * @param attrname The name of the attribute node to find.
+ * @return The desired node, or NULL if none exists in `composite_node'.
+ * @bug Describe in more detail how a composite node differs from an
+ *   object node.
+ */
 AttributeNode
 composite_find_attribute(DataNode composite_node,
 			 const char *attrname)
 {
   AttributeNode attr;
-  char *name;
+  xmlChar *name;
 
   while (composite_node && xmlIsBlankNode(composite_node)) 
     composite_node = composite_node->next;
@@ -274,6 +306,10 @@ composite_find_attribute(DataNode composite_node,
   return NULL;
 }
 
+/** The number of non-blank data nodes in an attribute node.
+ * @param attribute The attribute node to read from.
+ * @returns The number of non-blank data nodes in the node.
+ */
 int
 attribute_num_data(AttributeNode attribute)
 {
@@ -292,6 +328,10 @@ attribute_num_data(AttributeNode attribute)
   return nr;
 }
 
+/** Get the first data node in an attribute node.
+ * @param attribute The attribute node to look through.
+ * @return The first non-black data node in the attribute node.
+ */
 DataNode
 attribute_first_data(AttributeNode attribute)
 {
@@ -300,6 +340,10 @@ attribute_first_data(AttributeNode attribute)
   return (DataNode) data;
 }
 
+/** Get the next data node (sibling).
+ * @param data A data node to start from (e.g. just processed)
+ * @returns The next sibling data node.
+ */
 DataNode
 data_next(DataNode data)
 {
@@ -311,6 +355,15 @@ data_next(DataNode data)
   return (DataNode) data;
 }
 
+/** Get the type of a data node.
+ * @param data The data node.
+ * @return The type that the data node defines, or 0 on error.  In case of 
+ *  error, an error message is displayed.
+ * @note This function does a number of strcmp calls, which may not be the
+ *  fastest way to check if a node is of the expected type.
+ * @bug Make functions that check quickly if a node is of a specific type
+ *  (but profile first).
+ */
 DataType
 data_type(DataNode data)
 {
@@ -343,11 +396,15 @@ data_type(DataNode data)
   return 0;
 }
 
-
+/** Return the value of an integer-type data node.
+ * @param data The data node to read from.
+ * @returns The integer value found in the node.  If the node is not an
+ *  integer node, an error message is displayed and 0 is returned.
+ */
 int
 data_int(DataNode data)
 {
-  char *val;
+  xmlChar *val;
   int res;
   
   if (data_type(data)!=DATATYPE_INT) {
@@ -362,9 +419,14 @@ data_int(DataNode data)
   return res;
 }
 
+/** Return the value of an enum-type data node.
+ * @param data The data node to read from.
+ * @returns The enum value found in the node.  If the node is not an
+ *  enum node, an error message is displayed and 0 is returned.
+ */
 int data_enum(DataNode data)
 {
-  char *val;
+  xmlChar *val;
   int res;
   
   if (data_type(data)!=DATATYPE_ENUM) {
@@ -379,10 +441,15 @@ int data_enum(DataNode data)
   return res;
 }
 
+/** Return the value of a real-type data node.
+ * @param data The data node to read from.
+ * @returns The real value found in the node.  If the node is not a
+ *  real-type node, an error message is displayed and 0.0 is returned.
+ */
 real
 data_real(DataNode data)
 {
-  char *val;
+  xmlChar *val;
   real res;
 
   if (data_type(data)!=DATATYPE_REAL) {
@@ -397,10 +464,15 @@ data_real(DataNode data)
   return res;
 }
 
+/** Return the value of a boolean-type data node.
+ * @param data The data node to read from.
+ * @returns The boolean value found in the node.  If the node is not a
+ *  boolean node, an error message is displayed and FALSE is returned.
+ */
 int
 data_boolean(DataNode data)
 {
-  char *val;
+  xmlChar *val;
   int res;
   
   if (data_type(data)!=DATATYPE_BOOLEAN) {
@@ -420,7 +492,13 @@ data_boolean(DataNode data)
   return res;
 }
 
-static int hex_digit(char c)
+/** Return the integer value of a hex digit.
+ * @param c A hex digit, one of 0-9, a-f or A-F.
+ * @returns The value of the digit, i.e. 0-15.  If a non-gex digit is given
+ *  an error message is displayed to the user, and 0 is returned.
+ */
+static int 
+hex_digit(char c)
 {
   if ((c>='0') && (c<='9'))
     return c-'0';
@@ -428,13 +506,21 @@ static int hex_digit(char c)
     return (c-'a') + 10;
   if ((c>='A') && (c<='F'))
     return (c-'A') + 10;
-  message_error("wrong hex digit!");
+  message_error("wrong hex digit %c", c);
   return 0;
 }
+
+/** Return the value of a color-type data node.
+ * @param data The XML node to read from
+ * @param col A place to store the resulting RGB values.  If the node does
+ *  not contain a valid color value, an error message is displayed to the
+ *  user, and `col' is unchanged.
+ * @note Could be cool to use RGBA data here, even if we can't display it yet.
+ */
 void
 data_color(DataNode data, Color *col)
 {
-  char *val;
+  xmlChar *val;
   int r=0, g=0, b=0;
   
   if (data_type(data)!=DATATYPE_COLOR) {
@@ -460,11 +546,17 @@ data_color(DataNode data, Color *col)
   col->blue = ((float)b)/255.0;
 }
 
+/** Return the value of a point-type data node.
+ * @param data The XML node to read from
+ * @param point A place to store the resulting x, y values.  If the node does
+ *  not contain a valid point value, an error message is displayed to the
+ *  user, and `point' is unchanged.
+ */
 void
 data_point(DataNode data, Point *point)
 {
-  char *val;
-  char *str;
+  xmlChar *val;
+  gchar *str;
   real ax,ay;
 
   if (data_type(data)!=DATATYPE_POINT) {
@@ -500,11 +592,17 @@ data_point(DataNode data, Point *point)
   xmlFree(val);
 }
 
+/** Return the value of a rectangle-type data node.
+ * @param data The data node to read from.
+ * @param rect A place to store the resulting values.  If the node does
+ *  not contain a valid rectangle value, an error message is displayed to the
+ *  user, and `rect' is unchanged.
+ */
 void
 data_rectangle(DataNode data, Rectangle *rect)
 {
-  char *val;
-  char *str;
+  xmlChar *val;
+  gchar *str;
   
   if (data_type(data)!=DATATYPE_RECTANGLE) {
     message_error("Taking rectangle value of non-rectangle node.");
@@ -551,10 +649,17 @@ data_rectangle(DataNode data, Rectangle *rect)
   xmlFree(val);
 }
 
+/** Return the value of a string-type data node.
+ * @param data The data node to read from.
+ * @returns The string value found in the node.  If the node is not a
+ *  string node, an error message is displayed and NULL is returned.  The
+ *  returned valuee should be freed after use.
+ * @note For historical reasons, strings in Dia XML are surrounded by ##.
+ */
 gchar *
 data_string(DataNode data)
 {
-  gchar *val;
+  xmlChar *val;
   gchar *str, *p,*str2;
   int len;
   
@@ -620,6 +725,14 @@ data_string(DataNode data)
   return NULL;
 }
 
+/** Return the value of a filename-type data node.
+ * @param data The data node to read from.
+ * @return The filename value found in the node.  If the node is not a
+ *  filename node, an error message is displayed and NULL is returned.
+ *  The resulting string is in the local filesystem's encoding rather than
+ *  UTF-8, and should be freed after use.
+ * @bug data_string() can return NULL, what does g_filename_from_utf8 do then?
+ */
 char *
 data_filename(DataNode data)
 {
@@ -629,10 +742,17 @@ data_filename(DataNode data)
   return filename;
 }
 
+/** Return the value of a font-type data node.  This handles both the current
+ * format (family and style) and the old format (name).
+ * @param data The data node to read from.
+ * @return The font value found in the node.  If the node is not a
+ *  font node, an error message is displayed and NULL is returned.  The
+ *  resulting value should be freed after use.
+ */
 DiaFont *
 data_font(DataNode data)
 {
-  char *family;
+  xmlChar *family;
   DiaFont *font;
   
   if (data_type(data)!=DATATYPE_FONT) {
@@ -644,7 +764,7 @@ data_font(DataNode data)
   /* always prefer the new format */
   if (family) {
     DiaFontStyle style;
-    char* style_name = xmlGetProp(data, "style");
+    xmlChar* style_name = xmlGetProp(data, "style");
     style = style_name ? atoi(style_name) : 0;
 
     font = dia_font_new (family, style, 1.0);
@@ -661,6 +781,13 @@ data_font(DataNode data)
 
 /* ***** Saving XML **** */
 
+/** Create a new attribute node.
+ * @param obj_node The object node to create the attribute node under.
+ * @param attrname The name of the attribute node.
+ * @return A new attribute node.
+ * @bug Should have utility functions that creates the node and sets
+ *  the value based on type.
+ */
 AttributeNode
 new_attribute(ObjectNode obj_node,
 	      const char *attrname)
@@ -672,6 +799,12 @@ new_attribute(ObjectNode obj_node,
   return attr;
 }
 
+/** Add an attribute node to a composite node.
+ * @param composite_node The composite node.
+ * @param attrname The name of the new attribute node.
+ * @return The attribute node added.
+ * @bug This does exactly the same as new_attribute.
+ */
 AttributeNode
 composite_add_attribute(DataNode composite_node,
 			const char *attrname)
@@ -683,6 +816,10 @@ composite_add_attribute(DataNode composite_node,
   return attr;
 }
 
+/** Add integer data to an attribute node.
+ * @param attr The attribute node.
+ * @param data The value to set.
+ */
 void
 data_add_int(AttributeNode attr, int data)
 {
@@ -695,6 +832,10 @@ data_add_int(AttributeNode attr, int data)
   xmlSetProp(data_node, "val", buffer);
 }
 
+/** Add enum data to an attribute node.
+ * @param attr The attribute node.
+ * @param data The value to set.
+ */
 void
 data_add_enum(AttributeNode attr, int data)
 {
@@ -707,6 +848,10 @@ data_add_enum(AttributeNode attr, int data)
   xmlSetProp(data_node, "val", buffer);
 }
 
+/** Add real-typed data to an attribute node.
+ * @param attr The attribute node.
+ * @param data The value to set.
+ */
 void
 data_add_real(AttributeNode attr, real data)
 {
@@ -719,6 +864,10 @@ data_add_real(AttributeNode attr, real data)
   xmlSetProp(data_node, "val", buffer);
 }
 
+/** Add boolean data to an attribute node.
+ * @param attr The attribute node.
+ * @param data The value to set.
+ */
 void
 data_add_boolean(AttributeNode attr, int data)
 {
@@ -731,8 +880,16 @@ data_add_boolean(AttributeNode attr, int data)
     xmlSetProp(data_node, "val", "false");
 }
 
-
-static void convert_to_hex(float x, char *str)
+/** Convert a floating-point value to hexadecimal.
+ * @param x The floating point value.
+ * @param str A string to place the result in.
+ * @note Currently only works for 0 <= x <= 255 and will silently cap the value
+ *  to those limits.  Also expects str to have at least two bytes allocated,
+ *  and doesn't null-terminate it.  This works well for converting a color
+ *  value, but is pretty much useless for other values.
+ */
+static void
+convert_to_hex(float x, char *str)
 {
   static const char hex_digit[] = "0123456789abcdef";
   int val;
@@ -747,6 +904,10 @@ static void convert_to_hex(float x, char *str)
   str[1] = hex_digit[val%16];
 }
 
+/** Add color data to an attribute node.
+ * @param attr The attribute node.
+ * @param col The value to set.
+ */
 void
 data_add_color(AttributeNode attr, const Color *col)
 {
@@ -763,39 +924,58 @@ data_add_color(AttributeNode attr, const Color *col)
   xmlSetProp(data_node, "val", buffer);
 }
 
+/** Add point data to an attribute node.
+ * @param attr The attribute node.
+ * @param point The value to set.
+ */
 void
 data_add_point(AttributeNode attr, const Point *point)
 {
   DataNode data_node;
-  char buffer[80+1]; /* Large enought? */
-  char *old_locale;
-
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buffer, 80, "%g,%g", point->x, point->y);
-  setlocale(LC_NUMERIC, old_locale);
+  gchar *buffer;
+  gchar px_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar py_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  
+  g_ascii_formatd(px_buf, sizeof(px_buf), "%g", point->x);
+  g_ascii_formatd(py_buf, sizeof(py_buf), "%g", point->y);
+  buffer = g_strconcat(px_buf, ",", py_buf, NULL);
   
   data_node = xmlNewChild(attr, NULL, "point", NULL);
   xmlSetProp(data_node, "val", buffer);
+  g_free(buffer);
 }
 
+/** Add rectangle data to an attribute node.
+ * @param attr The attribute node.
+ * @param rect The value to set.
+ */
 void
 data_add_rectangle(AttributeNode attr, const Rectangle *rect)
 {
   DataNode data_node;
-  char buffer[160+1]; /* Large enough? */
-  char *old_locale;
+  gchar *buffer;
+  gchar rl_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar rr_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar rt_buf[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar rb_buf[G_ASCII_DTOSTR_BUF_SIZE];
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buffer, 160, "%g,%g;%g,%g",
-	     rect->left, rect->top,
-	     rect->right, rect->bottom);
-  setlocale(LC_NUMERIC, old_locale);
+  g_ascii_formatd(rl_buf, sizeof(rl_buf), "%g", rect->left);
+  g_ascii_formatd(rr_buf, sizeof(rr_buf), "%g", rect->right);
+  g_ascii_formatd(rt_buf, sizeof(rt_buf), "%g", rect->top);
+  g_ascii_formatd(rb_buf, sizeof(rb_buf), "%g", rect->bottom);
+
+  buffer = g_strconcat(rl_buf, ",", rt_buf, ";", rr_buf, ",", rb_buf, NULL);
   
   data_node = xmlNewChild(attr, NULL, "rectangle", NULL);
   xmlSetProp(data_node, "val", buffer);
 
+  g_free(buffer);
 }
 
+/** Add string data to an attribute node.
+ * @param attr The attribute node.
+ * @param str The value to set.
+ */
 void
 data_add_string(AttributeNode attr, const char *str)
 {
@@ -819,6 +999,11 @@ data_add_string(AttributeNode attr, const char *str)
     g_free(sharped_str);
 }
 
+/** Add filename data to an attribute node.
+ * @param attr The attribute node.
+ * @param filename The value to set.  This should be n the local filesystem
+ *  encoding, not utf-8.
+ */
 void
 data_add_filename(DataNode data, const char *str)
 {
@@ -829,6 +1014,10 @@ data_add_filename(DataNode data, const char *str)
   g_free(utf8);
 }
 
+/** Add font data to an attribute node.
+ * @param attr The attribute node.
+ * @param font The value to set.
+ */
 void
 data_add_font(AttributeNode attr, const DiaFont *font)
 {
@@ -846,6 +1035,11 @@ data_add_font(AttributeNode attr, const DiaFont *font)
   xmlSetProp(data_node, "name", dia_font_get_legacy_name(font));
 }
 
+/** Add a new composite node to an attribute node.
+ * @param attr The attribute node to add to.
+ * @param type The type of the new node.
+ * @returns The new child of `attr'.
+ */
 DataNode
 data_add_composite(AttributeNode attr, const char *type) 
 {
@@ -859,6 +1053,8 @@ data_add_composite(AttributeNode attr, const char *type)
   return data_node;
 }
 
+/** Emit a warning to the user that having UTF-8 as local charset doesn't.
+ */
 void 
 warn_about_broken_libxml1(void) 
 {
@@ -875,7 +1071,14 @@ warn_about_broken_libxml1(void)
 /* diarc option */
 int pretty_formated_xml = TRUE;
 
-int xmlDiaSaveFile(const char *filename,
+/** Save an XML document to a file.
+ * @param filename The file to save to.
+ * @param cur The XML document structure.
+ * @return The return value of xmlSaveFormatFileEnc.
+ * @bug Get the proper defn of the return value from libxml2.
+ */
+int
+xmlDiaSaveFile(const char *filename,
                    xmlDocPtr cur)
 {
     int old = 0, ret;
