@@ -36,6 +36,9 @@
 #include "color.h"
 #include "font.h"
 #include "text.h"
+#include "object.h"
+
+#include "time.h"
 
 #ifdef HAVE_FREETYPE
 #include <pango/pango.h>
@@ -116,8 +119,11 @@ static void renderer_init (DiaGdkRenderer *renderer, void*);
 
 static gpointer parent_class = NULL;
 
+/** Get the type object for the GdkRenderer.
+ * @return A static GType object describing GdkRenderers.
+ */
 GType
-dia_gdk_renderer_get_type (void)
+dia_gdk_renderer_get_type(void)
 {
   static GType object_type = 0;
 
@@ -144,8 +150,12 @@ dia_gdk_renderer_get_type (void)
   return object_type;
 }
 
+/** Initialize a renderer object.
+ * @param renderer A renderer object to initialize.
+ * @param p Ignored, purpose unknown.
+ */
 static void
-renderer_init (DiaGdkRenderer *renderer, void* p)
+renderer_init(DiaGdkRenderer *renderer, void* p)
 {
   renderer->line_width = 1;
   renderer->line_style = GDK_LINE_SOLID;
@@ -159,8 +169,11 @@ renderer_init (DiaGdkRenderer *renderer, void* p)
   renderer->highlight_color = NULL;
 }
 
+/** Clean up a renderer object after use.
+ * @param object Renderer object to free subobjects from.
+ */
 static void
-renderer_finalize (GObject *object)
+renderer_finalize(GObject *object)
 {
   DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
 
@@ -179,8 +192,12 @@ renderer_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/** Initialize members of the renderer class object.  This sets up a bunch
+ * of functions to call for various render functions.
+ * @param klass The class object to initialize. 
+ */
 static void
-dia_gdk_renderer_class_init (DiaGdkRendererClass *klass)
+dia_gdk_renderer_class_init(DiaGdkRendererClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   DiaRendererClass *renderer_class = DIA_RENDERER_CLASS (klass);
@@ -227,6 +244,13 @@ dia_gdk_renderer_class_init (DiaGdkRendererClass *klass)
   renderer_class->get_text_width = get_text_width;
 }
 
+/** Convert Dia color objects into GDK color objects.
+ * If the highlight color is set, that will be used instead.  This allows
+ * rendering of an object to do highlight rendering.
+ * @param renderer The renderer to check for highlight color.
+ * @param col A color object to convert.
+ * @param gdk_col Resulting GDK convert.
+ */
 static void
 renderer_color_convert(DiaGdkRenderer *renderer,
 		       Color *col, GdkColor *gdk_col)
@@ -319,6 +343,10 @@ set_linejoin (DiaRenderer *object, LineJoin mode)
     case LINEJOIN_BEVEL:
       renderer->join_style = GDK_JOIN_BEVEL;
       break;
+    default :
+      /* invalid mode, just here to set a breakpoint */
+      renderer->join_style = GDK_JOIN_ROUND;
+      break;
     }
   }
  
@@ -336,7 +364,7 @@ set_linejoin (DiaRenderer *object, LineJoin mode)
 void
 dia_gdk_renderer_set_dashes(DiaGdkRenderer *renderer, int offset)
 {
-  char dash_list[6];
+  gint8 dash_list[6];
   int hole_width;
   int pattern_length;
   
@@ -525,10 +553,10 @@ draw_fill_arc (DiaRenderer *object,
   dangle = angle2-angle1;
   if (dangle<0)
     dangle += 360.0;
-  
+
   gdk_draw_arc(renderer->pixmap,
 	       gc, fill,
-	       left, top, right-left, bottom-top,
+	       left, top, right-left+(fill?1:0), bottom-top+(fill?1:0),
 	       (int) (angle1*64.0), (int) (dangle*64.0));
 }
 static void 
@@ -716,20 +744,8 @@ draw_string (DiaRenderer *object,
 	   g_free(graybitmap);
 	 }
        }
-       /*
-	 gdk_draw_pixbuf(renderer->pixmap, gc, rgba, 0, 0, x, y, width, height,
-	 GDK_RGB_DITHER_NONE, 0, 0);
-       */
        if (rgba != NULL) { /* Non-null width */
-	 gdk_pixbuf_render_to_drawable_alpha(rgba, 
-					     renderer->pixmap,
-					     0, 0,
-					     x, y,
-					     width, height,
-					     GDK_PIXBUF_ALPHA_FULL,
-					     128,
-					     GDK_RGB_DITHER_NONE,
-					     0, 0);
+	 gdk_draw_pixbuf(renderer->pixmap, renderer->gc, rgba, 0, 0, x, y, width, height, GDK_RGB_DITHER_NONE, 0, 0);
 	 g_object_unref(G_OBJECT(rgba));
        }
        /*
@@ -766,7 +782,8 @@ draw_string (DiaRenderer *object,
 
 /** Caching Pango renderer */
 static void
-draw_text(DiaRenderer *renderer, Text *text) {
+draw_text(DiaRenderer *renderer, Text *text) 
+{
   Point pos;
   int i;
 
@@ -948,7 +965,8 @@ draw_polygon (DiaRenderer *self,
 static void
 draw_object (DiaRenderer *renderer, DiaObject *object)
 {
-  if (object->highlight_color != NULL) {
+  if (renderer->is_interactive &&
+      object->highlight_color != NULL) {
     DiaGdkRenderer *gdk_rend = DIA_GDK_RENDERER(renderer);
     gdk_rend->highlight_color = object->highlight_color;
     object->ops->draw(object, renderer);
