@@ -16,6 +16,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/** \file diagramdata.c  This file defines the DiagramData object, which holds (mostly) saveable
+ * data global to a diagram.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -24,17 +28,32 @@
 #include "diagramdata.h"
 #include "diarenderer.h"
 #include "paper.h"
+#include "persistence.h"
+
 #include "dynamic_obj.h"
+#include "diamarshal.h"
+
 
 static const Rectangle invalid_extents = { -1.0,-1.0,-1.0,-1.0 };
 static void set_parent_layer(gpointer layer, gpointer object);
 
 static void diagram_data_class_init (DiagramDataClass *klass);
+static void diagram_data_init (DiagramData *object);
+
+enum {
+  OBJECT_ADD,
+  OBJECT_REMOVE,
+  LAST_SIGNAL
+};
+
+static guint diagram_data_signals[LAST_SIGNAL] = { 0, };
 
 static gpointer parent_class = NULL;
 
+/** Get the type object for the DiagramData class.
+ */
 GType
-diagram_data_get_type (void)
+diagram_data_get_type(void)
 {
   static GType object_type = 0;
 
@@ -50,7 +69,7 @@ diagram_data_get_type (void)
         NULL,           /* class_data */
         sizeof (DiagramData),
         0,              /* n_preallocs */
-	NULL            /* init */
+	(GInstanceInitFunc)diagram_data_init /* init */
       };
 
       object_type = g_type_register_static (G_TYPE_OBJECT,
@@ -61,8 +80,54 @@ diagram_data_get_type (void)
   return object_type;
 }
 
+/* signal default handlers */
 static void
-diagram_data_finalize(GObject *object) 
+_diagram_data_object_add (DiagramData* dia,Layer* layer,DiaObject* obj)
+{
+}
+
+static void
+_diagram_data_object_remove (DiagramData* dia,Layer* layer,DiaObject* obj)
+{
+}
+
+/** Initialize a new diagram data object.
+ * @param data A diagram data object to initialize.
+ */
+static void
+diagram_data_init(DiagramData *data)
+{
+  Color* color = persistence_register_color ("new_diagram_bgcolour", &color_white);
+  gboolean compress = persistence_register_boolean ("compress_save", TRUE);
+  Layer *first_layer;
+
+  data->extents.left = 0.0; 
+  data->extents.right = 10.0; 
+  data->extents.top = 0.0; 
+  data->extents.bottom = 10.0; 
+ 
+  data->bg_color = *color;
+
+  get_paper_info (&data->paper, -1, NULL);
+
+  first_layer = new_layer(g_strdup(_("Background")),data);
+  
+  data->layers = g_ptr_array_new ();
+  g_ptr_array_add (data->layers, first_layer);
+  data->active_layer = first_layer;
+
+  data->selected_count_private = 0;
+  data->selected = NULL;
+  
+  data->is_compressed = compress; /* Overridden by doc */
+
+}
+
+/** Deallocate memory owned by a DiagramData object.
+ * @param object A DiagramData object to finalize.
+ */
+static void
+diagram_data_finalize(GObject *object)
 {
   DiagramData *data = DIA_DIAGRAM_DATA(object);
 
@@ -78,71 +143,59 @@ diagram_data_finalize(GObject *object)
 
   g_list_free(data->selected);
   data->selected = NULL; /* for safety */
-  data->selected_count = 0;
+  data->selected_count_private = 0;
 }
 
+/** Initialize the DiagramData class data.
+ * @param klass The class object to initialize functions for.
+ */
 static void
-diagram_data_class_init (DiagramDataClass *klass)
+diagram_data_class_init(DiagramDataClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
 
+  diagram_data_signals[OBJECT_ADD] =
+    g_signal_new ("object_add",
+              G_TYPE_FROM_CLASS (klass),
+              G_SIGNAL_RUN_FIRST,
+              G_STRUCT_OFFSET (DiagramDataClass, object_add),
+              NULL, NULL,
+              dia_marshal_VOID__POINTER_POINTER,
+              G_TYPE_NONE, 
+              2,
+              G_TYPE_POINTER,
+              G_TYPE_POINTER);
+              
+  diagram_data_signals[OBJECT_REMOVE] =
+    g_signal_new ("object_remove",
+              G_TYPE_FROM_CLASS (klass),
+              G_SIGNAL_RUN_FIRST,
+              G_STRUCT_OFFSET (DiagramDataClass, object_remove),
+              NULL, NULL,
+              dia_marshal_VOID__POINTER_POINTER,
+              G_TYPE_NONE, 
+              2,
+              G_TYPE_POINTER,
+              G_TYPE_POINTER);
+
+
   object_class->finalize = diagram_data_finalize;
+  klass->object_add = _diagram_data_object_add;
+  klass->object_remove = _diagram_data_object_remove;
+
 }
 
-DiagramData *
-new_diagram_data (NewDiagramData *prefs)
-{
-  DiagramData *data;
-  Layer *first_layer;
-   
-  data = g_object_new (DIA_TYPE_DIAGRAM_DATA, NULL);
-   
-  data->extents.left = 0.0; 
-  data->extents.right = 10.0; 
-  data->extents.top = 0.0; 
-  data->extents.bottom = 10.0; 
- 
-  data->bg_color = prefs->bg_color;
-  data->pagebreak_color = prefs->pagebreak_color;
-
-  get_paper_info (&data->paper, -1, prefs);
-
-  data->grid.dynamic = TRUE;
-  data->grid.width_x = 1.0;
-  data->grid.width_y = 1.0;
-  data->grid.width_w = 1.0;
-  data->grid.visible_x = 1;
-  data->grid.visible_y = 1;
-  data->grid.colour = prefs->grid_color;
-  data->grid.hex = FALSE;
-  data->grid.hex_size = 1.0;
-
-  data->guides.nhguides = 0;
-  data->guides.hguides = NULL;
-  data->guides.nvguides = 0;
-  data->guides.vguides = NULL;
-
-  first_layer = new_layer(g_strdup(_("Background")),data);
-  
-  data->layers = g_ptr_array_new ();
-  g_ptr_array_add (data->layers, first_layer);
-  data->active_layer = first_layer;
-
-  data->selected_count = 0;
-  data->selected = NULL;
-  
-  data->is_compressed = prefs->compress_save; /* Overridden by doc */
-
-  return data;
-}
-
-void
-diagram_data_destroy(DiagramData *data) {
-  g_object_unref(data);
-}
-
+/** Create a new layer in this diagram.
+ * @param name Name of the new layer.
+ * @param parent The DiagramData that the layer will belong to,.
+ * @return A new Layer object.
+ * @bug Must determine if a NULL name is ok.
+ * @bug This belongs in a layers.c file.
+ * @bug Even though this sets parent diagram, it doesn't call the
+ * update_extents functions that add_layer does.
+ */
 Layer *
 new_layer(gchar *name, DiagramData *parent)
 {
@@ -166,6 +219,10 @@ new_layer(gchar *name, DiagramData *parent)
   return layer;
 }
 
+/** Destroy a layer object.
+ * @param layer The layer object to deallocate entirely.
+ * @bug This belongs in a layers.c file.
+ */
 void
 layer_destroy(Layer *layer)
 {
@@ -174,6 +231,11 @@ layer_destroy(Layer *layer)
   g_free(layer);
 }
 
+/** Raise a layer up one in a diagram.
+ * @param data The diagram that the layer belongs to.
+ * @param layer The layer to raise.
+ * @bug The diagram doesn't really need to be passed, as the layer knows it.
+ */
 void
 data_raise_layer(DiagramData *data, Layer *layer)
 {
@@ -196,6 +258,11 @@ data_raise_layer(DiagramData *data, Layer *layer)
   }
 }
 
+/** Lower a layer by one in a diagram.
+ * @param data The diagram that the layer belongs to.
+ * @param layer The layer to lower.
+ * @bug The diagram doesn't really need to be passed, as the layer knows it.
+ */
 void
 data_lower_layer(DiagramData *data, Layer *layer)
 {
@@ -218,7 +285,11 @@ data_lower_layer(DiagramData *data, Layer *layer)
   }
 }
 
-
+/** Add a layer object to a diagram.
+ * @param data The diagram to add the layer to.
+ * @param layer The layer to add.
+ * @bug Should just call data_add_layer_at().
+ */
 void
 data_add_layer(DiagramData *data, Layer *layer)
 {
@@ -227,6 +298,7 @@ data_add_layer(DiagramData *data, Layer *layer)
   layer_update_extents(layer);
   data_update_extents(data);
 }
+
 void
 data_add_layer_at(DiagramData *data, Layer *layer, int pos)
 {
@@ -248,12 +320,22 @@ data_add_layer_at(DiagramData *data, Layer *layer, int pos)
   data_update_extents(data);
 }
 
+/** Set which layer is the active layer in a diagram.
+ * @param data The diagram in which to set the active layer.
+ * @param layer The layer that should be active.
+ * @bug The diagram doesn't really need to be passed, as the layer knows it.
+ */
 void
 data_set_active_layer(DiagramData *data, Layer *layer)
 {
   data->active_layer = layer;
 }
 
+/** Delete a layer from a diagram.
+ * @param data The diagram to delete the layer from.
+ * @param layer The layer to delete.
+ * @bug The diagram doesn't really need to be passed, as the layer knows it.
+ */
 void
 data_delete_layer(DiagramData *data, Layer *layer)
 {
@@ -271,31 +353,54 @@ data_delete_layer(DiagramData *data, Layer *layer)
   }
 }
 
+/** Select an object in a diagram.  Note that this does not unselect other
+ *  objects currently selected in the diagram.
+ * @param data The diagram to select in.
+ * @param obj The object to select.
+ * @bug Does not need to be passed the diagram, as it can be found from the 
+ *  object.
+ */
 void
 data_select(DiagramData *data, DiaObject *obj)
 {
+  if (g_list_find (data->selected, obj))
+    return; /* should this be an error?`*/
   data->selected = g_list_prepend(data->selected, obj);
-  data->selected_count++;
+  data->selected_count_private++;
 }
 
+/** Deselect an object in a diagram.  Note that other objects may still be
+ *  selected after this function is done.
+ * @param data The diagram to deselect in.
+ * @param obj The object to deselect.
+ * @bug Does not need to be passed the diagram, as it can be found from the 
+ *  object.
+ */
 void
 data_unselect(DiagramData *data, DiaObject *obj)
 {
+  if (!g_list_find (data->selected, obj))
+    return; /* should this be an error?`*/
   data->selected = g_list_remove(data->selected, obj);
-  data->selected_count--;
+  data->selected_count_private--;
 }
 
 /** Clears the list of selected objects.
- * Does *not* remove these objects from the object list.
+ * Does *not* remove these objects from the object list, despite its name.
+ * @param data The diagram to unselect all objects in.
  */
 void
 data_remove_all_selected(DiagramData *data)
 {
   g_list_free(data->selected); /* Remove previous selection */
-  data->selected_count = 0;
   data->selected = NULL;
+  data->selected_count_private = 0;
 }
 
+/** Return TRUE if the diagram has visible layers.
+ * @param data The diagram to check.
+ * @return TRUE if at least one layer in the diagram is visible.
+ */
 static gboolean
 data_has_visible_layers(DiagramData *data)
 {
@@ -307,8 +412,12 @@ data_has_visible_layers(DiagramData *data)
   return FALSE;
 }
 
+/** Set the diagram extents field to the union of the extents of the layers.
+ * @param data The diagram to get the extents for.
+ */
 static void
-data_get_layers_extents_union(DiagramData *data) {
+data_get_layers_extents_union(DiagramData *data)
+{
   guint i;
   gboolean first = TRUE;
   Rectangle new_extents;
@@ -332,6 +441,11 @@ data_get_layers_extents_union(DiagramData *data) {
   data->extents = new_extents;
 }
 
+/** Change diagram scaling so that the extents are exactly visible.
+ * @param data The diagram to adjust.
+ * @bug Consider making it a teeny bit larger, or check that *all* objects
+ *  calculate their extents correctly.
+ */
 static void
 data_adapt_scaling_to_extents(DiagramData *data)
 {
@@ -348,8 +462,12 @@ data_adapt_scaling_to_extents(DiagramData *data)
   data->paper.height = pheight / data->paper.scaling;
 }
 
+/** Adjust the extents field of a diagram.
+ * @param data The diagram to adjust.
+ * @return TRUE if the extents changed.
+ */
 static gboolean
-data_compute_extents(DiagramData *data) 
+data_compute_extents(DiagramData *data)
 {
   Rectangle old_extents = data->extents;
 
@@ -375,6 +493,10 @@ data_compute_extents(DiagramData *data)
   return (!rectangle_equals(&data->extents,&old_extents));
 }
 
+/** Update the extents of a diagram and adjust scaling if needed.
+ * @param data Diagram to update.
+ * @return TRUE if the diagram extents changed.
+ */
 gboolean
 data_update_extents(DiagramData *data)
 {
@@ -387,6 +509,12 @@ data_update_extents(DiagramData *data)
   return changed;
 }
 
+/** Get a list of selected objects in layer ordering.
+ * @param data The diagram to get objects from.
+ * @return A list of all currently selected objects.  These all reside in
+ *  the currently active layer.  This list should be freed after use.
+ * @bug Does selection update correctly when the layer changes?
+ */
 GList *
 data_get_sorted_selected(DiagramData *data)
 {
@@ -395,9 +523,10 @@ data_get_sorted_selected(DiagramData *data)
   GList *found;
   DiaObject *obj;
 
-  if (data->selected_count == 0)
+  g_assert (g_list_length (data->selected) == data->selected_count_private);
+  if (data->selected_count_private == 0)
     return NULL;
-  
+ 
   sorted_list = NULL;
   list = g_list_last(data->active_layer->objects);
   while (list != NULL) {
@@ -414,6 +543,10 @@ data_get_sorted_selected(DiagramData *data)
 
 /** Remove the currently selected objects from the list of objects.
  * The selected objects are returned in a newly created GList.
+ * @param data The diagram to get and remove the selected objects from.
+ * @return A list of all selected objects in the current diagram.  This list
+ *  should be freed after use.  Unlike data_get_sorted_selected, the
+ *  objects in the list are not in the diagram anymore.
  */
 GList *
 data_get_sorted_selected_remove(DiagramData *data)
@@ -423,7 +556,8 @@ data_get_sorted_selected_remove(DiagramData *data)
   GList *found;
   DiaObject *obj;
   
-  if (data->selected_count == 0)
+  g_assert (g_list_length (data->selected) == data->selected_count_private);
+  if (data->selected_count_private == 0)
     return NULL;
   
   sorted_list = NULL;
@@ -446,9 +580,38 @@ data_get_sorted_selected_remove(DiagramData *data)
   return sorted_list;
 }
 
+
+/** Emits a GObject signal on DiagramData
+ *  @param data The DiagramData that emits the signal.
+ *  @param layer The Layer that the fired signal carries.
+ *  @param obj The DiaObject that the fired signal carries.
+ *  @param signal_name The name of the signal.
+ */
+void 
+data_emit(DiagramData *data,Layer *layer,DiaObject* obj,const char *signal_name) 
+{
+
+    /* check what signal it is */
+    if (strcmp("object_add",signal_name) == 0)
+        g_signal_emit (data, diagram_data_signals[OBJECT_ADD], 0,layer,obj);
+    
+    if (strcmp("object_remove",signal_name) == 0)
+        g_signal_emit (data, diagram_data_signals[OBJECT_REMOVE], 0,layer,obj);
+          
+}
+
+
+/** Render a diagram.
+ * @param data The diagram to render.
+ * @param renderer The renderer to render on.
+ * @param update The area that needs updating.
+ * @param obj_renderer If non-NULL, an alternative renderer of objects.
+ * @param gdata User data passed on to inner calls.
+ * @bug Describe obj_renderer better.
+ */
 void
 data_render(DiagramData *data, DiaRenderer *renderer, Rectangle *update,
-	    ObjectRenderer obj_renderer /* Can be NULL */,
+	    ObjectRenderer obj_renderer,
 	    gpointer gdata)
 {
   Layer *layer;
@@ -467,6 +630,13 @@ data_render(DiagramData *data, DiaRenderer *renderer, Rectangle *update,
   if (!renderer->is_interactive) (DIA_RENDERER_GET_CLASS(renderer)->end_render)(renderer);
 }
 
+/** The default object renderer.
+ * @param obj An object to render.
+ * @param renderer The renderer to render on.
+ * @param active_layer The layer containing the object.
+ * @param data The diagram containing the layer.
+ * @bug The active_layer and data variables can be inferred from the object.
+ */
 static void
 normal_render(DiaObject *obj, DiaRenderer *renderer,
 	      int active_layer,
@@ -478,7 +648,18 @@ normal_render(DiaObject *obj, DiaRenderer *renderer,
 
 int render_bounding_boxes = FALSE;
 
-/* If obj_renderer is NULL normal_render is used. */
+/** Render all components of a single layer.  This function also handles
+ *  rendering of bounding boxes for debugging purposes.
+ * @param layer The layer to render.
+ * @param renderer The renderer to draw things with.
+ * @param update The rectangle that requires update.  Only objects that
+ *  intersect with this rectangle will actually be get rendered.
+ * @param obj_renderer A function that will render an object.
+ * @param data The diagram that the layer belongs to.
+ * @param active_layer Which number layer in the diagram is currently active.
+ * @bug data and active_layer can be inferred from layer, though possibly 
+ *  slowly.
+ */
 void
 layer_render(Layer *layer, DiaRenderer *renderer, Rectangle *update,
 	     ObjectRenderer obj_renderer,
@@ -518,56 +699,138 @@ layer_render(Layer *layer, DiaRenderer *renderer, Rectangle *update,
   }
 }
 
+/** Set the parent layer of an object.
+ * @param element A DiaObject that should be part of a layer.
+ * @param user_data The layer it should be part of.
+ */
 static void
 set_parent_layer(gpointer element, gpointer user_data) 
 {
   ((DiaObject*)element)->parent_layer = (Layer*)user_data;
   /* FIXME: even group members need a parent_layer and what about parent objects  ??? 
-      * Now I know again why I always try to avoid back-pointers )-; --hb */
+      * Now I know again why I always try to avoid back-pointers )-; --hb.
+      * If the group objects didn't actually leave the diagram, this wouldn't
+      * be a problem.  --LC */
 }
 
+/** Get the index of an object in a layer.
+ * @param layer The layer the object is (should be) in.
+ * @param obj The object to look for.
+ * @return The index of the object in the layers list of objects.  This is also
+ *  the vertical position of the object.
+ * @bug This should be in a separate layer.c file.
+ * @bug The layer could be inferred from the object, in which case the
+ *  layer arg is not needed, and we would be sure we always looked in the
+ *  right layer.
+ */
 int
 layer_object_index(Layer *layer, DiaObject *obj)
 {
   return (int)g_list_index(layer->objects, (gpointer) obj);
 }
 
+/** Add an object to the top of a layer.
+ * @param layer The layer to add the object to.
+ * @param obj The object to add.  This must not already be part of another
+ *  layer.
+ * @bug This should be in a separate layer.c file.
+ * @bug This should just call layer_add_object_at().
+ */
 void
 layer_add_object(Layer *layer, DiaObject *obj)
 {
   layer->objects = g_list_append(layer->objects, (gpointer) obj);
   set_parent_layer(obj, layer);
+
+  /* send a signal that we have added a object to the diagram */
+  g_signal_emit (layer_get_parent_diagram(layer), diagram_data_signals[OBJECT_ADD], 0,layer,obj);
 }
 
+/** Add an object to a layer at a specific position.
+ * @param layer The layer to add the object to.
+ * @param obj The object to add.  This must not be part of another layer.
+ * @param pos The top-to-bottom position this object should be inserted at.
+ * @bug This should be in a separate layer.c file.
+ */
 void
 layer_add_object_at(Layer *layer, DiaObject *obj, int pos)
 {
   layer->objects = g_list_insert(layer->objects, (gpointer) obj, pos);
   set_parent_layer(obj, layer);
+
+  /* send a signal that we have added a object to the diagram */
+  g_signal_emit (layer_get_parent_diagram(layer), diagram_data_signals[OBJECT_ADD], 0,layer,obj);
 }
 
+/** Add a list of objects to the end of a layer.
+ * @param layer The layer to add objects to.
+ * @param obj_list The list of objects to add.  These must not already
+ *  be part of another layer.
+ * @bug Determine if the list is kept by g_list_concat.
+ */
 void
 layer_add_objects(Layer *layer, GList *obj_list)
 {
+  GList *list = obj_list;
+
   layer->objects = g_list_concat(layer->objects, obj_list);
   g_list_foreach(obj_list, set_parent_layer, layer);
+
+  while (list != NULL)
+  {
+    /* send a signal that we have added a object to the diagram */
+    g_signal_emit (layer_get_parent_diagram(layer), diagram_data_signals[OBJECT_ADD], 0,layer,list->data);
+    
+    list = g_list_next(list);
+  }
 }
 
+/** Add a list of objects to the top of a layer.
+ * @param layer The layer to add objects to.
+ * @param obj_list The list of objects to add.  These must not already
+ *  be part of another layer.
+ * @bug Determine if the list is kept by g_list_concat.
+ */
 void
 layer_add_objects_first(Layer *layer, GList *obj_list)
 {
+  GList *list = obj_list;
+  
   layer->objects = g_list_concat(obj_list, layer->objects);
   g_list_foreach(obj_list, set_parent_layer, layer);
+
+  /* Send one signal per object added */
+  while (list != NULL)
+  {
+    /* send a signal that we have added a object to the diagram */
+    g_signal_emit (layer_get_parent_diagram(layer), diagram_data_signals[OBJECT_ADD], 0,layer,list->data);
+    
+    list = g_list_next(list);
+  }
+
 }
 
+/** Remove an object from a layer.
+ * @param layer The layer to remove the object from.
+ * @param obj The object to remove.
+ * @bug Why don't the layer_add functions deal with dynobj?
+ */
 void
 layer_remove_object(Layer *layer, DiaObject *obj)
 {
   layer->objects = g_list_remove(layer->objects, obj);
   dynobj_list_remove_object(obj);
   set_parent_layer(obj, NULL);
+
+  /* send a signal that we have removed a object from the diagram */
+  g_signal_emit (layer_get_parent_diagram(layer), diagram_data_signals[OBJECT_REMOVE], 0,layer,obj);
 }
 
+/** Remove a list of objects from a layer.
+ * @param layer The layer to remove the objects from.
+ * @param obj The objects to remove.
+ * @bug This should call layer_remove_object repeatedly.
+ */
 void
 layer_remove_objects(Layer *layer, GList *obj_list)
 {
@@ -580,10 +843,17 @@ layer_remove_objects(Layer *layer, GList *obj_list)
     obj_list = g_list_next(obj_list);
     dynobj_list_remove_object(obj);
     set_parent_layer(obj, NULL);
+    /* send a signal that we have removed a object from the diagram */
+    g_signal_emit (layer_get_parent_diagram(layer), diagram_data_signals[OBJECT_REMOVE], 0,layer,obj);
   }
 }
 
-
+/** Find the objects that intersect a given rectangle.
+ * @param layer The layer to search in.
+ * @param rect The rectangle to intersect with.
+ * @return List of objects whose bounding box intersect the rectangle.  The
+ *  list should be freed by the caller.
+ */
 GList *
 layer_find_objects_intersecting_rectangle(Layer *layer, Rectangle *rect)
 {
@@ -606,6 +876,12 @@ layer_find_objects_intersecting_rectangle(Layer *layer, Rectangle *rect)
   return selected_list;
 }
 
+/** Find objects entirely contained in a rectangle.
+ * @param layer The layer to search for objects in.
+ * @param rect The rectangle that the objects should be in.
+ * @return A list containing the objects that are entirely contained in the
+ *  rectangle.  The list should be freed by the caller.
+ */
 GList *
 layer_find_objects_in_rectangle(Layer *layer, Rectangle *rect)
 {
@@ -631,6 +907,12 @@ layer_find_objects_in_rectangle(Layer *layer, Rectangle *rect)
 /** Find the object closest to the given point in the layer,
  * no further away than maxdist, and not included in avoid.
  * Stops it if finds an object that includes the point.
+ * @param layer The layer to search in.
+ * @param pos The point to compare to.
+ * @param maxdist The maximum distance the object can be from the point.
+ * @param avoid A list of objects that cannot be returned by this search.
+ * @return The closest object, or NULL if no allowed objects are closer than
+ *  maxdist.
  */
 DiaObject *
 layer_find_closest_object_except(Layer *layer, Point *pos,
@@ -665,6 +947,14 @@ layer_find_closest_object_except(Layer *layer, Point *pos,
   return closest;
 }
 
+/** Find the object closest to the given point in the layer,
+ * no further away than maxdist.
+ * Stops it if finds an object that includes the point.
+ * @param layer The layer to search in.
+ * @param pos The point to compare to.
+ * @param maxdist The maximum distance the object can be from the point.
+ * @return The closest object, or NULL if none are closer than maxdist.
+ */
 DiaObject *
 layer_find_closest_object(Layer *layer, Point *pos, real maxdist)
 {
@@ -672,7 +962,15 @@ layer_find_closest_object(Layer *layer, Point *pos, real maxdist)
 }
 
 
-real layer_find_closest_connectionpoint(Layer *layer,
+/** Find the connectionpoint closest to pos in a layer.
+ * @param layer 
+ * @param closest 
+ * @param pos 
+ * @param notthis 
+ * @return 
+ */
+real 
+layer_find_closest_connectionpoint(Layer *layer,
 					ConnectionPoint **closest,
 					Point *pos,
 					DiaObject *notthis)
