@@ -15,6 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+/** The Element object type is a rectangular box that has
+ *  non-connectable handles on its corners and on the midsts of the
+ *  edges.  It also has connectionpoints in the same places as the
+ *  handles as well as a mainpoint in the middle.
+ */
+
 #include <config.h>
 
 #include <stdio.h>
@@ -25,22 +32,74 @@
 #include "element.h"
 #include "message.h"
 
+/** Update the boundingbox information for this element.
+ * @param An object to update bounding box on.
+ */
 void
 element_update_boundingbox(Element *elem) {
-  Rectangle *bb;
+  Rectangle bb;
   Point *corner;
   ElementBBExtras *extra = &elem->extra_spacing;
 
   assert(elem != NULL);
 
-  bb = &elem->object.bounding_box;
   corner = &elem->corner;
-  bb->left = corner->x - extra->border_trans;
-  bb->right = corner->x + elem->width + extra->border_trans;
-  bb->top = corner->y - extra->border_trans;
-  bb->bottom = corner->y + elem->height + extra->border_trans;
+  bb.left = corner->x;
+  bb.right = corner->x + elem->width;
+  bb.top = corner->y;
+  bb.bottom = corner->y + elem->height;
+  
+  rectangle_bbox(&bb,extra,&elem->object.bounding_box);
 }
 
+/** Update the 9 connections of this element to form a rectangle and
+ * point in the center.
+ * The connections go left-to-right, first top row, then middle row, then
+ * bottom row, then center.  Do not blindly use this in old objects where
+ * the order is different, as it will mess with the saved files.  If an
+ * object uses element_update_handles, it can use this.
+ * @param elem The element to update
+ * @param cps The list of connectionpoints to update, in the order described.
+ *            Usually, this will be the same list as elem->connectionpoints.
+ */
+void
+element_update_connections_rectangle(Element *elem,
+				     ConnectionPoint* cps)
+{
+  cps[0].pos = elem->corner;
+  cps[1].pos.x = elem->corner.x + elem->width / 2.0;
+  cps[1].pos.y = elem->corner.y;
+  cps[2].pos.x = elem->corner.x + elem->width;
+  cps[2].pos.y = elem->corner.y;
+  cps[3].pos.x = elem->corner.x;
+  cps[3].pos.y = elem->corner.y + elem->height / 2.0;
+  cps[4].pos.x = elem->corner.x + elem->width;
+  cps[4].pos.y = elem->corner.y + elem->height / 2.0;
+  cps[5].pos.x = elem->corner.x;
+  cps[5].pos.y = elem->corner.y + elem->height;
+  cps[6].pos.x = elem->corner.x + elem->width / 2.0;
+  cps[6].pos.y = elem->corner.y + elem->height;
+  cps[7].pos.x = elem->corner.x + elem->width;
+  cps[7].pos.y = elem->corner.y + elem->height;
+  g_assert(elem->object.num_connections >= 9);
+  cps[8].pos.x = elem->corner.x + elem->width / 2.0;
+  cps[8].pos.y = elem->corner.y + elem->height / 2.0;
+  
+  cps[0].directions = DIR_NORTH|DIR_WEST;
+  cps[1].directions = DIR_NORTH;
+  cps[2].directions = DIR_NORTH|DIR_EAST;
+  cps[3].directions = DIR_WEST;
+  cps[4].directions = DIR_EAST;
+  cps[5].directions = DIR_SOUTH|DIR_WEST;
+  cps[6].directions = DIR_SOUTH;
+  cps[7].directions = DIR_SOUTH|DIR_EAST;
+  cps[8].directions = DIR_ALL;
+}
+
+/** Update the corner and edge handles of an element to reflect its position
+ *  and size.
+ * @param elem An element to update.
+ */
 void
 element_update_handles(Element *elem)
 {
@@ -79,9 +138,21 @@ element_update_handles(Element *elem)
   elem->resize_handles[7].pos.y = corner->y + elem->height;
 }
 
-void
+/** Handle the moving of one of the elements handles.
+ *  This function is suitable for use as the move_handle object operation.
+ * @param elem The element whose handle is being moved.
+ * @param id The id of the handle.
+ * @param to Where it's being moved to.
+ * @param cp Ignored
+ * @param reason What is causing this handle to be moved (creation, movement..)
+ * @param modifiers Any modifier keys (shift, control...) that the user is
+ *                  pressing.
+ * @return Undo information for this change.
+ */
+ObjectChange*
 element_move_handle(Element *elem, HandleId id,
-		    Point *to, HandleMoveReason reason)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p;
   Point *corner;
@@ -150,8 +221,16 @@ element_move_handle(Element *elem, HandleId id,
   default:
     message_error("Error, called element_move_handle() with wrong handle-id\n");
   }
+  return NULL;
 }
 
+/** Move the handle of an element restricted to a certain aspect ration.
+ * @param elem The element to update on
+ * @param id The id of the handle being moved
+ * @param to Where the handle is being moved to
+ * @param aspect_ratio The aspect ratio (width:height) to obey.
+ *                     The aspect ratio must not be 0.
+ */
 void
 element_move_handle_aspect(Element *elem, HandleId id,
 			   Point *to, real aspect_ratio)
@@ -245,11 +324,20 @@ element_move_handle_aspect(Element *elem, HandleId id,
   elem->height = new_height;
 }
 
-/* Needs to have the first 8 handles */
+/** Initialization function for element objects.
+ *  An element must have at least 8 handles and 9 connectionpoints.
+ * @param elem The element to initialize.  This function will call
+ *             object_init() on the element.
+ * @param num_handles The number of handles to set up (>= 8).  The handles
+ *                    will be initialized by this function.
+ * @param num_connections The number of connectionpoints to set up (>= 9).
+ *                        The connectionpoints will <em>not</em> be
+ *                        initialized by this function.
+ */
 void
 element_init(Element *elem, int num_handles, int num_connections)
 {
-  Object *obj;
+  DiaObject *obj;
   int i;
 
   obj = &elem->object;
@@ -266,10 +354,15 @@ element_init(Element *elem, int num_handles, int num_connections)
   }
 }
 
+/** Copy an element, initializing the handles.
+ *  This function will in turn copy the underlying object.
+ * @paramm from An element to copy from.
+ * @param to An element (already allocated) to copy to.
+ */
 void
 element_copy(Element *from, Element *to)
 {
-  Object *toobj, *fromobj;
+  DiaObject *toobj, *fromobj;
   int i;
 
   fromobj = &from->object;
@@ -289,14 +382,23 @@ element_copy(Element *from, Element *to)
   memcpy(&to->extra_spacing,&from->extra_spacing,sizeof(to->extra_spacing));
 }
 
+/** Destroy an elements private information.
+ *  This function will in turn call object_destroy.
+ * @param elem The element to destroy.  It will <em>not</em> be deallocated
+ *             by this call, but will not be valid afterwards.
+ */
 void
 element_destroy(Element *elem)
 {
   object_destroy(&elem->object);
 }
 
-
-void element_save(Element *elem, ObjectNode obj_node)
+/** Save the element-specific parts of this element to XML.
+ * @param elem
+ * @param obj_node
+ */
+void 
+element_save(Element *elem, ObjectNode obj_node)
 {
   object_save(&elem->object, obj_node);
 
