@@ -15,23 +15,28 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+/* ** \file element.h -- Definition of a diagram - usual rectangular - object with eight handles  */
 #ifndef ELEMENT_H
 #define ELEMENT_H
 
+#include "diatypes.h"
 #include "object.h"
 #include "handle.h"
 #include "connectionpoint.h"
 #include "boundingbox.h"
 
-typedef struct _Element Element;
-
-/* This is a subclass of Object used to help implementing objects
- * of a type with 8 handles around ..... more info here. */
+/*!
+ * \class _Element
+ * \brief Beside OrthCon one of the most use object classes
+ *
+ * This is a subclass of DiaObject used to help implementing objects
+ * of a type with 8 handles. 
+ */
 struct _Element {
-  /* Object must be first because this is a 'subclass' of it. */
-  Object object;
+  DiaObject object; /*!< inheritance */
   
-  Handle resize_handles[8];
+  Handle resize_handles[8]; /*!< not only for resizing but may also be used for connections */
 
   Point corner;
   real width;
@@ -41,12 +46,16 @@ struct _Element {
 };
 
 void element_update_handles(Element *elem);
+void element_update_connections_rectangle(Element *elem,
+					  ConnectionPoint* cps);
 void element_update_boundingbox(Element *elem);
 void element_init(Element *elem, int num_handles, int num_connections);
 void element_destroy(Element *elem);
 void element_copy(Element *from, Element *to);
-void element_move_handle(Element *elem, HandleId id,
-			 Point *to, HandleMoveReason reason);
+ObjectChange* element_move_handle(Element *elem, HandleId id,
+				  Point *to, ConnectionPoint *cp,
+				  HandleMoveReason reason, 
+				  ModifierKeys modifiers);
 void element_move_handle_aspect(Element *elem, HandleId id,
 				Point *to, real aspect_ratio);
 
@@ -54,15 +63,33 @@ void element_save(Element *elem, ObjectNode obj_node);
 void element_load(Element *elem, ObjectNode obj_node);
 
 /* base property stuff ... */
+static PropNumData width_range = { -G_MAXFLOAT, G_MAXFLOAT, 0.1};
+
 #define ELEMENT_COMMON_PROPERTIES \
   OBJECT_COMMON_PROPERTIES, \
   { "elem_corner", PROP_TYPE_POINT, 0, \
     "Element corner", "The corner of the element"}, \
-  { "elem_width", PROP_TYPE_REAL, 0, \
-    "Element width", "The width of the element"}, \
-  { "elem_height", PROP_TYPE_REAL, 0, \
-    "Element height", "The height of the element"}
+  { "elem_width", PROP_TYPE_REAL, PROP_FLAG_DONT_MERGE, \
+    "Element width", "The width of the element", &width_range}, \
+  { "elem_height", PROP_TYPE_REAL, PROP_FLAG_DONT_MERGE, \
+    "Element height", "The height of the element", &width_range}
 
+   /* Would like to have the frame, but need to figure out why
+      custom_object ext_attributes lose their updates when they're on
+      (see http://mail.gnome.org/archives/dia-list/2005-August/msg00053.html)
+   */
+      /*
+#define ELEMENT_COMMON_PROPERTIES \
+  OBJECT_COMMON_PROPERTIES, \
+  PROP_FRAME_BEGIN("size",0,N_("Object dimensions")), \
+  { "elem_corner", PROP_TYPE_POINT, 0, \
+    "Element corner", "The corner of the element"}, \
+  { "elem_width", PROP_TYPE_REAL, PROP_FLAG_VISIBLE, \
+    "Element width", "The width of the element", &width_range}, \
+  { "elem_height", PROP_TYPE_REAL, PROP_FLAG_VISIBLE, \
+    "Element height", "The height of the element", &width_range}, \
+  PROP_FRAME_END("size", 0)
+      */
 #define ELEMENT_COMMON_PROPERTIES_OFFSETS \
   OBJECT_COMMON_PROPERTIES_OFFSETS, \
   { "elem_corner", PROP_TYPE_POINT, offsetof(Element, corner) }, \
