@@ -15,6 +15,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+/** \file geometry.h -- basic geometry classes and functions operationg on them */
 #ifndef GEOMETRY_H
 #define GEOMETRY_H
 
@@ -73,33 +75,40 @@
 typedef double real;
 typedef real coord;
 
+/*! \brief A two dimensional position */
 struct _Point {
-  coord x;
-  coord y;
+  coord x; /*!< horizontal */
+  coord y; /*!< vertical */
 };
 
+/*! \brief A rectangle given by upper left and lower right corner */
 struct _Rectangle {
-  coord top;
-  coord left;
-  coord bottom;
-  coord right;
+  coord top; /*!< y1 */
+  coord left; /*!< x1 */
+  coord bottom; /*!< y2 */
+  coord right; /*!< x2 */
 };
 
+/*! \brief A rectangle for fixed point e.g. pixel coordinates */
 struct _IntRectangle {
-  int top;
-  int left;
-  int bottom;
-  int right;
+  int top; /*!< y1 */
+  int left; /*!< x1 */
+  int bottom; /*!< y2 */
+  int right; /*!< x2 */
 };
 
-/* BezPoint types:
- *   BEZ_MOVE_TO: move to point p1
- *   BEZ_LINE_TO: line to point p1
- *   BEZ_CURVE_TO: curve to point p3 using p1 and p2 as control points.
+/*! 
+ * \brief BezPoint is a bezier point forming _Bezierline or _Beziergon
  */
 struct _BezPoint {
-  enum {BEZ_MOVE_TO, BEZ_LINE_TO, BEZ_CURVE_TO} type;
-  Point p1, p2, p3;
+  enum {
+    BEZ_MOVE_TO, /*!< move to point p1 */
+    BEZ_LINE_TO, /*!< line to point p1 */
+    BEZ_CURVE_TO /*!< curve to point p3 using p1 and p2 as control points */
+  } type;
+  Point p1; /*!< main point in case of move or line-to, otherwise first control point */
+  Point p2; /*!< second control point  */
+  Point p3; /*!< main point for 'true' bezier point */
 };
 
 
@@ -279,42 +288,6 @@ rectangle_equals(const Rectangle *r1, const Rectangle *r2)
 }
 #endif
 
-G_INLINE_FUNC void check_bb_x( Rectangle *bb, real val, real check );
-#ifdef G_CAN_INLINE
-G_INLINE_FUNC void 
-check_bb_x( Rectangle *bb, real val, real check )
-{
-  /* if x is outside the bounding box, and check is real, adjust the bb */
-  if (finite(check)) { 
-    if (bb->left > val) bb->left = val; 
-    if (bb->right < val) bb->right = val; 
-  }
-}
-#endif
-
-G_INLINE_FUNC void check_bb_y( Rectangle *bb, real val, real check );
-#ifdef G_CAN_INLINE
-G_INLINE_FUNC void 
-check_bb_y( Rectangle *bb, real val, real check )
-{
-  /* if y is outside the bounding box, and check is real, adjust the bb */
-  if (finite(check)) { 
-    if (bb->top > val) bb->top = val; 
-    if (bb->bottom < val) bb->bottom = (val); 
-  } 
-}
-#endif
-
-G_INLINE_FUNC void check_bb_pt( Rectangle *bb, const Point *pt, real check );
-#ifdef G_CAN_INLINE
-G_INLINE_FUNC void 
-check_bb_pt( Rectangle *bb, const Point *pt, real check )
-{
-  /* if pt is outside the bounding box, and check is real, adjust the bb */
-  if (finite(check)) rectangle_add_point(bb,pt);
-}
-#endif
-
 G_INLINE_FUNC real distance_point_point(const Point *p1, const Point *p2);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC real
@@ -326,6 +299,17 @@ distance_point_point(const Point *p1, const Point *p2)
 }
 #endif
 
+G_INLINE_FUNC const Point *closest_to(const Point *to, const Point *p1, const Point *p2);
+#ifdef G_CAN_INLINE
+G_INLINE_FUNC const Point *
+closest_to(const Point *to, const Point *p1, const Point *p2)
+{
+  if (distance_point_point(to,p1) < distance_point_point(to,p2))
+          return p1;
+  return p2;
+}
+#endif
+
 G_INLINE_FUNC real distance_point_point_manhattan(const Point *p1, 
                                                   const Point *p2);
 #ifdef G_CAN_INLINE
@@ -376,5 +360,6 @@ void point_perp(Point *p, real a, real b, real c, Point *perp);
 void fillet(Point *p1, Point *p2, Point *p3, Point *p4,
 	    real r, Point *c, real *pa, real *aa);
 real point_cross(Point *p1, Point *p2);
+Point calculate_object_edge(Point *objmid, Point *end, DiaObject *obj);
 
 #endif /* GEOMETRY_H */
