@@ -30,6 +30,7 @@
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkcolorseldialog.h>
+#include <gtk/gtkmenuitem.h>
 
 #include "diatypes.h"
 
@@ -95,7 +96,6 @@ struct _DiaColorSelector
 
   GtkWidget *area;
   GdkGC *gc;
-  Color col;
 
   GtkWidget *col_sel;
   
@@ -108,8 +108,8 @@ struct _DiaColorSelectorClass
 
 GtkType    dia_color_selector_get_type  (void);
 GtkWidget* dia_color_selector_new       (void);
-void       dia_color_selector_get_color (DiaColorSelector *cs, Color *color);
-void       dia_color_selector_set_color (DiaColorSelector *cs,
+void       dia_color_selector_get_color (GtkWidget *cs, Color *color);
+void       dia_color_selector_set_color (GtkWidget *cs,
 					 const Color *color);
 
 
@@ -155,17 +155,123 @@ void       dia_size_selector_set_locked(DiaSizeSelector *ss, gboolean locked);
 void       dia_size_selector_set_size        (DiaSizeSelector *ss, real width, real height);
 gboolean dia_size_selector_get_size        (DiaSizeSelector *ss, real *width, real *height);
 
+
+/* DiaUnitSpinner */
+#define DIA_UNIT_SPINNER(obj) GTK_CHECK_CAST(obj, dia_unit_spinner_get_type(), DiaUnitSpinner)
+#define DIA_UNIT_SPINNER_CLASS(klass) GTK_CHECK_CLASS_CAST(klass, dia_unit_spinner_get_type(), DiaUnitSpinnerClass)
+#define DIA_IS_UNIT_SPINNER(obj) GTK_CHECK_TYPE(obj, dia_unit_spinner_get_type())
+
+typedef struct _DiaUnitSpinner DiaUnitSpinner;
+typedef struct _DiaUnitSpinnerClass DiaUnitSpinnerClass;
+
+typedef enum {
+  DIA_UNIT_FEET,
+  DIA_UNIT_METER,
+  DIA_UNIT_DECIMETER,
+  DIA_UNIT_MILLIMETER,
+  DIA_UNIT_POINT,
+  DIA_UNIT_CENTIMETER,
+  DIA_UNIT_INCH,
+} DiaUnit;
+
+struct _DiaUnitSpinner {
+  GtkSpinButton parent;
+
+  DiaUnit unit_num;
+};
+
+struct _DiaUnitSpinnerClass {
+  GtkSpinButtonClass parent_class;
+
+};
+
+GtkType    dia_unit_spinner_get_type  (void);
+GtkWidget *dia_unit_spinner_new       (GtkAdjustment *adjustment,
+				       guint digits,
+				       DiaUnit adj_unit);
+void       dia_unit_spinner_set_value (DiaUnitSpinner *self, gfloat val);
+gfloat     dia_unit_spinner_get_value (DiaUnitSpinner *self);
+
+/* DiaDynamicMenu */
+
+#define DIA_DYNAMIC_MENU(obj) GTK_CHECK_CAST(obj, dia_dynamic_menu_get_type(), DiaDynamicMenu)
+#define DIA_DYNAMIC_MENU_CLASS(klass) GTK_CHECK_CLASS_CAST(klass, dia_dynamic_menu_get_type(), DiaDynamicMenuClass)
+#define DIA_IS_DYNAMIC_MENU(obj) GTK_CHECK_TYPE(obj, dia_dynamic_menu_get_type())
+
+typedef struct _DiaDynamicMenu DiaDynamicMenu;
+typedef struct _DiaDynamicMenuClass DiaDynamicMenuClass;
+
+/** The ways the non-default entries in the menu can be sorted:
+ * DDM_SORT_TOP: Just add new ones at the top, removing them from the middle.
+ * Not currently implemented.
+ * DDM_SORT_NEWEST:  Add new ones to the top, and move selected ones to the
+ * top as well.
+ * DDM_SORT_SORT:  Sort the entries according to the CompareFunc order.
+ */
+typedef enum { DDM_SORT_TOP, DDM_SORT_NEWEST, DDM_SORT_SORT } DdmSortType;
+
+typedef GtkWidget *(* DDMCreateItemFunc)(DiaDynamicMenu *, gchar *);
+typedef void (* DDMCallbackFunc)(DiaDynamicMenu *, const gchar *, gpointer);
+
+struct _DiaDynamicMenu {
+  GtkOptionMenu parent;
+
+  GList *default_entries;
+
+  DDMCreateItemFunc create_func;
+  DDMCallbackFunc activate_func;
+  gpointer userdata;
+
+  GtkMenuItem *other_item;
+
+  gchar *persistent_name;
+  gint cols;
+
+  gchar *active;
+  /** For the list-based versions, these are the options */
+  GList *options;
+
+};
+
+struct _DiaDynamicMenuClass {
+  GtkOptionMenuClass parent_class;
+};
+
+GtkType    dia_dynamic_menu_get_type  (void);
+
+GtkWidget *dia_dynamic_menu_new(DDMCreateItemFunc create,
+				DDMCallbackFunc activate, gpointer userdata,
+				GtkMenuItem *otheritem, gchar *persist);
+GtkWidget *dia_dynamic_menu_new_stringbased(GtkMenuItem *otheritem, 
+					    DDMCallbackFunc activate,
+					    gpointer userdata,
+					    gchar *persist);
+GtkWidget *dia_dynamic_menu_new_listbased(DDMCreateItemFunc create,
+					  DDMCallbackFunc activate,
+					  gpointer userdata,
+					  gchar *other_label,
+					  GList *items, gchar *persist);
+GtkWidget *dia_dynamic_menu_new_stringlistbased(gchar *other_label,
+						GList *items, 
+						DDMCallbackFunc activate,
+						gpointer userdata,
+						gchar *persist);
+void dia_dynamic_menu_add_default_entry(DiaDynamicMenu *ddm, const gchar *entry);
+gint dia_dynamic_menu_add_entry(DiaDynamicMenu *ddm, const gchar *entry);
+void dia_dynamic_menu_set_sorting_method(DiaDynamicMenu *ddm, DdmSortType sort);
+void dia_dynamic_menu_reset(GtkWidget *widget, gpointer userdata);
+void dia_dynamic_menu_set_max_entries(DiaDynamicMenu *ddm, gint max);
+void dia_dynamic_menu_set_columns(DiaDynamicMenu *ddm, gint cols);
+gchar *dia_dynamic_menu_get_entry(DiaDynamicMenu *ddm);
+void dia_dynamic_menu_select_entry(DiaDynamicMenu *ddm, const gchar *entry);
+
 /* **** Util functions for Gtk stuff **** */
-/** Gets the image name 'filename' out of the shared data dir 
- * To see how to get the image installed, check app/Makefile.am
+/** Create a toggle button with two icons (created with gdk-pixbuf-csource,
+ * for instance).  The icons represent on and off.
  */
-GtkWidget *dia_get_image_from_file(gchar *filename);
-
-/** Create a toggle button with two different images, one for when on and
- * one for when off.  Images are taken from standard image directory.
- */
-GtkWidget *dia_toggle_button_new_with_images(gchar *on_file, gchar *off_file);
-
+GtkWidget *
+dia_toggle_button_new_with_icons(const guint8 *on_icon,
+				 const guint8 *off_icon);
 
 /* Other common defaults */
 
