@@ -16,12 +16,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
@@ -29,7 +30,7 @@
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "arrows.h"
 #include "diamenu.h"
@@ -44,8 +45,6 @@ Color flow_color_material = { 0.8f, 0.0f, 0.8f };
 Color flow_color_signal   = { 0.0f, 0.0f, 1.0f };
 
 typedef struct _Flow Flow;
-typedef struct _FlowDialog FlowDialog;
-
 typedef enum {
   FLOW_ENERGY,
   FLOW_MATERIAL,
@@ -58,17 +57,9 @@ struct _Flow {
   Handle text_handle;
 
   Text* text;
+  TextAttributes attrs;
   FlowType type;
-};
-
-struct _FlowDialog {
-  GtkWidget *dialog;
-  
-  GtkWidget *text;
-
-  GtkWidget *m_energy;
-  GtkWidget *m_material;
-  GtkWidget *m_signal;
+  Point textpos; /* This is the master position, but overridden in load */
 };
   
 #define FLOW_WIDTH 0.1
@@ -79,41 +70,33 @@ struct _FlowDialog {
 #define FLOW_ARROWWIDTH 0.5
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM1)
 
-static Font *flow_font = NULL;
+static DiaFont *flow_font = NULL;
 
-static FlowDialog *defaults_dialog;
-
-/* Remember the most recently applied flow type and use it to
-   set the type for any newly created flows
- */
-static FlowType flow_most_recent_type = FLOW_ENERGY ;
-static Text* flow_default_label = 0 ;
-
-static void flow_move_handle(Flow *flow, Handle *handle,
-			     Point *to, HandleMoveReason reason);
-static void flow_move(Flow *flow, Point *to);
+static ObjectChange* flow_move_handle(Flow *flow, Handle *handle,
+				      Point *to, ConnectionPoint *cp,
+				      HandleMoveReason reason, 
+				      ModifierKeys modifiers);
+static ObjectChange* flow_move(Flow *flow, Point *to);
 static void flow_select(Flow *flow, Point *clicked_point,
-			Renderer *interactive_renderer);
-static void flow_draw(Flow *flow, Renderer *renderer);
-static Object *flow_create(Point *startpoint,
+			DiaRenderer *interactive_renderer);
+static void flow_draw(Flow *flow, DiaRenderer *renderer);
+static DiaObject *flow_create(Point *startpoint,
 			   void *user_data,
 			   Handle **handle1,
 			   Handle **handle2);
 static real flow_distance_from(Flow *flow, Point *point);
 static void flow_update_data(Flow *flow);
 static void flow_destroy(Flow *flow);
-static Object *flow_copy(Flow *flow);
-static GtkWidget *flow_get_defaults(void);
-static void flow_apply_defaults(void);
+static DiaObject *flow_copy(Flow *flow);
 static void flow_save(Flow *flow, ObjectNode obj_node,
 		      const char *filename);
-static Object *flow_load(ObjectNode obj_node, int version,
+static DiaObject *flow_load(ObjectNode obj_node, int version,
 			 const char *filename);
 static PropDescription *flow_describe_props(Flow *mes);
 static void
-flow_get_props(Flow * flow, Property *props, guint nprops);
+flow_get_props(Flow * flow, GPtrArray *props);
 static void
-flow_set_props(Flow * flow, Property *props, guint nprops);
+flow_set_props(Flow * flow, GPtrArray *props);
 static DiaMenu *flow_get_object_menu(Flow *flow, Point *clickedpoint) ;
 
 
@@ -122,11 +105,11 @@ static ObjectTypeOps flow_type_ops =
   (CreateFunc)		flow_create,
   (LoadFunc)		flow_load,
   (SaveFunc)		flow_save,
-  (GetDefaultsFunc)	flow_get_defaults,
-  (ApplyDefaultsFunc)	flow_apply_defaults
+  (GetDefaultsFunc)	NULL,
+  (ApplyDefaultsFunc)	NULL,
 } ;
 
-ObjectType flow_type =
+DiaObjectType flow_type =
 {
   "FS - Flow",         /* name */
   0,                   /* version */
@@ -161,6 +144,12 @@ static PropDescription flow_props[] = {
   OBJECT_COMMON_PROPERTIES,
   { "type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
     N_("Type:"), NULL, prop_flow_type_data },
+  { "text", PROP_TYPE_TEXT, 0, NULL, NULL },
+  PROP_STD_TEXT_ALIGNMENT,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  /* Colour determined from type, don't show */
+  { "text_colour", PROP_TYPE_COLOUR, PROP_FLAG_DONT_SAVE, },
   PROP_DESC_END
 };
 
@@ -176,21 +165,28 @@ flow_describe_props(Flow *mes)
 static PropOffset flow_offsets[] = {
   OBJECT_COMMON_PROPERTIES_OFFSETS,
   { "type", PROP_TYPE_ENUM, offsetof(Flow, type) },
+  { "text", PROP_TYPE_TEXT, offsetof (Flow, text) },
+  { "text_alignment", PROP_TYPE_ENUM, offsetof (Flow, attrs.alignment) },
+  { "text_font", PROP_TYPE_FONT, offsetof (Flow, attrs.font) },
+  { "text_height", PROP_TYPE_REAL, offsetof (Flow, attrs.height) },
+  { "text_colour", PROP_TYPE_COLOUR, offsetof (Flow, attrs.color) },
   { NULL, 0, 0 }
 };
 
 static void
-flow_get_props(Flow * flow, Property *props, guint nprops)
+flow_get_props(Flow * flow, GPtrArray *props)
 {
+  text_get_attributes (flow->text, &flow->attrs);
   object_get_props_from_offsets(&flow->connection.object, 
-                                flow_offsets, props, nprops);
+                                flow_offsets, props);
 }
 
 static void
-flow_set_props(Flow *flow, Property *props, guint nprops)
+flow_set_props(Flow *flow, GPtrArray *props)
 {
   object_set_props_from_offsets(&flow->connection.object, 
-                                flow_offsets, props, nprops);
+                                flow_offsets, props);
+  apply_textattr_properties (props, flow->text, "text", &flow->attrs);
   flow_update_data(flow);
 }
 
@@ -214,7 +210,7 @@ flow_distance_from(Flow *flow, Point *point)
 
 static void
 flow_select(Flow *flow, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   text_set_cursor(flow->text, clicked_point, interactive_renderer);
   text_grab_focus(flow->text, &flow->connection.object);
@@ -222,9 +218,10 @@ flow_select(Flow *flow, Point *clicked_point,
   connection_update_handles(&flow->connection);
 }
 
-static void
+static ObjectChange*
 flow_move_handle(Flow *flow, Handle *handle,
-		 Point *to, HandleMoveReason reason)
+		 Point *to, ConnectionPoint *cp,
+		 HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p1, p2;
   Point *endpoints;
@@ -234,7 +231,7 @@ flow_move_handle(Flow *flow, Handle *handle,
   assert(to!=NULL);
 
   if (handle->id == HANDLE_MOVE_TEXT) {
-    flow->text->position = *to;
+    flow->textpos = *to;
   } else  {
     real dest_length ;
     real orig_length2 ;
@@ -242,7 +239,7 @@ flow_move_handle(Flow *flow, Handle *handle,
     Point along ;
 
     endpoints = &flow->connection.endpoints[0]; 
-    p1 = flow->text->position ;
+    p1 = flow->textpos ;
     point_sub( &p1, &endpoints[0] ) ;
 
     p2 = endpoints[1] ;
@@ -262,11 +259,12 @@ flow_move_handle(Flow *flow, Handle *handle,
       norm_mag = (real)sqrt( (double) point_dot( &p1, &p1 ) ) ;
     }
 
-    connection_move_handle(&flow->connection, handle->id, to, reason);
+    connection_move_handle(&flow->connection, handle->id, to, cp, 
+			   reason, modifiers);
 
     p2 = endpoints[1] ;
     point_sub( &p2, &endpoints[0] ) ;
-    flow->text->position = endpoints[0] ;
+    flow->textpos = endpoints[0] ;
     along = p2 ;
     p2.x = -along.y ;
     p2.y = along.x ;
@@ -278,14 +276,16 @@ flow_move_handle(Flow *flow, Handle *handle,
     }
     point_scale( &p2, norm_mag ) ;
     point_scale( &along, along_mag ) ;
-    point_add( &flow->text->position, &p2 ) ;
-    point_add( &flow->text->position, &along ) ;
+    point_add( &flow->textpos, &p2 ) ;
+    point_add( &flow->textpos, &along ) ;
   }
 
   flow_update_data(flow);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 flow_move(Flow *flow, Point *to)
 {
   Point start_to_end;
@@ -301,66 +301,66 @@ flow_move(Flow *flow, Point *to)
   endpoints[1] = endpoints[0] = *to;
   point_add(&endpoints[1], &start_to_end);
 
-  point_add(&flow->text->position, &delta);
-  
+  point_add(&flow->textpos, &delta);
+
   flow_update_data(flow);
+
+  return NULL;
 }
 
 static void
-flow_draw(Flow *flow, Renderer *renderer)
+flow_draw(Flow *flow, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point *endpoints, p1, p2;
-  ArrowType arrow_type;
+  Arrow arrow;
   int n1 = 1, n2 = 0;
   Color* render_color = NULL;
 
   assert(flow != NULL);
   assert(renderer != NULL);
 
-  arrow_type = ARROW_FILLED_TRIANGLE;
+  arrow.type = ARROW_FILLED_TRIANGLE;
+  arrow.width = FLOW_ARROWWIDTH;
+  arrow.length = FLOW_ARROWLEN;
  
   endpoints = &flow->connection.endpoints[0];
   
-  renderer->ops->set_linewidth(renderer, FLOW_WIDTH);
-
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, FLOW_WIDTH);
 
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   switch (flow->type) {
   case FLOW_SIGNAL:
-    renderer->ops->set_dashlength(renderer, FLOW_DASHLEN);
-    renderer->ops->set_linestyle(renderer, LINESTYLE_DASHED);
+    renderer_ops->set_dashlength(renderer, FLOW_DASHLEN);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
     render_color = &flow_color_signal ;
     break ;
   case FLOW_MATERIAL:
-    renderer->ops->set_linewidth(renderer, FLOW_MATERIAL_WIDTH ) ;
-    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    renderer_ops->set_linewidth(renderer, FLOW_MATERIAL_WIDTH ) ;
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
     render_color = &flow_color_material ;
     break ;
   case FLOW_ENERGY:
     render_color = &flow_color_energy ;
-    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
   }
 
   p1 = endpoints[n1];
   p2 = endpoints[n2];
 
-  renderer->ops->draw_line(renderer,
-			   &p1, &p2,
-			   render_color); 
+  renderer_ops->draw_line_with_arrows(renderer,
+				       &p1, &p2,
+				       FLOW_WIDTH,
+				       render_color,
+				       &arrow, NULL); 
 
-  arrow_draw(renderer, arrow_type,
-	     &p1, &p2,
-	     FLOW_ARROWLEN, FLOW_ARROWWIDTH, FLOW_WIDTH,
-	     render_color, &color_white);
-
-  renderer->ops->set_font(renderer, flow_font,
-			  FLOW_FONTHEIGHT);
+  renderer_ops->set_font(renderer, flow_font,FLOW_FONTHEIGHT);
 
   text_draw(flow->text, renderer);
 }
 
-static Object *
+static DiaObject *
 flow_create(Point *startpoint,
 	    void *user_data,
 	    Handle **handle1,
@@ -368,10 +368,11 @@ flow_create(Point *startpoint,
 {
   Flow *flow;
   Connection *conn;
-  Object *obj;
-  ConnectionBBExtras *extra;
+  DiaObject *obj;
+  LineBBExtras *extra;
   Point p ;
   Point n ;
+  DiaFont *font;
 
   flow = g_malloc0(sizeof(Flow));
 
@@ -388,8 +389,6 @@ flow_create(Point *startpoint,
   
   connection_init(conn, 3, 0);
 
-  flow->type = flow_most_recent_type ;
-
   p = conn->endpoints[1] ;
   point_sub( &p, &conn->endpoints[0] ) ;
   point_scale( &p, 0.5 ) ;
@@ -402,47 +401,29 @@ flow_create(Point *startpoint,
     point_normalize( &n ) ;
   }
   point_scale( &n, 0.5*FLOW_FONTHEIGHT ) ;
+
   point_add( &p, &n ) ;
   point_add( &p, &conn->endpoints[0] ) ;
+  flow->textpos = p;
 
-  if ( flow_default_label ) {
-    flow->text = text_copy( flow_default_label ) ;
-    text_set_position( flow->text, &p ) ;
-  } else {
-    Color* color = NULL;
+  font = dia_font_new_from_style(DIA_FONT_SANS, 0.8);
 
-    if (flow_font == NULL)
-      flow_font = font_getfont("Helvetica-Oblique");
-
-    switch (flow->type) {
-    case FLOW_ENERGY:
-      color = &flow_color_energy ;
-      break ;
-    case FLOW_MATERIAL:
-      color = &flow_color_material ;
-      break ;
-    case FLOW_SIGNAL:
-      color = &flow_color_signal ;
-      break ;
-    }
-
-    flow->text = new_text("", flow_font, FLOW_FONTHEIGHT, 
-			  &p, color, ALIGN_CENTER);
-  }
+  flow->text = new_text("", font, 0.8, &p, &color_black, ALIGN_CENTER);
+  dia_font_unref(font);  
+  text_get_attributes(flow->text, &flow->attrs);
 
   flow->text_handle.id = HANDLE_MOVE_TEXT;
   flow->text_handle.type = HANDLE_MINOR_CONTROL;
   flow->text_handle.connect_type = HANDLE_NONCONNECTABLE;
   flow->text_handle.connected_to = NULL;
+  flow->text_handle.pos = p;
   obj->handles[2] = &flow->text_handle;
 
   extra->start_long = 
     extra->end_long =
     extra->start_trans = FLOW_WIDTH/2.0;
   extra->end_trans = MAX(FLOW_WIDTH, FLOW_ARROWLEN) / 2.0;
-  
   flow_update_data(flow);
-
   *handle1 = obj->handles[0];
   *handle2 = obj->handles[1];
   return &flow->connection.object;
@@ -456,12 +437,12 @@ flow_destroy(Flow *flow)
   text_destroy(flow->text) ;
 }
 
-static Object *
+static DiaObject *
 flow_copy(Flow *flow)
 {
   Flow *newflow;
   Connection *conn, *newconn;
-  Object *newobj;
+  DiaObject *newobj;
   
   conn = &flow->connection;
   
@@ -473,7 +454,7 @@ flow_copy(Flow *flow)
 
   newflow->text_handle = flow->text_handle;
   newobj->handles[2] = &newflow->text_handle;
-
+  newflow->textpos = flow->textpos;
   newflow->text = text_copy(flow->text);
   newflow->type = flow->type;
 
@@ -485,10 +466,14 @@ static void
 flow_update_data(Flow *flow)
 {
   Connection *conn = &flow->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
   Rectangle rect;
   Color* color = NULL;
   
+  if (connpoint_is_autogap(flow->connection.endpoint_handles[0].connected_to) ||
+      connpoint_is_autogap(flow->connection.endpoint_handles[1].connected_to)) {
+    connection_adjust_for_autogap(flow);
+  }
   obj->position = conn->endpoints[0];
 
   switch (flow->type) {
@@ -504,7 +489,8 @@ flow_update_data(Flow *flow)
   }
   text_set_color( flow->text, color ) ;
 
-  flow->text_handle.pos = flow->text->position;
+  flow->text->position = flow->textpos;
+  flow->text_handle.pos = flow->textpos;
 
   connection_update_handles(conn);
 
@@ -528,17 +514,19 @@ flow_save(Flow *flow, ObjectNode obj_node, const char *filename)
 	       flow->type);
 }
 
-static Object *
+static DiaObject *
 flow_load(ObjectNode obj_node, int version, const char *filename)
 {
   Flow *flow;
   AttributeNode attr;
   Connection *conn;
-  Object *obj;
-  ConnectionBBExtras *extra;
+  DiaObject *obj;
+  LineBBExtras *extra;
 
-  if (flow_font == NULL)
-    flow_font = font_getfont("Helvetica-Oblique");
+  if (flow_font == NULL) {
+	  flow_font = dia_font_new_from_style (DIA_FONT_SANS|DIA_FONT_ITALIC,
+                                         FLOW_FONTHEIGHT);
+  }
 
   flow = g_malloc0(sizeof(Flow));
 
@@ -573,63 +561,16 @@ flow_load(ObjectNode obj_node, int version, const char *filename)
     extra->start_trans = FLOW_WIDTH/2.0;
   extra->end_trans = MAX(FLOW_WIDTH, FLOW_ARROWLEN) / 2.0;
   
+  flow->textpos = flow->text->position;
   flow_update_data(flow);
   
   return &flow->connection.object;
 }
 
-static void
-fill_in_defaults_dialog(void)
-{
-  FlowDialog *prop_dialog;
-  GtkToggleButton *button=NULL;
-
-  prop_dialog = defaults_dialog;
-
-  if ( flow_default_label ) {
-    gtk_text_set_point( GTK_TEXT(prop_dialog->text), 0 ) ;
-    gtk_text_forward_delete( GTK_TEXT(prop_dialog->text), 
-			     gtk_text_get_length(GTK_TEXT(prop_dialog->text))) ;
-    gtk_text_insert( GTK_TEXT(prop_dialog->text),
-		     NULL, NULL, NULL,
-		     text_get_string_copy(flow_default_label),
-		     -1) ;
-  }
-
-  switch (flow_most_recent_type) {
-  case FLOW_ENERGY:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_energy);
-    break;
-  case FLOW_MATERIAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_material);
-    break;
-  case FLOW_SIGNAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_signal);
-    break;
-  }
-  if (button)
-    gtk_toggle_button_set_active(button, TRUE);
-}
-
-static void
-flow_update_defaults( Flow* flow, char update_text )
-{
-  flow_most_recent_type = flow->type ;
-
-  if (update_text) {
-    if ( flow_default_label )
-      text_destroy( flow_default_label ) ;
-    flow_default_label = text_copy( flow->text ) ;
-  }
-}
-
 static ObjectChange *
-flow_set_type_callback (Object* obj, Point* clicked, gpointer data)
+flow_set_type_callback (DiaObject* obj, Point* clicked, gpointer data)
 {
   ((Flow*)obj)->type = (int) data ;
-  flow_update_defaults( (Flow*) obj, 1 ) ;
-  if ( defaults_dialog )
-    fill_in_defaults_dialog() ;
   flow_update_data((Flow*)obj);
 
   return NULL;
@@ -655,109 +596,3 @@ flow_get_object_menu(Flow *flow, Point *clickedpoint)
   flow_menu_items[0].active = 1;
   return &flow_menu;
 }
-
-static GtkWidget *
-flow_get_defaults(void)
-{
-  FlowDialog *prop_dialog;
-  GtkWidget *dialog;
-  GtkWidget *entry;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GSList *group;
-
-  if (defaults_dialog == NULL) {
-
-    prop_dialog = g_new(FlowDialog, 1);
-    defaults_dialog = prop_dialog;
-    
-    dialog = gtk_vbox_new(FALSE, 0);
-    prop_dialog->dialog = dialog;
-
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-
-    label = gtk_label_new(_("Flow:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    entry = gtk_text_new(NULL, NULL);
-    prop_dialog->text = entry ;
-    gtk_text_set_editable(GTK_TEXT(entry), TRUE);
-    gtk_widget_set_usize( GTK_WIDGET(entry), 100, 50 ) ;
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    gtk_widget_show (label);
-    gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
-    gtk_widget_show(hbox);
-
-    label = gtk_hseparator_new ();
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    label = gtk_label_new(_("Flow type:"));
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    /* */
-    prop_dialog->m_energy = gtk_radio_button_new_with_label (NULL, _("Energy"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_energy, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_energy);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_dialog->m_energy), TRUE);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_energy));
-
-    prop_dialog->m_material = gtk_radio_button_new_with_label(group, _("Material"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_material, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_material);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_material));
-
-    prop_dialog->m_signal = gtk_radio_button_new_with_label(group, _("Signal"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_signal, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_signal);
-  }
-  
-  fill_in_defaults_dialog();
-  gtk_widget_show (defaults_dialog->dialog);
-
-  return defaults_dialog->dialog;
-}
-
-static void
-flow_apply_defaults(void)
-{
-  FlowDialog *prop_dialog;
-  Color* color = 0 ;
-
-  prop_dialog = defaults_dialog;
-
-  if (GTK_TOGGLE_BUTTON(prop_dialog->m_energy)->active) {
-    flow_most_recent_type = FLOW_ENERGY;
-    color = &flow_color_energy ;
-  } else if (GTK_TOGGLE_BUTTON( prop_dialog->m_material )->active) {
-    flow_most_recent_type = FLOW_MATERIAL;
-    color = &flow_color_material ;
-  } else if (GTK_TOGGLE_BUTTON( prop_dialog->m_signal )->active) {
-    flow_most_recent_type = FLOW_SIGNAL;
-    color = &flow_color_signal ;
-  }
-
-  if ( ! flow_default_label ) {
-    Point p ;
-
-    if (flow_font == NULL)
-      flow_font = font_getfont("Helvetica-Oblique");
-    flow_default_label =
-      new_text(
-	       gtk_editable_get_chars( GTK_EDITABLE(prop_dialog->text),0,-1), 
-	       flow_font, FLOW_FONTHEIGHT, &p, color, ALIGN_CENTER
-	       ) ;
-  } else {
-    text_set_string(flow_default_label,
-		    gtk_editable_get_chars( GTK_EDITABLE(prop_dialog->text),
-					    0, -1));
-    text_set_color( flow_default_label, color ) ;
-  }
-
-}
