@@ -41,10 +41,12 @@
 typedef struct _Function Function;
 typedef struct _FunctionChange FunctionChange;
 
+#define NUM_CONNECTIONS 9
+
 struct _Function {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[NUM_CONNECTIONS];
   
   Text *text;
   TextAttributes attrs;
@@ -407,6 +409,10 @@ function_update_data(Function *pkg)
 		   elem->corner.x + elem->width,
 		   elem->corner.y + elem->height,
 		   DIR_SOUTHEAST);
+  connpoint_update(&pkg->connections[8],
+		   elem->corner.x + elem->width / 2.0,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_SOUTHEAST);
   
   element_update_boundingbox(elem);
 
@@ -450,13 +456,15 @@ function_create(Point *startpoint,
                        ALIGN_CENTER);
   dia_font_unref(font);
   
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &pkg->connections[i];
     pkg->connections[i].object = obj;
     pkg->connections[i].connected = NULL;
   }
+  pkg->connections[8].flags = CP_FLAGS_MAIN;
+
   pkg->element.extra_spacing.border_trans = FUNCTION_FONTHEIGHT / FUNCTION_BORDERWIDTH_SCALE/2.0;
   function_update_data(pkg);
 
@@ -497,12 +505,13 @@ function_copy(Function *pkg)
 
   newpkg->text = text_copy(pkg->text);
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     newobj->connections[i] = &newpkg->connections[i];
     newpkg->connections[i].object = newobj;
     newpkg->connections[i].connected = NULL;
     newpkg->connections[i].pos = pkg->connections[i].pos;
     newpkg->connections[i].last_pos = pkg->connections[i].last_pos;
+    newpkg->connections[i].flags = pkg->connections[i].flags;
   }
   newpkg->is_wish = pkg->is_wish ;
   newpkg->is_user = pkg->is_user ;
@@ -565,13 +574,15 @@ function_load(ObjectNode obj_node, int version, const char *filename)
   else
     pkg->is_user = FALSE;
 
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
 
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &pkg->connections[i];
     pkg->connections[i].object = obj;
     pkg->connections[i].connected = NULL;
   }
+  pkg->connections[8].flags = CP_FLAGS_MAIN;
+
   pkg->element.extra_spacing.border_trans = pkg->text ? pkg->text->height : FUNCTION_FONTHEIGHT / FUNCTION_BORDERWIDTH_SCALE/2.0;
   function_update_data(pkg);
 
@@ -591,8 +602,8 @@ function_insert_word( Function* func, const char* word, gboolean newline )
 		  	+ ( newline ? 2 : 1) ) ;
   sprintf( new_chars, newline ? "%s\n%s" : "%s%s", old_chars, word ) ;
   text_set_string( func->text, new_chars ) ;
-  free( new_chars ) ;
-  free( old_chars ) ;
+  g_free( new_chars ) ;
+  g_free( old_chars ) ;
   function_update_data( func ) ;
   text_set_cursor_at_end( func->text ) ;
 
@@ -1196,19 +1207,19 @@ function_get_object_menu( Function* func, Point* clickedpoint )
     int curitem[ FS_SUBMENU_MAXINDENT ] ;
 
     curitem[0] = 0 ;
-    curMenu[0] = malloc( sizeof( DiaMenu ) ) ;
+    curMenu[0] = g_malloc( sizeof( DiaMenu ) ) ;
     curMenu[0]->title = "Function" ;
     curMenu[0]->num_items = function_count_submenu_items( &(fmenu[0]) ) ;
-    curMenu[0]->items = malloc( curMenu[0]->num_items * sizeof(DiaMenuItem) );
+    curMenu[0]->items = g_malloc( curMenu[0]->num_items * sizeof(DiaMenuItem) );
     curMenu[0]->app_data = NULL ;
     for (i = 0 ; fmenu[i].depth >= 0; i++) {
       if ( fmenu[i].depth > curDepth ) {
 	curDepth++ ;
-	curMenu[curDepth] = malloc( sizeof( DiaMenu ) ) ;
+	curMenu[curDepth] = g_malloc( sizeof( DiaMenu ) ) ;
 	curMenu[curDepth]->title = NULL ;
 	curMenu[curDepth]->app_data = NULL ;
 	curMenu[curDepth]->num_items = function_count_submenu_items(&fmenu[i]);
-	curMenu[curDepth]->items = malloc( curMenu[curDepth]->num_items *
+	curMenu[curDepth]->items = g_malloc( curMenu[curDepth]->num_items *
 				  sizeof(DiaMenuItem) ) ;
 	/* Point this menu's parent to this new structure */
 	curMenu[curDepth-1]->items[curitem[curDepth-1]-1].callback = NULL ;
@@ -1229,5 +1240,3 @@ function_get_object_menu( Function* func, Point* clickedpoint )
   }
   return function_menu ;
 }
-
-
