@@ -21,29 +21,22 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "arrows.h"
 #include "properties.h"
 #include "stereotype.h"
+#include "attributes.h"
 
 #include "pixmaps/constraint.xpm"
 
 typedef struct _Constraint Constraint;
-typedef struct _ConstraintState ConstraintState;
-
-struct _ConstraintState {
-  ObjectState obj_state;
-  
-  char *text;
-};
 
 struct _Constraint {
   Connection connection;
@@ -51,8 +44,12 @@ struct _Constraint {
   Handle text_handle;
 
   char *text;
+  char *brtext;
   Point text_pos;
   real text_width;
+
+  Color text_color;
+  Color line_color;
 };
 
   
@@ -65,43 +62,40 @@ struct _Constraint {
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM1)
 
 
-static Font *constraint_font = NULL;
+static DiaFont *constraint_font = NULL;
 
-static void constraint_move_handle(Constraint *constraint, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void constraint_move(Constraint *constraint, Point *to);
+static ObjectChange* constraint_move_handle(Constraint *constraint, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* constraint_move(Constraint *constraint, Point *to);
 static void constraint_select(Constraint *constraint, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void constraint_draw(Constraint *constraint, Renderer *renderer);
-static Object *constraint_create(Point *startpoint,
+			      DiaRenderer *interactive_renderer);
+static void constraint_draw(Constraint *constraint, DiaRenderer *renderer);
+static DiaObject *constraint_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real constraint_distance_from(Constraint *constraint, Point *point);
 static void constraint_update_data(Constraint *constraint);
 static void constraint_destroy(Constraint *constraint);
-static Object *constraint_copy(Constraint *constraint);
 
-static ConstraintState *constraint_get_state(Constraint *constraint);
-static void constraint_set_state(Constraint *constraint,
-				 ConstraintState *state);
 static PropDescription *constraint_describe_props(Constraint *constraint);
-static void constraint_get_props(Constraint * constraint, Property *props, guint nprops);
-static void constraint_set_props(Constraint * constraint, Property *props, guint nprops);
+static void constraint_get_props(Constraint * constraint, GPtrArray *props);
+static void constraint_set_props(Constraint * constraint, GPtrArray *props);
 
-static void constraint_save(Constraint *constraint, ObjectNode obj_node,
-			    const char *filename);
-static Object *constraint_load(ObjectNode obj_node, int version,
+static DiaObject *constraint_load(ObjectNode obj_node, int version,
 			       const char *filename);
 
 static ObjectTypeOps constraint_type_ops =
 {
   (CreateFunc) constraint_create,
-  (LoadFunc)   constraint_load,
-  (SaveFunc)   constraint_save
+  (LoadFunc)   constraint_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
-ObjectType constraint_type =
+DiaObjectType constraint_type =
 {
   "UML - Constraint",        /* name */
   0,                         /* version */
@@ -114,7 +108,7 @@ static ObjectOps constraint_ops = {
   (DrawFunc)            constraint_draw,
   (DistanceFunc)        constraint_distance_from,
   (SelectFunc)          constraint_select,
-  (CopyFunc)            constraint_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            constraint_move,
   (MoveHandleFunc)      constraint_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
@@ -126,82 +120,48 @@ static ObjectOps constraint_ops = {
 };
 
 static PropDescription constraint_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  CONNECTION_COMMON_PROPERTIES,
   { "constraint", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Constraint:"), NULL, NULL },
+  { "text_pos", PROP_TYPE_POINT, 0, NULL, NULL, NULL},
+  /* can't use PROP_STD_TEXT_COLOUR_OPTIONAL cause it has PROP_FLAG_DONT_SAVE. It is designed to fill the Text object - not some subset */
+  PROP_STD_TEXT_COLOUR_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL),
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
   PROP_DESC_END
 };
 
 static PropDescription *
 constraint_describe_props(Constraint *constraint)
 {
-  if (constraint_props[0].quark == 0)
+  if (constraint_props[0].quark == 0) {
     prop_desc_list_calculate_quarks(constraint_props);
+  }
   return constraint_props;
 }
 
 static PropOffset constraint_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
-  /*  { "constraint", PROP_TYPE_STRING, offsetof(Constraint, text) },*/
+  CONNECTION_COMMON_PROPERTIES_OFFSETS,
+  { "constraint", PROP_TYPE_STRING, offsetof(Constraint, text) },
+  { "text_pos", PROP_TYPE_POINT, offsetof(Constraint, text_pos) },
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Constraint, text_color)},
+  { "line_colour",PROP_TYPE_COLOUR,offsetof(Constraint, line_color)},
   { NULL, 0, 0 }
 };
 
-static struct { const gchar *name; GQuark q; } quarks[] = {
-  { "constraint" }
-};
-
 static void
-constraint_get_props(Constraint * constraint, Property *props, guint nprops)
+constraint_get_props(Constraint * constraint, GPtrArray *props)
 {
-  guint i;
-
-  if (object_get_props_from_offsets(&constraint->connection.object, 
-                                    constraint_offsets, props, nprops))
-    return;
-  /* these props can't be handled as easily */
-  if (quarks[0].q == 0)
-    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-      quarks[i].q = g_quark_from_static_string(quarks[i].name);
-  for (i = 0; i < nprops; i++) {
-    GQuark pquark = g_quark_from_string(props[i].name);
-
-    if (pquark == quarks[0].q) {
-      props[i].type = PROP_TYPE_STRING;
-      g_free(PROP_VALUE_STRING(props[i]));
-      if (constraint->text != NULL &&
-	  constraint->text[0] != '\0')
-	PROP_VALUE_STRING(props[i]) =
-	  bracketted_to_string(constraint->text, strlen("{"));
-      else
-	PROP_VALUE_STRING(props[i]) = NULL;	
-    }
-  }
+  object_get_props_from_offsets(&constraint->connection.object, 
+                                constraint_offsets, props);
 }
 
 static void
-constraint_set_props(Constraint *constraint, Property *props, guint nprops)
+constraint_set_props(Constraint *constraint, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&constraint->connection.object, 
-                                     constraint_offsets, props, nprops)) {
-    guint i;
-
-    if (quarks[0].q == 0)
-      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-	quarks[i].q = g_quark_from_static_string(quarks[i].name);
-    
-    for (i = 0; i < nprops; i++) {
-      GQuark pquark = g_quark_from_string(props[i].name);
-      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_STRING) {
-	g_free(constraint->text);
-	if (PROP_VALUE_STRING(props[i]) != NULL &&
-	    PROP_VALUE_STRING(props[i])[0] != '\0')
-	  constraint->text=
-	    string_to_bracketted(PROP_VALUE_STRING(props[i]), "{", "}");
-	else 
-	  constraint->text = g_strdup("{}");
-      }
-    }
-  }
+  object_set_props_from_offsets(&constraint->connection.object, 
+                                constraint_offsets, props);
+  g_free(constraint->brtext);
+  constraint->brtext = NULL;
   constraint_update_data(constraint);
 }
 
@@ -213,21 +173,22 @@ constraint_distance_from(Constraint *constraint, Point *point)
   
   endpoints = &constraint->connection.endpoints[0];
   
-  dist = distance_line_point(&endpoints[0], &endpoints[1], CONSTRAINT_WIDTH, point);
-  
+  dist = distance_line_point(&endpoints[0], &endpoints[1], 
+                             CONSTRAINT_WIDTH, point);  
   return dist;
 }
 
 static void
 constraint_select(Constraint *constraint, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   connection_update_handles(&constraint->connection);
 }
 
-static void
+static ObjectChange*
 constraint_move_handle(Constraint *constraint, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p1, p2;
   Point *endpoints;
@@ -242,7 +203,8 @@ constraint_move_handle(Constraint *constraint, Handle *handle,
     endpoints = &constraint->connection.endpoints[0]; 
     p1.x = 0.5*(endpoints[0].x + endpoints[1].x);
     p1.y = 0.5*(endpoints[0].y + endpoints[1].y);
-    connection_move_handle(&constraint->connection, handle->id, to, reason);
+    connection_move_handle(&constraint->connection, handle->id, to, cp, 
+			   reason, modifiers);
     p2.x = 0.5*(endpoints[0].x + endpoints[1].x);
     p2.y = 0.5*(endpoints[0].y + endpoints[1].y);
     point_sub(&p2, &p1);
@@ -250,9 +212,11 @@ constraint_move_handle(Constraint *constraint, Handle *handle,
   }
 
   constraint_update_data(constraint);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 constraint_move(Constraint *constraint, Point *to)
 {
   Point start_to_end;
@@ -271,41 +235,46 @@ constraint_move(Constraint *constraint, Point *to)
   point_add(&constraint->text_pos, &delta);
   
   constraint_update_data(constraint);
+
+  return NULL;
 }
 
 static void
-constraint_draw(Constraint *constraint, Renderer *renderer)
+constraint_draw(Constraint *constraint, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point *endpoints;
-  
+  Arrow arrow;
+
   assert(constraint != NULL);
   assert(renderer != NULL);
 
   endpoints = &constraint->connection.endpoints[0];
   
-  renderer->ops->set_linewidth(renderer, CONSTRAINT_WIDTH);
-  renderer->ops->set_dashlength(renderer, CONSTRAINT_DASHLEN);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_DASHED);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, CONSTRAINT_WIDTH);
+  renderer_ops->set_dashlength(renderer, CONSTRAINT_DASHLEN);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_line(renderer,
-			   &endpoints[0], &endpoints[1],
-			   &color_black);
+  arrow.type = ARROW_LINES;
+  arrow.length = CONSTRAINT_ARROWLEN;
+  arrow.width = CONSTRAINT_ARROWWIDTH;
+
+  renderer_ops->draw_line_with_arrows(renderer,
+				       &endpoints[0], &endpoints[1],
+				       CONSTRAINT_WIDTH,
+				       &constraint->line_color,
+				       NULL, &arrow);
   
-  arrow_draw(renderer, ARROW_LINES,
-	     &endpoints[1], &endpoints[0],
-	     CONSTRAINT_ARROWLEN, CONSTRAINT_ARROWWIDTH, CONSTRAINT_WIDTH,
-	     &color_black, &color_white);
-
-  renderer->ops->set_font(renderer, constraint_font,
+  renderer_ops->set_font(renderer, constraint_font,
 			  CONSTRAINT_FONTHEIGHT);
-  renderer->ops->draw_string(renderer,
-			     constraint->text,
+  renderer_ops->draw_string(renderer,
+			     constraint->brtext,
 			     &constraint->text_pos, ALIGN_LEFT,
-			     &color_black);
+			     &constraint->text_color);
 }
 
-static Object *
+static DiaObject *
 constraint_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -313,11 +282,13 @@ constraint_create(Point *startpoint,
 {
   Constraint *constraint;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { 1.0, 1.0 };
 
-  if (constraint_font == NULL)
-    constraint_font = font_getfont("Courier");
+  if (constraint_font == NULL) {
+    constraint_font = 
+      dia_font_new_from_style (DIA_FONT_MONOSPACE, CONSTRAINT_FONTHEIGHT);
+  }
   
   constraint = g_malloc0(sizeof(Constraint));
 
@@ -333,9 +304,11 @@ constraint_create(Point *startpoint,
   
   connection_init(conn, 3, 0);
 
-  constraint->text = strdup("{}");
+  constraint->text_color = color_black;
+  constraint->line_color = attributes_get_foreground();
+  constraint->text = g_strdup("");
   constraint->text_pos.x = 0.5*(conn->endpoints[0].x + conn->endpoints[1].x);
-  constraint->text_pos.y = 0.5*(conn->endpoints[0].y + conn->endpoints[1].y);
+  constraint->text_pos.y = 0.5*(conn->endpoints[0].y + conn->endpoints[1].y) - 0.2;
 
   constraint->text_handle.id = HANDLE_MOVE_TEXT;
   constraint->text_handle.type = HANDLE_MINOR_CONTROL;
@@ -343,6 +316,7 @@ constraint_create(Point *startpoint,
   constraint->text_handle.connected_to = NULL;
   obj->handles[2] = &constraint->text_handle;
   
+  constraint->brtext = NULL;
   constraint_update_data(constraint);
 
   *handle1 = obj->handles[0];
@@ -355,79 +329,36 @@ static void
 constraint_destroy(Constraint *constraint)
 {
   connection_destroy(&constraint->connection);
-
+  g_free(constraint->brtext);
   g_free(constraint->text);
-
 }
 
-static Object *
-constraint_copy(Constraint *constraint)
-{
-  Constraint *newconstraint;
-  Connection *conn, *newconn;
-  Object *newobj;
-  
-  conn = &constraint->connection;
-  
-  newconstraint = g_malloc0(sizeof(Constraint));
-  newconn = &newconstraint->connection;
-  newobj = &newconn->object;
-
-  connection_copy(conn, newconn);
-
-  newconstraint->text_handle = constraint->text_handle;
-  newobj->handles[2] = &newconstraint->text_handle;
-
-  newconstraint->text = strdup(constraint->text);
-  newconstraint->text_pos = constraint->text_pos;
-  newconstraint->text_width = constraint->text_width;
-
-  return &newconstraint->connection.object;
-}
-
-static void
-constraint_state_free(ObjectState *ostate)
-{
-  ConstraintState *state = (ConstraintState *)ostate;
-  g_free(state->text);
-}
-
-static ConstraintState *
-constraint_get_state(Constraint *constraint)
-{
-  ConstraintState *state = g_new0(ConstraintState, 1);
-
-  state->obj_state.free = constraint_state_free;
-
-  state->text = g_strdup(constraint->text);
-
-  return state;
-}
-
-static void
-constraint_set_state(Constraint *constraint, ConstraintState *state)
-{
-  g_free(constraint->text);
-  constraint->text = state->text;
-  
-  g_free(state);
-  
-  constraint_update_data(constraint);
-}
-
-
 static void
 constraint_update_data(Constraint *constraint)
 {
   Connection *conn = &constraint->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
   Rectangle rect;
-  ConnectionBBExtras *extra;
+  LineBBExtras *extra;
 
+  if ((constraint->text) && (constraint->text[0] == '{')) {
+    /* we might have a string loaded from an older dia. Clean it up. */
+    g_free(constraint->brtext);
+    constraint->brtext = constraint->text;
+    constraint->text = bracketted_to_string(constraint->text,"{","}");
+  } else if (!constraint->brtext) {
+    constraint->brtext = string_to_bracketted(constraint->text, "{", "}");
+  }
+  
+  if (connpoint_is_autogap(conn->endpoint_handles[0].connected_to) ||
+      connpoint_is_autogap(conn->endpoint_handles[1].connected_to)) {
+    connection_adjust_for_autogap(conn);
+  }
   obj->position = conn->endpoints[0];
 
-  constraint->text_width =
-    font_string_width(constraint->text, constraint_font, CONSTRAINT_FONTHEIGHT);
+  constraint->text_width = dia_font_string_width(constraint->brtext, 
+                                                 constraint_font, 
+                                                 CONSTRAINT_FONTHEIGHT);
   
   constraint->text_handle.pos = constraint->text_pos;
 
@@ -446,67 +377,17 @@ constraint_update_data(Constraint *constraint)
   /* Add boundingbox for text: */
   rect.left = constraint->text_pos.x;
   rect.right = rect.left + constraint->text_width;
-  rect.top = constraint->text_pos.y - font_ascent(constraint_font, CONSTRAINT_FONTHEIGHT);
+  rect.top = constraint->text_pos.y -
+      dia_font_ascent(constraint->brtext,
+                      constraint_font, CONSTRAINT_FONTHEIGHT);
   rect.bottom = rect.top + CONSTRAINT_FONTHEIGHT;
   rectangle_union(&obj->bounding_box, &rect);
 }
 
 
-static void
-constraint_save(Constraint *constraint, ObjectNode obj_node,
-		const char *filename)
-{
-  connection_save(&constraint->connection, obj_node);
-
-  data_add_string(new_attribute(obj_node, "text"),
-		  constraint->text);
-  data_add_point(new_attribute(obj_node, "text_pos"),
-		 &constraint->text_pos);
-}
-
-static Object *
+static DiaObject *
 constraint_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Constraint *constraint;
-  AttributeNode attr;
-  Connection *conn;
-  ConnectionBBExtras *extra;
-  Object *obj;
-
-  if (constraint_font == NULL)
-    constraint_font = font_getfont("Courier");
-
-  constraint = g_malloc0(sizeof(Constraint));
-
-  conn = &constraint->connection;
-  obj = &conn->object;
-  extra = &conn->extra_spacing;
-
-  obj->type = &constraint_type;
-  obj->ops = &constraint_ops;
-
-  connection_load(conn, obj_node);
-  
-  connection_init(conn, 3, 0);
-
-  constraint->text = NULL;
-  attr = object_find_attribute(obj_node, "text");
-  if (attr != NULL)
-    constraint->text = data_string(attribute_first_data(attr));
-  else
-    constraint->text = strdup("{}");
-
-  attr = object_find_attribute(obj_node, "text_pos");
-  if (attr != NULL)
-    data_point(attribute_first_data(attr), &constraint->text_pos);
-
-  constraint->text_handle.id = HANDLE_MOVE_TEXT;
-  constraint->text_handle.type = HANDLE_MINOR_CONTROL;
-  constraint->text_handle.connect_type = HANDLE_NONCONNECTABLE;
-  constraint->text_handle.connected_to = NULL;
-  obj->handles[2] = &constraint->text_handle;
-  
-  constraint_update_data(constraint);
-  
-  return &constraint->connection.object;
+  return object_load_using_properties(&constraint_type,
+                                      obj_node,version,filename);
 }
