@@ -36,7 +36,7 @@
 
 #include "create.h"
 
-#include "pixmaps/polyline.xpm"
+#include "tool-icons.h"
 
 #define DEFAULT_WIDTH 0.15
 
@@ -47,24 +47,27 @@ typedef struct _Polyline {
   LineStyle line_style;
   real dashlength;
   real line_width;
+  real corner_radius;
   Arrow start_arrow, end_arrow;
+  real absolute_start_gap, absolute_end_gap;
 } Polyline;
 
 
-static void polyline_move_handle(Polyline *polyline, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void polyline_move(Polyline *polyline, Point *to);
+static ObjectChange* polyline_move_handle(Polyline *polyline, Handle *handle,
+					  Point *to, ConnectionPoint *cp,
+					  HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* polyline_move(Polyline *polyline, Point *to);
 static void polyline_select(Polyline *polyline, Point *clicked_point,
 			      DiaRenderer *interactive_renderer);
 static void polyline_draw(Polyline *polyline, DiaRenderer *renderer);
-static Object *polyline_create(Point *startpoint,
+static DiaObject *polyline_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real polyline_distance_from(Polyline *polyline, Point *point);
 static void polyline_update_data(Polyline *polyline);
 static void polyline_destroy(Polyline *polyline);
-static Object *polyline_copy(Polyline *polyline);
+static DiaObject *polyline_copy(Polyline *polyline);
 
 static PropDescription *polyline_describe_props(Polyline *polyline);
 static void polyline_get_props(Polyline *polyline, GPtrArray *props);
@@ -72,9 +75,11 @@ static void polyline_set_props(Polyline *polyline, GPtrArray *props);
 
 static void polyline_save(Polyline *polyline, ObjectNode obj_node,
 			  const char *filename);
-static Object *polyline_load(ObjectNode obj_node, int version,
+static DiaObject *polyline_load(ObjectNode obj_node, int version,
 			     const char *filename);
 static DiaMenu *polyline_get_object_menu(Polyline *polyline, Point *clickedpoint);
+void polyline_calculate_gap_endpoints(Polyline *polyline, Point *gap_endpoints);
+static void polyline_exchange_gap_points(Polyline *polyline,  Point *gap_points);
 
 static ObjectTypeOps polyline_type_ops =
 {
@@ -85,16 +90,18 @@ static ObjectTypeOps polyline_type_ops =
   (ApplyDefaultsFunc) NULL /*polyline_apply_defaults*/
 };
 
-static ObjectType polyline_type =
+static DiaObjectType polyline_type =
 {
   "Standard - PolyLine",   /* name */
   0,                         /* version */
-  (char **) polyline_xpm,      /* pixmap */
+  (char **) polyline_icon,      /* pixmap */
   
-  &polyline_type_ops       /* ops */
+  &polyline_type_ops,       /* ops */
+  NULL, /* pixmap_file */
+  0 /* default_user_data */
 };
 
-ObjectType *_polyline_type = (ObjectType *) &polyline_type;
+DiaObjectType *_polyline_type = (DiaObjectType *) &polyline_type;
 
 
 static ObjectOps polyline_ops = {
@@ -113,13 +120,24 @@ static ObjectOps polyline_ops = {
   (SetPropsFunc)        polyline_set_props,
 };
 
+static PropNumData polyline_corner_radius_data = { 0.0, 10.0, 0.1 };
+static PropNumData gap_range = { -G_MAXFLOAT, G_MAXFLOAT, 0.1};
+
 static PropDescription polyline_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  POLYCONN_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_LINE_STYLE,
   PROP_STD_START_ARROW,
   PROP_STD_END_ARROW,
+  { "corner_radius", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Corner radius"), NULL, &polyline_corner_radius_data },
+  PROP_FRAME_BEGIN("gaps",0,N_("Line gaps")),
+  { "absolute_start_gap", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Absolute start gap"), NULL, &gap_range },
+  { "absolute_end_gap", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Absolute end gap"), NULL, &gap_range },
+  PROP_FRAME_END("gaps",0),
   PROP_DESC_END
 };
 
@@ -132,13 +150,16 @@ polyline_describe_props(Polyline *polyline)
 }
 
 static PropOffset polyline_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  POLYCONN_COMMON_PROPERTIES_OFFSETS,
   { "line_width", PROP_TYPE_REAL, offsetof(Polyline, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Polyline, line_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
     offsetof(Polyline, line_style), offsetof(Polyline, dashlength) },
   { "start_arrow", PROP_TYPE_ARROW, offsetof(Polyline, start_arrow) },
   { "end_arrow", PROP_TYPE_ARROW, offsetof(Polyline, end_arrow) },
+  { "corner_radius", PROP_TYPE_REAL, offsetof(Polyline, corner_radius) },
+  { "absolute_start_gap", PROP_TYPE_REAL, offsetof(Polyline, absolute_start_gap) },
+  { "absolute_end_gap", PROP_TYPE_REAL, offsetof(Polyline, absolute_end_gap) },
   { NULL, 0, 0 }
 };
 
@@ -161,7 +182,13 @@ static real
 polyline_distance_from(Polyline *polyline, Point *point)
 {
   PolyConn *poly = &polyline->poly;
-  return polyconn_distance_from(poly, point, polyline->line_width);
+  real dist;
+  Point gap_endpoints[2];
+  polyline_calculate_gap_endpoints(polyline, gap_endpoints);
+  polyline_exchange_gap_points(polyline, gap_endpoints);
+  dist = polyconn_distance_from(poly, point, polyline->line_width);
+  polyline_exchange_gap_points(polyline, gap_endpoints);
+  return dist;
 }
 
 static Handle *polyline_closest_handle(Polyline *polyline, Point *point) {
@@ -180,29 +207,85 @@ polyline_select(Polyline *polyline, Point *clicked_point,
   polyconn_update_data(&polyline->poly);
 }
 
-static void
+static ObjectChange*
 polyline_move_handle(Polyline *polyline, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		     Point *to, ConnectionPoint *cp,
+		     HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(polyline!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  polyconn_move_handle(&polyline->poly, handle, to, reason);
+  polyconn_move_handle(&polyline->poly, handle, to, cp, reason, modifiers);
   polyline_update_data(polyline);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 polyline_move(Polyline *polyline, Point *to)
 {
   polyconn_move(&polyline->poly, to);
   polyline_update_data(polyline);
+
+  return NULL;
 }
 
+void
+polyline_calculate_gap_endpoints(Polyline *polyline, Point *gap_endpoints)
+{
+  Point  start_vec, end_vec;
+  ConnectionPoint *start_cp, *end_cp;
+  int n = polyline->poly.numpoints;
+
+  gap_endpoints[0] = polyline->poly.points[0];
+  gap_endpoints[1] = polyline->poly.points[n-1];
+
+  start_cp = (polyline->poly.object.handles[0])->connected_to;
+  end_cp = (polyline->poly.object.handles[polyline->poly.object.num_handles-1])->connected_to;
+
+  if (connpoint_is_autogap(start_cp)) {
+      gap_endpoints[0] = calculate_object_edge(&gap_endpoints[0],
+					   &polyline->poly.points[1],
+					   start_cp->object);
+  }
+  if (connpoint_is_autogap(end_cp)) {    
+      gap_endpoints[1] = calculate_object_edge(&gap_endpoints[1],
+					   &polyline->poly.points[n-2],
+					   end_cp->object);
+  }
+
+  start_vec = gap_endpoints[0];
+  point_sub(&start_vec, &polyline->poly.points[0]);
+  point_normalize(&start_vec);
+
+  end_vec = gap_endpoints[1];
+  point_sub(&end_vec, &polyline->poly.points[n-1]);
+  point_normalize(&end_vec);
+  
+  /* add absolute gap */
+  point_add_scaled(&gap_endpoints[0], &start_vec, polyline->absolute_start_gap);
+  point_add_scaled(&gap_endpoints[1], &end_vec, polyline->absolute_end_gap);
+}
+static void
+polyline_exchange_gap_points(Polyline *polyline,  Point *gap_points)
+{
+        Point tmp[2];
+        int n = polyline->poly.numpoints;
+        tmp[0] = gap_points[0];
+        tmp[1] = gap_points[1];
+        gap_points[0] = polyline->poly.points[0];
+        gap_points[1] = polyline->poly.points[n-1];
+        polyline->poly.points[0] = tmp[0];
+        polyline->poly.points[n-1] = tmp[1];
+}
+
+        
 static void
 polyline_draw(Polyline *polyline, DiaRenderer *renderer)
 {
+  Point gap_endpoints[2];
   DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   PolyConn *poly = &polyline->poly;
   Point *points;
@@ -210,19 +293,22 @@ polyline_draw(Polyline *polyline, DiaRenderer *renderer)
   
   points = &poly->points[0];
   n = poly->numpoints;
-
   renderer_ops->set_linewidth(renderer, polyline->line_width);
   renderer_ops->set_linestyle(renderer, polyline->line_style);
   renderer_ops->set_dashlength(renderer, polyline->dashlength);
   renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
   renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer_ops->draw_polyline_with_arrows(renderer,
-					   points, n,
-					   polyline->line_width,
-					   &polyline->line_color,
-					   &polyline->start_arrow,
-					   &polyline->end_arrow);
+  polyline_calculate_gap_endpoints(polyline, gap_endpoints);
+  polyline_exchange_gap_points(polyline, gap_endpoints);
+  renderer_ops->draw_rounded_polyline_with_arrows(renderer,
+						  points, n,
+						  polyline->line_width,
+						  &polyline->line_color,
+						  &polyline->start_arrow,
+						  &polyline->end_arrow,
+						  polyline->corner_radius);
+  polyline_exchange_gap_points(polyline, gap_endpoints);
 }
 
 /** user_data is a struct polyline_create_data, containing an array of 
@@ -230,7 +316,7 @@ polyline_draw(Polyline *polyline, DiaRenderer *renderer)
     If user_data is NULL, the startpoint is used and a 1x1 line is created.
     Otherwise, the startpoint is ignored.
 */
-static Object *
+static DiaObject *
 polyline_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -238,7 +324,7 @@ polyline_create(Point *startpoint,
 {
   Polyline *polyline;
   PolyConn *poly;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { 1.0, 1.0 };
 
   /*polyline_init_defaults();*/
@@ -270,7 +356,8 @@ polyline_create(Point *startpoint,
     *handle1 = poly->object.handles[0];
     *handle2 = poly->object.handles[pcd->num_points-1];
   }
-  
+
+
   polyline_update_data(polyline);
 
   polyline->line_width =  attributes_get_default_linewidth();
@@ -279,6 +366,7 @@ polyline_create(Point *startpoint,
 				    &polyline->dashlength);
   polyline->start_arrow = attributes_get_default_start_arrow();
   polyline->end_arrow = attributes_get_default_end_arrow();
+  polyline->corner_radius = 0.0;
 
   return &polyline->poly.object;
 }
@@ -289,12 +377,12 @@ polyline_destroy(Polyline *polyline)
   polyconn_destroy(&polyline->poly);
 }
 
-static Object *
+static DiaObject *
 polyline_copy(Polyline *polyline)
 {
   Polyline *newpolyline;
   PolyConn *poly, *newpoly;
-  Object *newobj;
+  DiaObject *newobj;
   
   poly = &polyline->poly;
  
@@ -310,6 +398,9 @@ polyline_copy(Polyline *polyline)
   newpolyline->dashlength = polyline->dashlength;
   newpolyline->start_arrow = polyline->start_arrow;
   newpolyline->end_arrow = polyline->end_arrow;
+  newpolyline->corner_radius = polyline->corner_radius;
+  newpolyline->absolute_start_gap = polyline->absolute_start_gap;
+  newpolyline->absolute_end_gap = polyline->absolute_end_gap;
 
   return &newpolyline->poly.object;
 }
@@ -318,7 +409,7 @@ static void
 polyline_update_data(Polyline *polyline)
 {
   PolyConn *poly = &polyline->poly;
-  Object *obj = &poly->object;
+  DiaObject *obj = &poly->object;
   PolyBBExtras *extra = &poly->extra_spacing;
 
   polyconn_update_data(&polyline->poly);
@@ -380,14 +471,25 @@ polyline_save(Polyline *polyline, ObjectNode obj_node,
     data_add_real(new_attribute(obj_node, "end_arrow_width"),
 		  polyline->end_arrow.width);
   }
+
+  if (polyline->absolute_start_gap)
+    data_add_real(new_attribute(obj_node, "absolute_start_gap"),
+                 polyline->absolute_start_gap);
+  if (polyline->absolute_end_gap)
+    data_add_real(new_attribute(obj_node, "absolute_end_gap"),
+                 polyline->absolute_end_gap);
+
+  if (polyline->corner_radius > 0.0)
+    data_add_real(new_attribute(obj_node, "corner_radius"),
+                  polyline->corner_radius);
 }
 
-static Object *
+static DiaObject *
 polyline_load(ObjectNode obj_node, int version, const char *filename)
 {
   Polyline *polyline;
   PolyConn *poly;
-  Object *obj;
+  DiaObject *obj;
   AttributeNode attr;
 
   polyline = g_malloc0(sizeof(Polyline));
@@ -421,8 +523,8 @@ polyline_load(ObjectNode obj_node, int version, const char *filename)
     polyline->dashlength = data_real(attribute_first_data(attr));
 
   polyline->start_arrow.type = ARROW_NONE;
-  polyline->start_arrow.length = 0.8;
-  polyline->start_arrow.width = 0.8;
+  polyline->start_arrow.length = DEFAULT_ARROW_LENGTH;
+  polyline->start_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "start_arrow");
   if (attr != NULL)
     polyline->start_arrow.type = data_enum(attribute_first_data(attr));
@@ -434,8 +536,8 @@ polyline_load(ObjectNode obj_node, int version, const char *filename)
     polyline->start_arrow.width = data_real(attribute_first_data(attr));
 
   polyline->end_arrow.type = ARROW_NONE;
-  polyline->end_arrow.length = 0.8;
-  polyline->end_arrow.width = 0.8;
+  polyline->end_arrow.length = DEFAULT_ARROW_LENGTH;
+  polyline->end_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "end_arrow");
   if (attr != NULL)
     polyline->end_arrow.type = data_enum(attribute_first_data(attr));
@@ -446,13 +548,28 @@ polyline_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     polyline->end_arrow.width = data_real(attribute_first_data(attr));
 
+  polyline->absolute_start_gap = 0.0;
+  attr = object_find_attribute(obj_node, "absolute_start_gap");
+  if (attr != NULL)
+    polyline->absolute_start_gap =  data_real( attribute_first_data(attr) );
+  polyline->absolute_end_gap = 0.0;
+  attr = object_find_attribute(obj_node, "absolute_end_gap");
+  if (attr != NULL)
+    polyline->absolute_end_gap =  data_real( attribute_first_data(attr) );
+
+
+  polyline->corner_radius = 0.0;
+  attr = object_find_attribute(obj_node, "corner_radius");
+  if (attr != NULL)
+    polyline->corner_radius =  data_real( attribute_first_data(attr) );
+
   polyline_update_data(polyline);
 
   return &polyline->poly.object;
 }
 
 static ObjectChange *
-polyline_add_corner_callback (Object *obj, Point *clicked, gpointer data)
+polyline_add_corner_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Polyline *poly = (Polyline*) obj;
   int segment;
@@ -465,7 +582,7 @@ polyline_add_corner_callback (Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-polyline_delete_corner_callback (Object *obj, Point *clicked, gpointer data)
+polyline_delete_corner_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Handle *handle;
   int handle_nr, i;
