@@ -105,6 +105,8 @@ struct _WmfRenderer
 
     int nDashLen; /* the scaled dash length */
     gboolean platform_is_nt; /* advanced line styles supported */
+    gboolean target_emf; /* write enhanced metafile */
+    W32::RECT margins;
 };
 
 struct _WmfRendererClass
@@ -136,7 +138,7 @@ UsePen(WmfRenderer* renderer, Color* colour)
     if (colour) {
 	W32::COLORREF rgb = W32COLOR(colour);
 #ifdef G_OS_WIN32
-	if (renderer->platform_is_nt && renderer->hPrintDC) {
+	if ((renderer->platform_is_nt && renderer->hPrintDC) || renderer->target_emf) {
           W32::LOGBRUSH logbrush;
 	  W32::DWORD    dashes[6];
 	  int num_dashes = 0;
@@ -175,11 +177,9 @@ UsePen(WmfRenderer* renderer, Color* colour)
 	  }
 
 	  renderer->hPen = W32::ExtCreatePen (
-	    renderer->fnPenStyle & PS_SOLID ? renderer->fnPenStyle :
-	      (renderer->fnPenStyle & ~(PS_STYLE_MASK)) | (PS_GEOMETRIC | PS_USERSTYLE),
+	    (renderer->fnPenStyle & ~(PS_STYLE_MASK)) | (PS_GEOMETRIC | (num_dashes > 0 ? PS_USERSTYLE : 0)),
 	    renderer->nLineWidth,
-	    &logbrush, num_dashes, dashes);
-
+	    &logbrush, num_dashes, num_dashes > 0 ? dashes : 0);
 	}
 	else 
 #endif /* G_OS_WIN32 */
@@ -259,7 +259,7 @@ end_render(DiaRenderer *self)
     DIAG_NOTE(renderer, "end_render\n");
     hEmf = W32::CloseEnhMetaFile(renderer->hFileDC);
 
-#ifndef SAVE_EMF
+#if !defined SAVE_EMF && defined G_OS_WIN32 /* the later offers both */
     /* Don't do it when printing */
     if (renderer->sFileName && strlen (renderer->sFileName)) {
 
@@ -267,14 +267,19 @@ end_render(DiaRenderer *self)
 
         f = fopen(renderer->sFileName, "wb");
 
-	/* write the placeable header */
-	fwrite(&renderer->pmh, 1, 22 /* NOT: sizeof(PLACEABLEMETAHEADER) */, f);
-
-	/* get size */
-	nSize = W32::GetWinMetaFileBits(hEmf, 0, NULL, MM_ANISOTROPIC, hdc);
-	pData = g_new(W32::BYTE, nSize);
-	/* get data */
-	nSize = W32::GetWinMetaFileBits(hEmf, nSize, pData, MM_ANISOTROPIC, hdc);
+	if (renderer->target_emf) {
+	  nSize = W32::GetEnhMetaFileBits (hEmf, 0, NULL);
+	  pData = g_new(W32::BYTE, nSize);
+	  nSize = W32::GetEnhMetaFileBits (hEmf, nSize, pData);
+	} else {
+	  /* write the placeable header */
+	  fwrite(&renderer->pmh, 1, 22 /* NOT: sizeof(PLACEABLEMETAHEADER) */, f);
+	  /* get size */
+	  nSize = W32::GetWinMetaFileBits(hEmf, 0, NULL, MM_ANISOTROPIC, hdc);
+	  pData = g_new(W32::BYTE, nSize);
+	  /* get data */
+	  nSize = W32::GetWinMetaFileBits(hEmf, nSize, pData, MM_ANISOTROPIC, hdc);
+	}
 
 	/* write file */
 	fwrite(pData,1,nSize,f);
@@ -282,13 +287,9 @@ end_render(DiaRenderer *self)
     
 	g_free(pData);
 
-        W32::DeleteDC(hdc);
+        W32::ReleaseDC(NULL, hdc);
     } else {
-        W32::RECT r = {0, 0, 0, 0};
-        r.right = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALWIDTH); 
-        r.bottom = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALHEIGHT); 
-
-        W32::PlayEnhMetaFile (renderer->hPrintDC, hEmf, &r);
+        W32::PlayEnhMetaFile (renderer->hPrintDC, hEmf, &renderer->margins);
     }
 #endif
     g_free(renderer->sFileName);
@@ -868,15 +869,7 @@ draw_string(DiaRenderer *self,
 
     hOld = W32::SelectObject(renderer->hFileDC, renderer->hFont);
     {
-# if 0 // one way to go, but see below ...
-        gint wclen = 0;
-        gunichar2* swc = g_utf8_to_utf16 (text, -1, NULL, &wclen, NULL);
-        W32::TextOutW (renderer->hFileDC,
-                       SCX(pos->x), SCY(pos->y),
-                       swc, wclen);
-        g_free (swc);
-# else
-        // works with newest cvs and tml's "official" 2000-12-26 release
+        // one way to go, but see below ...
         char* scp; 
         /* convert from utf8 to active codepage */
         static char codepage[10];
@@ -892,11 +885,15 @@ draw_string(DiaRenderer *self,
                          scp, strlen(scp));
             g_free (scp);
         }
-        else // converson failed, write unconverted
-            W32::TextOut(renderer->hFileDC,
-                         SCX(pos->x), SCY(pos->y),
-                         text, strlen (text));
-# endif
+        else // converson failed, write unicode
+        {
+            long wclen = 0;
+            gunichar2* swc = g_utf8_to_utf16 (text, -1, NULL, &wclen, NULL);
+            W32::TextOutW (renderer->hFileDC,
+                           SCX(pos->x), SCY(pos->y),
+                           swc, wclen);
+            g_free (swc);
+        }
     }
 
     W32::SelectObject(renderer->hFileDC, hOld);
@@ -1008,7 +1005,49 @@ draw_image(DiaRenderer *self,
 #endif /* SAVE_EMF */
 }
 
-//XXX: use RoundRect
+static void
+draw_rounded_rect (DiaRenderer *self, 
+	           Point *ul_corner, Point *lr_corner,
+	           Color *colour, real radius)
+{
+    WmfRenderer *renderer = WMF_RENDERER (self);
+
+    W32::HPEN hPen;
+
+    DIAG_NOTE(renderer, "draw_rounded_rect %f,%f -> %f,%f %f\n", 
+              ul_corner->x, ul_corner->y, lr_corner->x, lr_corner->y, radius);
+
+    hPen = UsePen(renderer, colour);
+
+    W32::RoundRect(renderer->hFileDC,
+                   SCX(ul_corner->x), SCY(ul_corner->y),
+                   SCX(lr_corner->x), SCY(lr_corner->y),
+                   SC(radius*2), SC(radius*2));
+
+    DonePen(renderer, hPen);
+}
+
+static void
+fill_rounded_rect (DiaRenderer *self, 
+	           Point *ul_corner, Point *lr_corner,
+	           Color *colour, real radius)
+{
+    WmfRenderer *renderer = WMF_RENDERER (self);
+    W32::HGDIOBJ hBrush, hBrOld;
+    W32::COLORREF rgb = W32COLOR(colour);
+
+    DIAG_NOTE(renderer, "fill_rounded_rect %f,%f -> %f,%f\n", 
+              ul_corner->x, ul_corner->y, lr_corner->x, lr_corner->y);
+
+    hBrush = W32::CreateSolidBrush(rgb);
+    hBrOld = W32::SelectObject(renderer->hFileDC, hBrush);
+
+    draw_rounded_rect(self, ul_corner, lr_corner, NULL, radius);
+
+    W32::SelectObject(renderer->hFileDC, 
+                    W32::GetStockObject (HOLLOW_BRUSH) );
+    W32::DeleteObject(hBrush);
+}
 
 /* GObject boiler plate */
 static void wmf_renderer_class_init (WmfRendererClass *klass);
@@ -1095,6 +1134,8 @@ wmf_renderer_class_init (WmfRendererClass *klass)
 #ifndef SAVE_EMF
   renderer_class->fill_bezier   = fill_bezier;
 #endif
+  renderer_class->draw_rounded_rect = draw_rounded_rect;
+  renderer_class->fill_rounded_rect = fill_rounded_rect;
 }
 
 /* plug-in export api */
@@ -1104,6 +1145,7 @@ export_data(DiagramData *data, const gchar *filename,
 {
     WmfRenderer *renderer;
     W32::HDC  file;
+    W32::HDC refDC;
     Rectangle *extent;
     gint len;
     double scale;
@@ -1120,11 +1162,17 @@ export_data(DiagramData *data, const gchar *filename,
     else {
         scale = floor (32000.0 / (data->extents.bottom - data->extents.top));
     }
-    bbox.right = (int)((data->extents.right - data->extents.left) * scale);
-    bbox.bottom = (int)((data->extents.bottom - data->extents.top) * scale);
+    scale /= 2; // Without this there can be some smallint overflow, dunno why
+
+    refDC = W32::GetDC(NULL);
+    
+    bbox.right = (int)((data->extents.right - data->extents.left) * scale *
+        100 * W32::GetDeviceCaps(refDC, HORZSIZE) / W32::GetDeviceCaps(refDC, HORZRES));
+    bbox.bottom = (int)((data->extents.bottom - data->extents.top) * scale *
+        100 * W32::GetDeviceCaps(refDC, VERTSIZE) / W32::GetDeviceCaps(refDC, VERTRES));
 
     file = (W32::HDC)W32::CreateEnhMetaFile(
-                    W32::GetDC(NULL), // handle to a reference device context
+                    refDC, // handle to a reference device context
 #ifdef SAVE_EMF
                     filename, // pointer to a filename string
 #else
@@ -1142,26 +1190,43 @@ export_data(DiagramData *data, const gchar *filename,
 
     renderer->hFileDC = file;
     renderer->sFileName = g_strdup(filename);
-    renderer->hPrintDC = (W32::HDC)user_data;
-
+    if (user_data == (void*)1) {
+	renderer->target_emf = TRUE;
+	renderer->hPrintDC = 0;
+    } else {
+        renderer->hPrintDC = (W32::HDC)user_data;
+    }
     /* printing is platform dependent */
     renderer->platform_is_nt = (W32::GetVersion () < 0x80000000);
 
     extent = &data->extents;
 
     /* calculate offsets */
-    renderer->xoff = - data->extents.left;
-    renderer->yoff = - data->extents.top;
-    renderer->scale = scale / 25.4; /* don't know why this is required ... */
+    if (!renderer->hPrintDC) {
+	renderer->xoff = - data->extents.left;
+	renderer->yoff = - data->extents.top;
+	renderer->scale = scale;
+    } else {
+        int  ppc = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALWIDTH) 
+	            / ( data->paper.lmargin + data->paper.width + data->paper.rmargin);
+	/* respect margins */
+	renderer->margins.left   = ppc * data->paper.lmargin - W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALOFFSETX);
+	renderer->margins.top    = ppc * data->paper.tmargin - W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALOFFSETY);
+	renderer->margins.right  = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALWIDTH) - ppc * data->paper.rmargin;
+	renderer->margins.bottom = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALHEIGHT) - ppc * data->paper.bmargin;
 
+	renderer->xoff = - data->extents.left;
+	renderer->yoff = - data->extents.top;
+	renderer->scale = scale;
+    }
     /* initialize placeable header */
     /* bounding box in twips 1/1440 of an inch */
     renderer->pmh.Key = 0x9AC6CDD7;
     renderer->pmh.Handle = 0;
     renderer->pmh.Left   = 0;
     renderer->pmh.Top    = 0;
-    renderer->pmh.Right  = (gint16)(SC(data->extents.right - data->extents.left) * 25.4);
-    renderer->pmh.Bottom = (gint16)(SC(data->extents.bottom - data->extents.top) * 25.4);
+    renderer->pmh.Right  = (gint16)SC(data->extents.right - data->extents.left);
+    renderer->pmh.Bottom = (gint16)SC(data->extents.bottom - data->extents.top);
     renderer->pmh.Inch   = 1440 * 10;
     renderer->pmh.Reserved = 0;
 
@@ -1178,8 +1243,8 @@ export_data(DiagramData *data, const gchar *filename,
     /* bounding box in device units */
     bbox.left = 0;
     bbox.top = 0;
-    bbox.right = (int)(SC(data->extents.right - data->extents.left) * 25.4);
-    bbox.bottom = (int)(SC(data->extents.bottom - data->extents.top) * 25.4);
+    bbox.right = (int)SC(data->extents.right - data->extents.left);
+    bbox.bottom = (int)SC(data->extents.bottom - data->extents.top);
 
     /* initialize drawing */
     W32::SetBkMode(renderer->hFileDC, TRANSPARENT);
@@ -1188,8 +1253,6 @@ export_data(DiagramData *data, const gchar *filename,
                            bbox.left, bbox.top,
                            bbox.right, bbox.bottom);
 
-    renderer->scale *= 0.95; /* just a little smaller ... */
-
     /* write extents */
     DIAG_NOTE(renderer, "export_data extents %f,%f -> %f,%f\n", 
               extent->left, extent->top, extent->right, extent->bottom);
@@ -1197,15 +1260,26 @@ export_data(DiagramData *data, const gchar *filename,
     data_render(data, DIA_RENDERER(renderer), NULL, NULL, NULL);
 
     g_object_unref(renderer);
+    
+    W32::ReleaseDC (NULL, refDC);
 }
 
-static const gchar *extensions[] = { "wmf", NULL };
-static DiaExportFilter my_export_filter = {
+static const gchar *wmf_extensions[] = { "wmf", NULL };
+static DiaExportFilter wmf_export_filter = {
     N_("Windows Meta File"),
-    extensions,
+    wmf_extensions,
     export_data,
     NULL, /* user data */
-    "wmf::native"
+    "wmf"
+};
+
+static const gchar *emf_extensions[] = { "emf", NULL };
+static DiaExportFilter emf_export_filter = {
+    N_("Enhanced Meta File"),
+    emf_extensions,
+    export_data,
+    (void*)1, /* user data: urgh, reused as bool and pointer */
+    "emf"
 };
 
 
@@ -1227,7 +1301,8 @@ dia_plugin_init(PluginInfo *info)
      * useful at compile/develoment time. The output is broken
      * when processed by wmf_gdi.cpp ...
      */
-    filter_register_export(&my_export_filter);
+    filter_register_export(&wmf_export_filter);
+    filter_register_export(&emf_export_filter);
 #endif
 
     return DIA_PLUGIN_INIT_OK;
