@@ -441,11 +441,17 @@ create_standard_group(GList *items, DiagramData *dia) {
 static Color
 fig_color(int color_index) 
 {
-    if (color_index == -1) 
+    if (color_index <= -1) 
         return color_black; /* Default color */
-    if (color_index < FIG_MAX_DEFAULT_COLORS) 
+    else if (color_index < FIG_MAX_DEFAULT_COLORS) 
         return fig_default_colors[color_index];
-    else return fig_colors[color_index-FIG_MAX_DEFAULT_COLORS];
+    else if (color_index < FIG_MAX_USER_COLORS) 
+	return fig_colors[color_index-FIG_MAX_DEFAULT_COLORS];
+    else {
+	message_error(_("Color index %d too high, only 512 colors allowed. Using black instead."),
+		      color_index);
+	return color_black;
+    }
 }
 
 static Color
@@ -563,23 +569,25 @@ fig_simple_properties(DiaObject *obj,
 static int
 fig_read_n_points(FILE *file, int n, Point **points) {
     int i;
-    Point *new_points;
-
-    new_points = (Point*)g_malloc(sizeof(Point)*n);
+    GArray *points_list = g_array_sized_new(FALSE, FALSE, sizeof(Point), n);
 
     for (i = 0; i < n; i++) {
 	int x,y;
+	Point p;
 	if (fscanf(file, " %d %d ", &x, &y) != 2) {
 	    message_error(_("Error while reading %dth of %d points: %s\n"),
 			  i, n, strerror(errno));
-	    free(new_points);
+	    g_array_free(points_list, TRUE);
 	    return FALSE;
 	}
-	new_points[i].x = x/FIG_UNIT;
-	new_points[i].y = y/FIG_UNIT;
+	p.x = x/FIG_UNIT;
+	p.y = y/FIG_UNIT;
+	g_array_append_val(points_list, p);
     }
     fscanf(file, "\n");
-    *points = new_points;
+    
+    *points = (Point *)points_list->data;
+    g_array_free(points_list, FALSE);
     return TRUE;
 }
 
@@ -588,13 +596,18 @@ fig_read_arrow(FILE *file) {
     int arrow_type, style;
     real thickness, width, height;
     Arrow *arrow;
+    char* old_locale;
+
+    old_locale = setlocale(LC_NUMERIC, "C");
 
     if (fscanf(file, "%d %d %lf %lf %lf\n",
 	       &arrow_type, &style, &thickness,
 	       &width, &height) != 5) {
 	message_error(_("Error while reading arrowhead\n"));
+	setlocale(LC_NUMERIC,old_locale);
 	return NULL;
     }
+    setlocale(LC_NUMERIC,old_locale);
 
     arrow = g_new(Arrow, 1);
 
@@ -683,7 +696,7 @@ fig_read_text_line(FILE *file) {
     return text_buf;
 }
 
-static GList *depths[1000];
+static GList *depths[FIG_MAX_DEPTHS];
 
 /* If there's something in the compound stack, we ignore the depth field,
    as it will be determined by the group anyway */
@@ -693,6 +706,26 @@ static GSList *compound_stack = NULL;
    level.  Best we can do now. */
 static int compound_depth;
 
+/** Add an object at a given depth.  This function checks for depth limits
+ * and updates the compound depth if needed.
+ *
+ * @param newobj An object to add.  If we're inside a compound, this
+ * doesn't really add the object.
+ * @param depth A depth as in the Fig format, max 999
+ */
+static void
+add_at_depth(DiaObject *newobj, int depth) {
+    if (depth < 0 || depth >= FIG_MAX_DEPTHS) {
+	message_error(_("Depth %d of of range, only 0-%d allowed.\n"),
+		      depth, FIG_MAX_DEPTHS-1);
+	depth = FIG_MAX_DEPTHS - 1;
+    }
+    if (compound_stack == NULL) 
+	depths[depth] = g_list_append(depths[depth], newobj);
+    else 
+	if (compound_depth > depth) compound_depth = depth;
+}
+
 static DiaObject *
 fig_read_ellipse(FILE *file, DiagramData *dia) {
     int sub_type;
@@ -711,7 +744,9 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
     int start_x, start_y;
     int end_x, end_y;
     DiaObject *newobj = NULL;
+    char* old_locale;
 
+    old_locale = setlocale(LC_NUMERIC, "C");
     if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %lf %d %d %d %d %d %d %d %d\n",
 	       &sub_type,
 	       &line_style,
@@ -729,8 +764,10 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
 	       &start_x, &start_y,
 	       &end_x, &end_y) < 19) {
 	message_error(_("Couldn't read ellipse info: %s\n"), strerror(errno));
+	setlocale(LC_NUMERIC, old_locale);
 	return NULL;
     }
+    setlocale(LC_NUMERIC, old_locale);
     
     /* Curiously, the sub_type doesn't matter, as all info can be
        extracted this way */
@@ -749,10 +786,7 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
     /* Angle -- can't rotate yet */
 
     /* Depth field */
-    if (compound_stack == NULL)
-	depths[depth] = g_list_append(depths[depth], newobj);
-    else
-	if (compound_depth > depth) compound_depth = depth;
+    add_at_depth(newobj, depth);
 
     return newobj;
 }
@@ -779,7 +813,9 @@ fig_read_polyline(FILE *file, DiagramData *dia) {
     DiaObject *newobj = NULL;
     int flipped = 0;
     char *image_file = NULL;
+    char* old_locale;
 
+    old_locale = setlocale(LC_NUMERIC, "C");
     if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %d %d %d %d %d\n",
 	       &sub_type,
 	       &line_style,
@@ -885,11 +921,9 @@ fig_read_polyline(FILE *file, DiagramData *dia) {
     /* Cap style */
      
     /* Depth field */
-    if (compound_stack == NULL)
-	depths[depth] = g_list_append(depths[depth], newobj);
-    else
-	if (compound_depth > depth) compound_depth = depth;
+    add_at_depth(newobj, depth);
  exit:
+    setlocale(LC_NUMERIC, old_locale);
     prop_list_free(props);
     g_free(forward_arrow_info);
     g_free(backward_arrow_info);
@@ -1008,7 +1042,9 @@ fig_read_spline(FILE *file, DiagramData *dia) {
     DiaObject *newobj = NULL;
     BezPoint *bezpoints;
     int i;
+    char* old_locale;
 
+    old_locale = setlocale(LC_NUMERIC, "C");
     if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %d %d %d\n",
 	       &sub_type,
 	       &line_style,
@@ -1111,11 +1147,9 @@ fig_read_spline(FILE *file, DiagramData *dia) {
     /* Cap style */
      
     /* Depth field */
-    if (compound_stack == NULL)
-	depths[depth] = g_list_append(depths[depth], newobj);
-    else
-	if (compound_depth > depth) compound_depth = depth;
+    add_at_depth(newobj, depth);
  exit:
+    setlocale(LC_NUMERIC, old_locale);
     prop_list_free(props);
     g_free(forward_arrow_info);
     g_free(backward_arrow_info);
@@ -1144,7 +1178,9 @@ fig_read_arc(FILE *file, DiagramData *dia) {
     int x2, y2;
     int x3, y3;
     real radius;
+    char* old_locale;
 
+    old_locale = setlocale(LC_NUMERIC, "C");
     if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %d %d %d %lf %lf %d %d %d %d %d %d\n",
 	       &sub_type,
 	       &line_style,
@@ -1202,12 +1238,10 @@ fig_read_arc(FILE *file, DiagramData *dia) {
     /* Cap style */
      
     /* Depth field */
-    if (compound_stack == NULL)
-	depths[depth] = g_list_append(depths[depth], newobj);
-    else
-	if (compound_depth > depth) compound_depth = depth;
+    add_at_depth(newobj, depth);
 
  exit:
+    setlocale(LC_NUMERIC, old_locale);
     g_free(forward_arrow_info);
     g_free(backward_arrow_info);
     return newobj;
@@ -1239,7 +1273,9 @@ fig_read_text(FILE *file, DiagramData *dia) {
     real length;
     int x, y;
     char *text_buf = NULL;
+    char* old_locale;
 
+    old_locale = setlocale(LC_NUMERIC, "C");
     if (fscanf(file, " %d %d %d %d %d %lf %lf %d %lf %lf %d %d",
 	       &sub_type,
 	       &color,
@@ -1254,12 +1290,13 @@ fig_read_text(FILE *file, DiagramData *dia) {
 	       &x,
 	       &y) != 12) {
 	message_error(_("Couldn't read text info: %s\n"), strerror(errno));
+	setlocale(LC_NUMERIC, old_locale);
 	return NULL;
     }
     /* Skip one space exactly */
     text_buf = fig_read_text_line(file);
 
-    newobj = create_standard_text(x, y, dia);
+    newobj = create_standard_text(x/FIG_UNIT, y/FIG_UNIT, dia);
     if (newobj == NULL) goto exit;
 
     props = prop_list_from_descs(xfig_text_descs,pdtpp_true);
@@ -1298,13 +1335,11 @@ fig_read_text(FILE *file, DiagramData *dia) {
     newobj->ops->set_props(newobj, props);
     
     /* Depth field */
-    if (compound_stack == NULL)
-	depths[depth] = g_list_append(depths[depth], newobj);
-    else
-	if (compound_depth > depth) compound_depth = depth;
+    add_at_depth(newobj, depth);
 
  exit:
-    if (text_buf != NULL) free(text_buf);
+    setlocale(LC_NUMERIC, old_locale);
+    if (text_buf != NULL) g_free(text_buf);
     if (props != NULL) prop_list_free(props);
     return newobj;
 }
@@ -1347,6 +1382,12 @@ fig_read_object(FILE *file, DiagramData *dia) {
 	    return FALSE;
 	}
 
+	if (colornumber < 32 || colornumber > FIG_MAX_USER_COLORS) {
+	    message_error(_("Color number %d out of range 0..%d.  Discarding color.\n"),
+			  colornumber, FIG_MAX_USER_COLORS);
+	    return FALSE;
+	}
+
 	color.red = ((colorvalues & 0x00ff0000)>>16) / 255.0;
 	color.green = ((colorvalues & 0x0000ff00)>>8) / 255.0;
 	color.blue = (colorvalues & 0x000000ff) / 255.0;
@@ -1393,7 +1434,7 @@ fig_read_object(FILE *file, DiagramData *dia) {
 	}
 	/* Group extends don't really matter */
 	if (compound_stack == NULL)
-	    compound_depth = 999;
+	    compound_depth = FIG_MAX_DEPTHS - 1;
 	compound_stack = g_slist_append(compound_stack, NULL);
 	return TRUE;
 	break;
@@ -1488,12 +1529,16 @@ fig_read_meta_data(FILE *file, DiagramData *dia) {
 
     {
 	real mag;
+	char* old_locale;
 
+	old_locale = setlocale(LC_NUMERIC, "C");
 	if (fscanf(file, "%lf\n", &mag) != 1) {
 	    message_error(_("Error reading magnification: %s\n"), strerror(errno));
+	    setlocale(LC_NUMERIC, old_locale);
 	    return FALSE;
 	}
-    
+        setlocale(LC_NUMERIC, old_locale);
+
 	dia->paper.scaling = mag/100;
     }
 
@@ -1523,7 +1568,7 @@ fig_read_meta_data(FILE *file, DiagramData *dia) {
 	if (!feof(file)) {
 	    message_error(_("Error reading FIG file: %s\n"), strerror(errno));
 	} else {
-	    message_error(_("Premature end of FIG file\n"), strerror(errno));
+	    message_error(_("Premature end of FIG file\n"));
 	}
 	return FALSE;
     }
@@ -1551,13 +1596,14 @@ import_fig(const gchar *filename, DiagramData *dia, void* user_data) {
     for (i = 0; i < FIG_MAX_USER_COLORS; i++) {
 	fig_colors[i] = color_black;
     }
-    for (i = 0; i < 1000; i++) {
+    for (i = 0; i < FIG_MAX_DEPTHS; i++) {
 	depths[i] = NULL;
     }
 
     figfile = fopen(filename,"r");
     if(figfile == NULL){
-	message_error(_("Couldn't open: '%s' for reading.\n"), filename);
+	message_error(_("Couldn't open: '%s' for reading.\n"), 
+		      dia_message_filename(filename));
 	return FALSE;
     }
   
@@ -1578,7 +1624,7 @@ import_fig(const gchar *filename, DiagramData *dia, void* user_data) {
 	if (!feof(figfile)) {
 	    message_error(_("Error reading FIG file: %s\n"), strerror(errno));
 	} else {
-	    message_error(_("Premature end of FIG file\n"), strerror(errno));
+	    message_error(_("Premature end of FIG file\n"));
 	}
 	fclose(figfile);
 	return FALSE;
@@ -1606,7 +1652,7 @@ import_fig(const gchar *filename, DiagramData *dia, void* user_data) {
     } while (TRUE);
 
     /* Now we can reorder for the depth fields */
-    for (i = 0; i < 1000; i++) {
+    for (i = 0; i < FIG_MAX_DEPTHS; i++) {
 	if (depths[i] != NULL)
 	    layer_add_objects_first(dia->active_layer, depths[i]);
     }
