@@ -40,15 +40,13 @@
 
 
 /* Possible stylesheets */
-fromxsl_t *froms;
+fromxsl_t *froms = NULL;
 
 /* Selected stylesheets */
 toxsl_t *xsl_to;
 fromxsl_t *xsl_from;
 
 
-void xslt_ok(void);
-
 static char *diafilename;
 static char *filename;
 
@@ -70,7 +68,7 @@ export_xslt(DiagramData *data, const gchar *f,
 
 
 void
-xslt_ok() 
+xslt_ok(void) 
 {
 	FILE *file, *out;
 	int err;
@@ -84,14 +82,16 @@ xslt_ok()
 	file = fopen(diafilename, "r");
 
 	if (file == NULL) {
-	    message_error(_("Couldn't open: '%s' for reading.\n"), diafilename);
+	    message_error(_("Couldn't open: '%s' for reading.\n"), 
+			  dia_message_filename(diafilename));
 	    return;
 	}
 
 	out = fopen(filename, "w+");
 	
 	if (out == NULL) {
-	  message_error(_("Can't open output file %s: %s\n"), filename, strerror(errno));
+	  message_error(_("Can't open output file %s: %s\n"), 
+			dia_message_filename(filename), strerror(errno));
 	  return;
 	}
 	
@@ -99,7 +99,8 @@ xslt_ok()
 	doc = xmlParseFile(diafilename);
 
 	if(doc == NULL) {
-		message_error(_("Error while parsing %s\n"), diafilename);
+		message_error(_("Error while parsing %s\n"), 
+			      dia_message_filename(diafilename));
 		return;
 	}
 	
@@ -107,13 +108,15 @@ xslt_ok()
 
 	style = xsltParseStylesheetFile((const xmlChar *) stylefname);
 	if(style == NULL) {
-		message_error(_("Error while parsing stylesheet %s\n"), stylefname);
+		message_error(_("Error while parsing stylesheet %s\n"), 
+			      dia_message_filename(stylefname));
 		return;
 	}
 	
 	res = xsltApplyStylesheet(style, doc, NULL);
 	if(res == NULL) {
-		message_error(_("Error while applying stylesheet %s\n"), stylefname);
+		message_error(_("Error while applying stylesheet %s\n"), 
+			      dia_message_filename(stylefname));
 		return;
 	}       
 
@@ -121,7 +124,8 @@ xslt_ok()
 
 	codestyle = xsltParseStylesheetFile((const xmlChar *) stylefname);
 	if(codestyle == NULL) {
-	    message_error(_("Error while parsing stylesheet: %s\n"), stylefname);
+	    message_error(_("Error while parsing stylesheet: %s\n"), 
+			  dia_message_filename(stylefname));
 	    return;
 	}
 	
@@ -129,7 +133,8 @@ xslt_ok()
 	
 	doc = xsltApplyStylesheet(codestyle, res, (const char **) params);
 	if(doc == NULL) {
-		message_error(_("Error while applying stylesheet: %s\n"), stylefname);
+		message_error(_("Error while applying stylesheet: %s\n"), 
+			      dia_message_filename(stylefname));
 		return;
 	}
 
@@ -242,6 +247,10 @@ static PluginInitResult read_configuration(const char *config)
 		if (froms == NULL)
 		    froms = cur_from = new_from;
 		else {
+		    if (!cur_from)
+		        cur_from = froms;
+		    while (cur_from->next)
+		        cur_from = cur_from->next;
 		    cur_from->next = new_from;
 		    cur_from = new_from;
 		}
